#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "amsr2latlon.h"

#define MV -9999.0 // missing value

void amsr2latlon_(int *num, double *prm1, double *prm2
, float lathi[][SMPH]
, float lonhi[][SMPH]
, float latlo[][SMPL]
, float lonlo[][SMPL]
){
	// working variable
	int i, j, k;
	double  pi, rad, deg;
	float lat1, lat2, lon1, lon2;
	double p1[3];
	double p2[3];
	double pt[3];
	double ex[3];
	double ey[3];
	double ez[3];
	double temp, theta;
	double J, K, L, M;

	// parameter
	pi = acos(-1.0);
	rad = pi / 180.0;
	deg = 180.0 / pi;

	// pixel loop
	for(j = 0; j < *num; ++j){
	for(i = 0; i < SMPL; ++i){
		// set short cut
		lat1 = lathi[j][i*2+0];
		lat2 = lathi[j][i*2+1];
		lon1 = lonhi[j][i*2+0];
		lon2 = lonhi[j][i*2+1];

		// check input range
		if(lat1 <  -90.0 ||  +90.0 < lat1
		|| lon1 < -180.0 || +180.0 < lon1
		|| lat2 <  -90.0 ||  +90.0 < lat2
		|| lon2 < -180.0 || +180.0 < lon2){
			printf("amsr2latlon: out of range warning: "
			"latlon1(%f,%f)->latlon2(%f,%f) "
			"(low_pixel_by_1origin=%d,scan_by_1origin=%d)\n"
			, lat1, lon1, lat2, lon2, i+1, j+1);
			latlo[j][i] = MV;
			lonlo[j][i] = MV;
			continue;
		}

		// calculate
		p1[0] = cos(lon1*rad)*cos(lat1*rad);
		p1[1] = sin(lon1*rad)*cos(lat1*rad);
		p1[2] = sin(lat1*rad);
		p2[0] = cos(lon2*rad)*cos(lat2*rad);
		p2[1] = sin(lon2*rad)*cos(lat2*rad);
		p2[2] = sin(lat2*rad);
		temp = p1[0]*p2[0]+p1[1]*p2[1]+p1[2]*p2[2];
		theta = acos(temp);
		for(k = 0; k < 3; ++k) ex[k]=p1[k];
		temp = sqrt(p1[0]*p1[0]+p1[1]*p1[1]+p1[2]*p1[2])*
		sqrt(p2[0]*p2[0]+p2[1]*p2[1]+p2[2]*p2[2])*sin(theta);
		ez[0] = (p1[1]*p2[2]-p1[2]*p2[1])/temp;
		ez[1] = (p1[2]*p2[0]-p1[0]*p2[2])/temp;
		ez[2] = (p1[0]*p2[1]-p1[1]*p2[0])/temp;
		ey[0] = ez[1]*ex[2]-ez[2]*ex[1];
		ey[1] = ez[2]*ex[0]-ez[0]*ex[2];
		ey[2] = ez[0]*ex[1]-ez[1]*ex[0];
		J = cos((*prm2)*theta);
		K = cos((*prm1)*theta);
		L = sin((*prm1)*theta);
		M = sin((*prm2)*theta);
		pt[0] = J*(K*ex[0]+L*ey[0])+M*ez[0];
		pt[1] = J*(K*ex[1]+L*ey[1])+M*ez[1];
		pt[2] = J*(K*ex[2]+L*ey[2])+M*ez[2];
		temp = sqrt(pt[0]*pt[0] + pt[1]*pt[1]);
		lonlo[j][i] = atan2(pt[1],pt[0])*deg;
		latlo[j][i] = atan2(pt[2],temp )*deg;
	}
	}
}
