#include <stdio.h>
#include <stdlib.h>
#include "hdf5.h"
#include "amsr2time.h"
#include "amsr2latlon.h"

// fixed value
#define LMT 2200 // limit of NumberOfScans
#define AM2_DEF_SNUM_HI 486 // high resolution pixel width
#define AM2_DEF_SNUM_LO 243 // low  resolution pixel width

int main(int argc, char *argv[]){
  // interface variable
  int i,j;          // loop variable
  herr_t ret;       // return status
  char *buf = NULL; // text buffer
  char *fn  = NULL; // filename
  hid_t fhnd;       // file handle
  hid_t ahnd;       // attribute handle
  hid_t atyp;       // attribute type
  hid_t dhnd;       // dataset handle
  hid_t shnd;       // dataspace handle

  // meta data
  char *geo = NULL; // GeophysicalName
  char *gid = NULL; // GranuleID
  char *tm1 = NULL; // ObservationStartDateTime
  char *tm2 = NULL; // EquatorCrossingDateTime
  char *tm3 = NULL; // ObservationEndDateTime
  int num;          // NumberOfScans
  int ovr;          // OverlapScans
  double prm1[7];   // CoRegistrationParameterA1 -> 6G,7G,10G,18G,23G,36G,MEAN
  double prm2[7];   // CoRegistrationParameterA2 -> 6G,7G,10G,18G,23G,36G,MEAN

  // array buffer
  float sca; // scale
  double r8d1[LMT]; // array buffer for double D1
  unsigned char i1d2hi[LMT][AM2_DEF_SNUM_HI]; // array buffer for unsigned char D2 high

  // array data
  AM2_COMMON_SCANTIME st[LMT]; // scantime
  float lat89a[LMT][AM2_DEF_SNUM_HI]; // lat for 89a
  float lat89b[LMT][AM2_DEF_SNUM_HI]; // lat for 89b
  float latlm [LMT][AM2_DEF_SNUM_LO]; // lat for low mean
  float lat06 [LMT][AM2_DEF_SNUM_LO]; // lat for 06
  float lat07 [LMT][AM2_DEF_SNUM_LO]; // lat for 07
  float lat10 [LMT][AM2_DEF_SNUM_LO]; // lat for 10
  float lat18 [LMT][AM2_DEF_SNUM_LO]; // lat for 18
  float lat23 [LMT][AM2_DEF_SNUM_LO]; // lat for 23
  float lat36 [LMT][AM2_DEF_SNUM_LO]; // lat for 36
  float lon89a[LMT][AM2_DEF_SNUM_HI]; // lon for 89a
  float lon89b[LMT][AM2_DEF_SNUM_HI]; // lon for 89b
  float lonlm [LMT][AM2_DEF_SNUM_LO]; // lon for low mean
  float lon06 [LMT][AM2_DEF_SNUM_LO]; // lon for 06
  float lon07 [LMT][AM2_DEF_SNUM_LO]; // lon for 07
  float lon10 [LMT][AM2_DEF_SNUM_LO]; // lon for 10
  float lon18 [LMT][AM2_DEF_SNUM_LO]; // lon for 18
  float lon23 [LMT][AM2_DEF_SNUM_LO]; // lon for 23
  float lon36 [LMT][AM2_DEF_SNUM_LO]; // lon for 36
  float tb06h [LMT][AM2_DEF_SNUM_LO]; // tb for 06h
  float tb06v [LMT][AM2_DEF_SNUM_LO]; // tb for 06v
  float tb07h [LMT][AM2_DEF_SNUM_LO]; // tb for 07h
  float tb07v [LMT][AM2_DEF_SNUM_LO]; // tb for 07v
  float tb10h [LMT][AM2_DEF_SNUM_LO]; // tb for 10h
  float tb10v [LMT][AM2_DEF_SNUM_LO]; // tb for 10v
  float tb18h [LMT][AM2_DEF_SNUM_LO]; // tb for 18h
  float tb18v [LMT][AM2_DEF_SNUM_LO]; // tb for 18v
  float tb23h [LMT][AM2_DEF_SNUM_LO]; // tb for 23h
  float tb23v [LMT][AM2_DEF_SNUM_LO]; // tb for 23v
  float tb36h [LMT][AM2_DEF_SNUM_LO]; // tb for 36h
  float tb36v [LMT][AM2_DEF_SNUM_LO]; // tb for 36v
  float tb89ah[LMT][AM2_DEF_SNUM_HI]; // tb for 89ah
  float tb89av[LMT][AM2_DEF_SNUM_HI]; // tb for 89av
  float tb89bh[LMT][AM2_DEF_SNUM_HI]; // tb for 89bh
  float tb89bv[LMT][AM2_DEF_SNUM_HI]; // tb for 89bv
  unsigned char pdq06h [LMT][AM2_DEF_SNUM_LO]; // pixel data quality for 06h
  unsigned char pdq06v [LMT][AM2_DEF_SNUM_LO]; // pixel data quality for 06v
  unsigned char pdq07h [LMT][AM2_DEF_SNUM_LO]; // pixel data quality for 07h
  unsigned char pdq07v [LMT][AM2_DEF_SNUM_LO]; // pixel data quality for 07v
  unsigned char pdq89ah[LMT][AM2_DEF_SNUM_HI]; // pixel data quality for 89ah
  unsigned char pdq89av[LMT][AM2_DEF_SNUM_HI]; // pixel data quality for 89av
  unsigned char pdq89bh[LMT][AM2_DEF_SNUM_HI]; // pixel data quality for 89bh
  unsigned char pdq89bv[LMT][AM2_DEF_SNUM_HI]; // pixel data quality for 89bv
  unsigned char loflo [LMT*6][AM2_DEF_SNUM_LO]; // land ocean flag for low
  unsigned char lof06 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 06 
  unsigned char lof07 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 07 
  unsigned char lof10 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 10 
  unsigned char lof18 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 18 
  unsigned char lof23 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 23 
  unsigned char lof36 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 36 
  unsigned char lofhi [LMT*2][AM2_DEF_SNUM_HI]; // land ocean flag for high
  unsigned char lof89a[LMT  ][AM2_DEF_SNUM_HI]; // land ocean flag for 89a
  unsigned char lof89b[LMT  ][AM2_DEF_SNUM_HI]; // land ocean flag for 89b
  float ear_in[LMT][AM2_DEF_SNUM_LO]; // earth incidence
  float ear_az[LMT][AM2_DEF_SNUM_LO]; // earth azimuth

  // argument check
  if(argc < 2){
    printf("argument: AMSR2 L1B DATA FILE (HDF5)\n");
    exit(1);
  }

  // set filename
  fn = argv[1];
  printf("input file: %s\n", fn);

  // hdf5 initialize
  ret = H5open();
  if(ret < 0){
    printf("h5open error: %d\n" ,ret);
    exit(1);
  }

  // open
  fhnd = H5Fopen(fn, H5F_ACC_RDONLY, H5P_DEFAULT);
  if(fhnd < 0){
    printf("H5Fopen error: %s\n", fn);
    exit(1);
  }

  // read meta: GeophysicalName
  ahnd = H5Aopen(fhnd, "GeophysicalName", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &geo);
  if(ret < 0){
    printf("H5Aread error: GeophysicalName\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("GeophysicalName: %s\n", geo);

  // read meta: GranuleID
  ahnd = H5Aopen(fhnd, "GranuleID", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &gid);
  if(ret < 0){
    printf("H5Aread error: GranuleID\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("GranuleID: %s\n", gid);

  // read meta: ObservationStartDateTime
  ahnd = H5Aopen(fhnd, "ObservationStartDateTime", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &tm1);
  if(ret < 0){
    printf("H5Aread error: ObservationStartDateTime\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("ObservationStartDateTime: %s\n", tm1);

  // read meta: EquatorCrossingDateTime 
  ahnd = H5Aopen(fhnd, "EquatorCrossingDateTime", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &tm2);
  if(ret < 0){
    printf("H5Aread error: EquatorCrossingDateTime\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("EquatorCrossingDateTime: %s\n", tm2);

  // read meta: ObservationEndDateTime
  ahnd = H5Aopen(fhnd, "ObservationEndDateTime", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &tm3);
  if(ret < 0){
    printf("H5Aread error: ObservationEndDateTime\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("ObservationEndDateTime: %s\n", tm3);

  // read meta: NumberOfScans
  ahnd = H5Aopen(fhnd, "NumberOfScans", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &buf);
  if(ret < 0){
    printf("H5Aread error: NumberOfScans\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  num = atoi(buf);
  printf("NumberOfScans: %d\n", num);

  // check limit
  if(num > LMT){
    printf("limit error: limit of NumberOfScans = %d\n", LMT);
    exit(1);
  }
  else{
    printf("limit of NumberOfScans = %d\n", LMT);
  }

  // read meta: OverlapScans
  ahnd = H5Aopen(fhnd, "OverlapScans", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &buf);
  if(ret < 0){
    printf("H5Aread error: OverlapScans\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  ovr = atoi(buf);
  printf("OverlapScans: %d\n", ovr);

  // read meta: CoRegistrationParameterA1
  ahnd = H5Aopen(fhnd, "CoRegistrationParameterA1", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &buf);
  if(ret < 0){
    printf("H5Aread error: CoRegistrationParameterA1\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  sscanf(buf, "6G-%lf,7G-%lf,10G-%lf,18G-%lf,23G-%lf,36G-%lf"
  , &prm1[0]
  , &prm1[1]
  , &prm1[2]
  , &prm1[3]
  , &prm1[4]
  , &prm1[5]
  );
  prm1[6] = (prm1[0] + prm1[1] + prm1[2] + prm1[3] + prm1[4] + prm1[5]) / 6.0;
  printf("CoRegistrationParameterA1: "
  "6G-%8.5lf,7G-%8.5lf,10G-%8.5lf,18G-%8.5lf,23G-%8.5lf,36G-%8.5lf\n"
  , prm1[0]
  , prm1[1]
  , prm1[2]
  , prm1[3]
  , prm1[4]
  , prm1[5]
  );

  // read meta: CoRegistrationParameterA2
  ahnd = H5Aopen(fhnd, "CoRegistrationParameterA2", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &buf);
  if(ret < 0){
    printf("H5Aread error: CoRegistrationParameterA2\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  sscanf(buf, "6G-%lf,7G-%lf,10G-%lf,18G-%lf,23G-%lf,36G-%lf"
  , &prm2[0]
  , &prm2[1]
  , &prm2[2]
  , &prm2[3]
  , &prm2[4]
  , &prm2[5]
  );
  prm2[6] = (prm2[0] + prm2[1] + prm2[2] + prm2[3] + prm2[4] + prm2[5]) / 6.0;
  printf("CoRegistrationParameterA2: "
  "6G-%8.5lf,7G-%8.5lf,10G-%8.5lf,18G-%8.5lf,23G-%8.5lf,36G-%8.5lf\n"
  , prm2[0]
  , prm2[1]
  , prm2[2]
  , prm2[3]
  , prm2[4]
  , prm2[5]
  );

  // read array: scantime
  dhnd = H5Dopen(fhnd, "Scan Time", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, r8d1);
  if(ret < 0){
    printf("H5Dread error: Scan Time\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap
  for(j = 0; j < num; ++j){
    r8d1[j] = r8d1[j + ovr];
  }
  for(j = num; j < LMT; ++j){
    r8d1[j] = 0;
  }
  // convert
  amsr2time_(&num, r8d1, st);
  // sample display
  printf("time[scan=0]: %04d/%02d/%02d %02d:%02d:%02d\n"
  , st[0].year
  , st[0].month
  , st[0].day
  , st[0].hour
  , st[0].minute
  , st[0].second
  );

  // read array: latlon for 89a
  // read lat
  dhnd = H5Dopen(fhnd ,"Latitude of Observation Point for 89A", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lat89a);
  if(ret < 0){
    printf("H5Dread error: Latitude of Observation Point for 89A\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // read lon
  dhnd = H5Dopen(fhnd ,"Longitude of Observation Point for 89A", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lon89a);
  if(ret < 0){
    printf("H5Dread error: Longitude of Observation Point for 89A\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lat89a[j][i] = lat89a[j+ovr][i];
      lon89a[j][i] = lon89a[j+ovr][i];
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lat89a[j][i] = 0;
      lon89a[j][i] = 0;
    }
  }
  // sample display
  printf("latlon89a[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat89a[0][0], lon89a[0][0]);

  // read array: latlon for 89b
  // read lat
  dhnd = H5Dopen(fhnd ,"Latitude of Observation Point for 89B", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lat89b);
  if(ret < 0){
    printf("H5Dread error: Latitude of Observation Point for 89B\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // read lon
  dhnd = H5Dopen(fhnd ,"Longitude of Observation Point for 89B", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lon89b);
  if(ret < 0){
    printf("H5Dread error: Longitude of Observation Point for 89B\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lat89b[j][i] = lat89b[j+ovr][i];
      lon89b[j][i] = lon89b[j+ovr][i];
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lat89b[j][i] = 0;
      lon89b[j][i] = 0;
    }
  }
  // sample display
  printf("latlon89b[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat89b[0][0], lon89b[0][0]);

  // read array: latlon for low mean 
  amsr2latlon_(&num, &prm1[6], &prm2[6], lat89a, lon89a, latlm, lonlm);
  printf("latlonlm[scan=0][pixel=0]: (%9.4f,%9.4f)\n", latlm[0][0], lonlm[0][0]);

  // read array: latlon for 06
  amsr2latlon_(&num, &prm1[0], &prm2[0], lat89a, lon89a, lat06, lon06);
  printf("latlon06[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat06[0][0], lon06[0][0]);

  // read array: latlon for 07
  amsr2latlon_(&num, &prm1[1], &prm2[1], lat89a, lon89a, lat07, lon07);
  printf("latlon07[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat07[0][0], lon07[0][0]);

  // read array: latlon for 10
  amsr2latlon_(&num, &prm1[2], &prm2[2], lat89a, lon89a, lat10, lon10);
  printf("latlon10[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat10[0][0], lon10[0][0]);

  // read array: latlon for 18
  amsr2latlon_(&num, &prm1[3], &prm2[3], lat89a, lon89a, lat18, lon18);
  printf("latlon18[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat18[0][0], lon18[0][0]);

  // read array: latlon for 23
  amsr2latlon_(&num, &prm1[4], &prm2[4], lat89a, lon89a, lat23, lon23);
  printf("latlon23[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat23[0][0], lon23[0][0]);

  // read array: latlon for 36
  amsr2latlon_(&num, &prm1[5], &prm2[5], lat89a, lon89a, lat36, lon36);
  printf("latlon36[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat36[0][0], lon36[0][0]);

  // read array: tb for 06h
  dhnd = H5Dopen(fhnd, "Brightness Temperature (6.9GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb06h);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (6.9GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb06h[j][i] = tb06h[j+ovr][i];
      if(tb06h[j][i] < 65534) tb06h[j][i] = tb06h[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb06h[j][i] = 0;
    }
  }
  // sample display
  printf("tb06h[scan=0][pixel=0]: %9.2f\n", tb06h[0][0]);

  // read array: tb for 06v
  dhnd = H5Dopen(fhnd, "Brightness Temperature (6.9GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb06v);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (6.9GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb06v[j][i] = tb06v[j+ovr][i];
      if(tb06v[j][i] < 65534) tb06v[j][i] = tb06v[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb06v[j][i] = 0;
    }
  }
  // sample display
  printf("tb06v[scan=0][pixel=0]: %9.2f\n", tb06v[0][0]);

  // read array: tb for 07h
  dhnd = H5Dopen(fhnd, "Brightness Temperature (7.3GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb07h);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (7.3GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb07h[j][i] = tb07h[j+ovr][i];
      if(tb07h[j][i] < 65534) tb07h[j][i] = tb07h[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb07h[j][i] = 0;
    }
  }
  // sample display
  printf("tb07h[scan=0][pixel=0]: %9.2f\n", tb07h[0][0]);

  // read array: tb for 07v
  dhnd = H5Dopen(fhnd, "Brightness Temperature (7.3GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb07v);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (7.3GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb07v[j][i] = tb07v[j+ovr][i];
      if(tb07v[j][i] < 65534) tb07v[j][i] = tb07v[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb07v[j][i] = 0;
    }
  }
  // sample display
  printf("tb07v[scan=0][pixel=0]: %9.2f\n", tb07v[0][0]);

  // read array: tb for 10h
  dhnd = H5Dopen(fhnd, "Brightness Temperature (10.7GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb10h);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (10.7GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb10h[j][i] = tb10h[j+ovr][i];
      if(tb10h[j][i] < 65534) tb10h[j][i] = tb10h[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb10h[j][i] = 0;
    }
  }
  // sample display
  printf("tb10h[scan=0][pixel=0]: %9.2f\n", tb10h[0][0]);

  // read array: tb for 10v
  dhnd = H5Dopen(fhnd, "Brightness Temperature (10.7GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb10v);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (10.7GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb10v[j][i] = tb10v[j+ovr][i];
      if(tb10v[j][i] < 65534) tb10v[j][i] = tb10v[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb10v[j][i] = 0;
    }
  }
  // sample display
  printf("tb10v[scan=0][pixel=0]: %9.2f\n", tb10v[0][0]);

  // read array: tb for 18h
  dhnd = H5Dopen(fhnd, "Brightness Temperature (18.7GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb18h);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (18.7GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb18h[j][i] = tb18h[j+ovr][i];
      if(tb18h[j][i] < 65534) tb18h[j][i] = tb18h[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb18h[j][i] = 0;
    }
  }
  // sample display
  printf("tb18h[scan=0][pixel=0]: %9.2f\n", tb18h[0][0]);

  // read array: tb for 18v
  dhnd = H5Dopen(fhnd, "Brightness Temperature (18.7GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb18v);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (18.7GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb18v[j][i] = tb18v[j+ovr][i];
      if(tb18v[j][i] < 65534) tb18v[j][i] = tb18v[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb18v[j][i] = 0;
    }
  }
  // sample display
  printf("tb18v[scan=0][pixel=0]: %9.2f\n", tb18v[0][0]);

  // read array: tb for 23h
  dhnd = H5Dopen(fhnd, "Brightness Temperature (23.8GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb23h);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (23.8GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb23h[j][i] = tb23h[j+ovr][i];
      if(tb23h[j][i] < 65534) tb23h[j][i] = tb23h[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb23h[j][i] = 0;
    }
  }
  // sample display
  printf("tb23h[scan=0][pixel=0]: %9.2f\n", tb23h[0][0]);

  // read array: tb for 23v
  dhnd = H5Dopen(fhnd, "Brightness Temperature (23.8GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb23v);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (23.8GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb23v[j][i] = tb23v[j+ovr][i];
      if(tb23v[j][i] < 65534) tb23v[j][i] = tb23v[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb23v[j][i] = 0;
    }
  }
  // sample display
  printf("tb23v[scan=0][pixel=0]: %9.2f\n", tb23v[0][0]);

  // read array: tb for 36h
  dhnd = H5Dopen(fhnd, "Brightness Temperature (36.5GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb36h);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (36.5GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb36h[j][i] = tb36h[j+ovr][i];
      if(tb36h[j][i] < 65534) tb36h[j][i] = tb36h[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb36h[j][i] = 0;
    }
  }
  // sample display
  printf("tb36h[scan=0][pixel=0]: %9.2f\n", tb36h[0][0]);

  // read array: tb for 36v
  dhnd = H5Dopen(fhnd, "Brightness Temperature (36.5GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb36v);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (36.5GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb36v[j][i] = tb36v[j+ovr][i];
      if(tb36v[j][i] < 65534) tb36v[j][i] = tb36v[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb36v[j][i] = 0;
    }
  }
  // sample display
  printf("tb36v[scan=0][pixel=0]: %9.2f\n", tb36v[0][0]);

  // read array: tb for 89ah
  dhnd = H5Dopen(fhnd, "Brightness Temperature (89.0GHz-A,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb89ah);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (89.0GHz-A,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89ah[j][i] = tb89ah[j+ovr][i];
      if(tb89ah[j][i] < 65534) tb89ah[j][i] = tb89ah[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89ah[j][i] = 0;
    }
  }
  // sample display
  printf("tb89ah[scan=0][pixel=0]: %9.2f\n", tb89ah[0][0]);

  // read array: tb for 89av
  dhnd = H5Dopen(fhnd, "Brightness Temperature (89.0GHz-A,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb89av);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (89.0GHz-A,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89av[j][i] = tb89av[j+ovr][i];
      if(tb89av[j][i] < 65534) tb89av[j][i] = tb89av[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89av[j][i] = 0;
    }
  }
  // sample display
  printf("tb89av[scan=0][pixel=0]: %9.2f\n", tb89av[0][0]);

  // read array: tb for 89bh
  dhnd = H5Dopen(fhnd, "Brightness Temperature (89.0GHz-B,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb89bh);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (89.0GHz-B,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89bh[j][i] = tb89bh[j+ovr][i];
      if(tb89bh[j][i] < 65534) tb89bh[j][i] = tb89bh[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89bh[j][i] = 0;
    }
  }
  // sample display
  printf("tb89bh[scan=0][pixel=0]: %9.2f\n", tb89bh[0][0]);

  // read array: tb for 89bv
  dhnd = H5Dopen(fhnd, "Brightness Temperature (89.0GHz-B,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb89bv);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (89.0GHz-B,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89bv[j][i] = tb89bv[j+ovr][i];
      if(tb89bv[j][i] < 65534) tb89bv[j][i] = tb89bv[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89bv[j][i] = 0;
    }
  }
  // sample display
  printf("tb89bv[scan=0][pixel=0]: %9.2f\n", tb89bv[0][0]);

  // read array: pixel data quality for low
  dhnd = H5Dopen(fhnd, "Pixel Data Quality 6 to 36", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, i1d2hi);
  if(ret < 0){
    printf("H5Dread error: Pixel Data Quality 6 to 36\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & separate
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      pdq06v[j][i]=0;
      if((i1d2hi[j+ovr][i*2+0] &   1) != 0) pdq06v[j][i]=1;
      if((i1d2hi[j+ovr][i*2+0] &   2) != 0) pdq06v[j][i]=10;
      if(((i1d2hi[j+ovr][i*2+0] &   1) != 0) && ((i1d2hi[j+ovr][i*2+0] &   2) != 0)) pdq06v[j][i]=11;
      pdq06h[j][i]=0;
      if((i1d2hi[j+ovr][i*2+0] &   4) != 0) pdq06h[j][i]=1;
      if((i1d2hi[j+ovr][i*2+0] &   8) != 0) pdq06h[j][i]=10;
      if(((i1d2hi[j+ovr][i*2+0] &   4) != 0) && ((i1d2hi[j+ovr][i*2+0] &   8) != 0)) pdq06h[j][i]=11;
      pdq07v[j][i]=0;
      if((i1d2hi[j+ovr][i*2+0] &  16) != 0) pdq07v[j][i]=1;
      if((i1d2hi[j+ovr][i*2+0] &  32) != 0) pdq07v[j][i]=10;
      if(((i1d2hi[j+ovr][i*2+0] &  16) != 0) && ((i1d2hi[j+ovr][i*2+0] &  32) != 0)) pdq07v[j][i]=11;
      pdq07h[j][i]=0;
      if((i1d2hi[j+ovr][i*2+0] &  64) != 0) pdq07h[j][i]=1;
      if((i1d2hi[j+ovr][i*2+0] & 128) != 0) pdq07h[j][i]=10;
      if(((i1d2hi[j+ovr][i*2+0] &  64) != 0) && ((i1d2hi[j+ovr][i*2+0] & 128) != 0)) pdq07h[j][i]=11;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      pdq06h[j][i]=0;
      pdq06v[j][i]=0;
      pdq07h[j][i]=0;
      pdq07v[j][i]=0;
    }
  }
  // sample display
  printf("pdq06h[scan=0][pixel=0]): %3d\n", pdq06h[0][0]);
  printf("pdq06v[scan=0][pixel=0]): %3d\n", pdq06v[0][0]);
  printf("pdq07h[scan=0][pixel=0]): %3d\n", pdq07h[0][0]);
  printf("pdq07v[scan=0][pixel=0]): %3d\n", pdq07v[0][0]);

  // read array: pixel data quality for high
  dhnd = H5Dopen(fhnd, "Pixel Data Quality 89", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, i1d2hi);
  if(ret < 0){
    printf("H5Dread error: Pixel Data Quality 89\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & separate
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      pdq89ah[j][i]=0; if((i1d2hi[j+ovr][i] & 1) != 0) pdq89ah[j][i]=1;
      pdq89av[j][i]=0; if((i1d2hi[j+ovr][i] & 2) != 0) pdq89av[j][i]=1;
      pdq89ah[j][i]=0; if((i1d2hi[j+ovr][i] & 4) != 0) pdq89ah[j][i]=1;
      pdq89av[j][i]=0; if((i1d2hi[j+ovr][i] & 8) != 0) pdq89av[j][i]=1;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      pdq89ah[j][i]=0;
      pdq89av[j][i]=0;
      pdq89ah[j][i]=0;
      pdq89av[j][i]=0;
    }
  }
  // sample display
  printf("pdq89ah[scan=0][pixel=0]): %3d\n", pdq89ah[0][0]);
  printf("pdq89av[scan=0][pixel=0]): %3d\n", pdq89av[0][0]);
  printf("pdq89ah[scan=0][pixel=0]): %3d\n", pdq89ah[0][0]);
  printf("pdq89av[scan=0][pixel=0]): %3d\n", pdq89av[0][0]);

  // read array: land ocean flag for low
  dhnd = H5Dopen(fhnd, "Land_Ocean Flag 6 to 36", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, loflo);
  if(ret < 0){
    printf("H5Dread error: Land_Ocean Flag 6 to 36\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // separate
  for(j = 0; j < num+ovr*2; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      lof06[j][i]=loflo[(num+ovr*2)*0+j][i];
      lof07[j][i]=loflo[(num+ovr*2)*1+j][i];
      lof10[j][i]=loflo[(num+ovr*2)*2+j][i];
      lof18[j][i]=loflo[(num+ovr*2)*3+j][i];
      lof23[j][i]=loflo[(num+ovr*2)*4+j][i];
      lof36[j][i]=loflo[(num+ovr*2)*5+j][i];
    }
  }
  // cutoff overlap
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      lof06[j][i]=lof06[j+ovr][i];
      lof07[j][i]=lof07[j+ovr][i];
      lof10[j][i]=lof10[j+ovr][i];
      lof18[j][i]=lof18[j+ovr][i];
      lof23[j][i]=lof23[j+ovr][i];
      lof36[j][i]=lof36[j+ovr][i];
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      lof06[j][i]=0;
      lof07[j][i]=0;
      lof10[j][i]=0;
      lof18[j][i]=0;
      lof23[j][i]=0;
      lof36[j][i]=0;
    }
  }
  // sample display
  printf("lof06[scan=0][pixel=0]: %3d\n", lof06[0][0]);
  printf("lof07[scan=0][pixel=0]: %3d\n", lof07[0][0]);
  printf("lof10[scan=0][pixel=0]: %3d\n", lof10[0][0]);
  printf("lof18[scan=0][pixel=0]: %3d\n", lof18[0][0]);
  printf("lof23[scan=0][pixel=0]: %3d\n", lof23[0][0]);
  printf("lof36[scan=0][pixel=0]: %3d\n", lof36[0][0]);

  // read array: land ocean flag for high
  dhnd = H5Dopen(fhnd, "Land_Ocean Flag 89", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, lofhi);
  if(ret < 0){
    printf("H5Dread error: Land_Ocean Flag 89\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // separate
  for(j = 0; j < num+ovr*2; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lof89a[j][i]=lofhi[(num+ovr*2)*0+j][i];
      lof89b[j][i]=lofhi[(num+ovr*2)*1+j][i];
    }
  }
  // cutoff overlap
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lof89a[j][i]=lof89a[j+ovr][i];
      lof89b[j][i]=lof89b[j+ovr][i];
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lof89a[j][i]=0;
      lof89b[j][i]=0;
    }
  }
  // sample display
  printf("lof89a[scan=0][pixel=0]: %3d\n", lof89a[0][0]);
  printf("lof89b[scan=0][pixel=0]: %3d\n", lof89b[0][0]);

  // read array: earth incidence
  dhnd = H5Dopen(fhnd, "Earth Incidence", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, ear_in);
  if(ret < 0){
    printf("H5Dread error: Earth Incidence\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      ear_in[j][i] = ear_in[j+ovr][i];
      if(ear_in[j][i] > -32767) ear_in[j][i] = ear_in[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      ear_in[j][i] = 0;
    }
  }
  // sample display
  printf("ear_in[scan=0][pixel=0]: %9.2f\n",ear_in[0][0]);

  // read array: earth azimuth
  dhnd = H5Dopen(fhnd, "Earth Azimuth", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, ear_az);
  if(ret < 0){
    printf("H5Dread error: Earth Azimuth\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      ear_az[j][i] = ear_az[j+ovr][i];
      if(ear_az[j][i] > -32767) ear_az[j][i] = ear_az[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      ear_az[j][i] = 0;
    }
  }
  // sample display
  printf("ear_az[scan=0][pixel=0]: %9.2f\n",ear_az[0][0]);

  // close
  ret = H5Fclose(fhnd);
  ret = H5close();
}
