#include <stdio.h>
#include <stdlib.h>
#include "AMTK.h"

// fixed value
#define LMT 2200 // limit of NumberOfScans

int main(int argc, char *argv[]){
  // interface variable
  int i,j;       // loop variable
  int ret;       // return status
  char buf[512]; // text buffer
  void *vpnt;    // pointer to void
  char *fn;      // filename
  hid_t hnd;     // file handle

  // meta data
  char geo[512];  // GeophysicalName
  char gid[512];  // GranuleID
  char tm1[512];  // ObservationStartDateTime
  char tm2[512];  // EquatorCrossingDateTime
  char tm3[512];  // ObservationEndDateTime
  int num;        // NumberOfScans
  int ovr;        // OverlapScans
  char prm1[512]; // CoRegistrationParameterA1
  char prm2[512]; // CoRegistrationParameterA2

  // array data
  AM2_COMMON_SCANTIME st[LMT]; // scantime
  AM2_COMMON_LATLON ll89ar[LMT][AM2_DEF_SNUM_HI]; // latlon for 89a altitude revised
  AM2_COMMON_LATLON ll89br[LMT][AM2_DEF_SNUM_HI]; // latlon for 89b altitude revised
  AM2_COMMON_LATLON lllr  [LMT][AM2_DEF_SNUM_LO]; // latlon for low resolution
  float tb06h06 [LMT][AM2_DEF_SNUM_LO]; // tb for 06h, resolution 06G
  float tb06v06 [LMT][AM2_DEF_SNUM_LO]; // tb for 06v, resolution 06G
  float tb07h06 [LMT][AM2_DEF_SNUM_LO]; // tb for 07h, resolution 06G
  float tb07v06 [LMT][AM2_DEF_SNUM_LO]; // tb for 07v, resolution 06G
  float tb10h10 [LMT][AM2_DEF_SNUM_LO]; // tb for 10h, resolution 10G
  float tb10v10 [LMT][AM2_DEF_SNUM_LO]; // tb for 10v, resolution 10G
  float tb18h23 [LMT][AM2_DEF_SNUM_LO]; // tb for 18h, resolution 23G
  float tb18v23 [LMT][AM2_DEF_SNUM_LO]; // tb for 18v, resolution 23G
  float tb23h23 [LMT][AM2_DEF_SNUM_LO]; // tb for 23h, resolution 23G
  float tb23v23 [LMT][AM2_DEF_SNUM_LO]; // tb for 23v, resolution 23G
  float tb36h36 [LMT][AM2_DEF_SNUM_LO]; // tb for 36h, resolution 36G
  float tb36v36 [LMT][AM2_DEF_SNUM_LO]; // tb for 36v, resolution 36G
  float tb89h36 [LMT][AM2_DEF_SNUM_LO]; // tb for 89h, resolution 36G
  float tb89v36 [LMT][AM2_DEF_SNUM_LO]; // tb for 89v, resolution 36G
  float tb89ah[LMT][AM2_DEF_SNUM_HI]; // tb for 89ah
  float tb89av[LMT][AM2_DEF_SNUM_HI]; // tb for 89av
  float tb89bh[LMT][AM2_DEF_SNUM_HI]; // tb for 89bh
  float tb89bv[LMT][AM2_DEF_SNUM_HI]; // tb for 89bv
  int           pdqlo  [LMT][AM2_DEF_SNUM_LO*16]; // pixel data quality for low
  unsigned char pdq06h [LMT][AM2_DEF_SNUM_LO];    // pixel data quality for 06h
  unsigned char pdq06v [LMT][AM2_DEF_SNUM_LO];    // pixel data quality for 06v
  unsigned char pdq07h [LMT][AM2_DEF_SNUM_LO];    // pixel data quality for 07h
  unsigned char pdq07v [LMT][AM2_DEF_SNUM_LO];    // pixel data quality for 07v
  int           pdqhi  [LMT][AM2_DEF_SNUM_HI* 8]; // pixel data quality for high
  unsigned char pdq89ah[LMT][AM2_DEF_SNUM_HI];    // pixel data quality for 89ah
  unsigned char pdq89av[LMT][AM2_DEF_SNUM_HI];    // pixel data quality for 89av
  unsigned char pdq89bh[LMT][AM2_DEF_SNUM_HI];    // pixel data quality for 89bh
  unsigned char pdq89bv[LMT][AM2_DEF_SNUM_HI];    // pixel data quality for 89bv
  int           loflo [LMT*4][AM2_DEF_SNUM_LO]; // land ocean flag for low
  unsigned char lof06 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 06 
  unsigned char lof10 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 10 
  unsigned char lof23 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 23 
  unsigned char lof36 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 36 
  int           lofhi [LMT*2][AM2_DEF_SNUM_HI]; // land ocean flag for high
  unsigned char lof89a[LMT  ][AM2_DEF_SNUM_HI]; // land ocean flag for 89a
  unsigned char lof89b[LMT  ][AM2_DEF_SNUM_HI]; // land ocean flag for 89b
  float ear_in[AM2_DEF_SNUM_LO][LMT]; // earth incidence
  float ear_az[AM2_DEF_SNUM_LO][LMT]; // earth azimuth

  // argument check
  if(argc < 2){
    printf("argument: AMSR2 L1R DATA FILE (HDF5)\n");
    exit(1);
  }

  // set filename
  fn = argv[1];
  printf("input file: %s\n", fn);

  // open
  hnd=AMTK_openH5(fn);
  if(hnd<0){
    printf("AMTK_openH5 error: %s\n", fn);
    printf("amtk status: %d\n", hnd);
    exit(1);
  }

  // read meta: GeophysicalName
  vpnt=geo;
  ret=AMTK_getMetaDataName(hnd,"GeophysicalName",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: GeophysicalName\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("GeophysicalName: %s\n",geo);

  // read meta: GranuleID
  vpnt=gid;
  ret=AMTK_getMetaDataName(hnd,"GranuleID",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: GranuleID\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("GranuleID: %s\n",gid);

  // read meta: ObservationStartDateTime
  vpnt=tm1;
  ret=AMTK_getMetaDataName(hnd,"ObservationStartDateTime",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: ObservationStartDateTime\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("ObservationStartDateTime: %s\n",tm1);

  // read meta: EquatorCrossingDateTime
  vpnt=tm2;
  ret=AMTK_getMetaDataName(hnd,"EquatorCrossingDateTime",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: EquatorCrossingDateTime\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("EquatorCrossingDateTime: %s\n",tm2);

  // read meta: ObservationEndDateTime
  vpnt=tm3;
  ret=AMTK_getMetaDataName(hnd,"ObservationEndDateTime",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: ObservationEndDateTime\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("ObservationEndDateTime: %s\n",tm3);

  // read meta: NumberOfScans
  vpnt=buf;
  ret=AMTK_getMetaDataName(hnd,"NumberOfScans",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: NumberOfScans\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  num=atoi(buf);
  printf("NumberOfScans: %d\n",num);

  // check limit
  if(num>LMT){
    printf("limit error: limit of NumberOfScans = %d\n",LMT);
    exit(1);
  }
  else{
    printf("limit of NumberOfScans = %d\n",LMT);
  }

  // read meta: OverlapScans
  vpnt=buf;
  ret=AMTK_getMetaDataName(hnd,"OverlapScans",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: OverlapScans\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  ovr=atoi(buf);
  printf("OverlapScans: %d\n",ovr);

  // read meta: CoRegistrationParameterA1
  vpnt=prm1;
  ret=AMTK_getMetaDataName(hnd,"CoRegistrationParameterA1",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: CoRegistrationParameterA1\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("CoRegistrationParameterA1: %s\n",prm1);

  // read meta: CoRegistrationParameterA2
  vpnt=prm2;
  ret=AMTK_getMetaDataName(hnd,"CoRegistrationParameterA2",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: CoRegistrationParameterA2\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("CoRegistrationParameterA2: %s\n",prm2);

  // read array: scantime
  vpnt=st;
  ret=AMTK_getScanTime(hnd,1,num,(AM2_COMMON_SCANTIME **)&vpnt);
  if(ret<0){
    printf("AMTK_getScanTime error.\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("time[scan=0]: %04d/%02d/%02d %02d:%02d:%02d\n"
  , st[0].year
  , st[0].month
  , st[0].day
  , st[0].hour
  , st[0].minute
  , st[0].second
  );

  // read array: latlon for 89a altitude revised
  vpnt=ll89ar;
  ret=AMTK_getLatLon(hnd,(AM2_COMMON_LATLON **)&vpnt,1,num,AM2_LATLON_RS_89A);
  if(ret<0){
    printf("AMTK_getLatLon error: AM2_LATLON_RS_89A\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("latlon89ar[scan=0][pixel=0]: (%9.4f,%9.4f)\n", ll89ar[0][0].lat, ll89ar[0][0].lon);

  // read array: latlon for 89b altitude revised
  vpnt=ll89br;
  ret=AMTK_getLatLon(hnd,(AM2_COMMON_LATLON **)&vpnt,1,num,AM2_LATLON_RS_89B);
  if(ret<0){
    printf("AMTK_getLatLon error: AM2_LATLON_RS_89B\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("latlon89br[scan=0][pixel=0]: (%9.4f,%9.4f)\n", ll89br[0][0].lat, ll89br[0][0].lon);

  // read array: latlon for low resolution
  vpnt=lllr;
  ret=AMTK_getLatLon(hnd,(AM2_COMMON_LATLON **)&vpnt,1,num,AM2_LATLON_RS_LO);
  if(ret<0){
    printf("AMTK_getLatLon error: AM2_LATLON_RS_LO\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("latlonlr[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lllr[0][0].lat, lllr[0][0].lon);

  // read array: tb for 06h, resolution 06G
  vpnt=tb06h06;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES06_TB06H);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES06_TB06H\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb06h06[scan=0][pixel=0]: %9.2f\n", tb06h06[0][0]);

  // read array: tb for 06v, resolution 06G
  vpnt=tb06v06;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES06_TB06V);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES06_TB06V\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb06v06[scan=0][pixel=0]: %9.2f\n", tb06v06[0][0]);

  // read array: tb for 07h, resolution 06G
  vpnt=tb07h06;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES06_TB07H);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES06_TB07H\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb07h06[scan=0][pixel=0]: %9.2f\n", tb07h06[0][0]);

  // read array: tb for 07v, resolution 06G
  vpnt=tb07v06;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES06_TB07V);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES06_TB07V\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb07v06[scan=0][pixel=0]: %9.2f\n", tb07v06[0][0]);

  // read array: tb for 10h, resolution 10G
  vpnt=tb10h10;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES10_TB10H);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES10_TB10H\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb10h10[scan=0][pixel=0]: %9.2f\n", tb10h10[0][0]);

  // read array: tb for 10v, resolution 10G
  vpnt=tb10v10;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES10_TB10V);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES10_TB10V\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb10v10[scan=0][pixel=0]: %9.2f\n", tb10v10[0][0]);

  // read array: tb for 18h, resolution 23G
  vpnt=tb18h23;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES23_TB18H);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES23_TB18H\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb18h23[scan=0][pixel=0]: %9.2f\n", tb18h23[0][0]);

  // read array: tb for 18v, resolution 23G
  vpnt=tb18v23;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES23_TB18V);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES23_TB18V\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb18v23[scan=0][pixel=0]: %9.2f\n", tb18v23[0][0]);

  // read array: tb for 23h, resolution 23G
  vpnt=tb23h23;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES23_TB23H);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES23_TB23H\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb23h23[scan=0][pixel=0]: %9.2f\n", tb23h23[0][0]);

  // read array: tb for 23v, resolution 23G
  vpnt=tb23v23;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES23_TB23V);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES23_TB23V\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb23v23[scan=0][pixel=0]: %9.2f\n", tb23v23[0][0]);

  // read array: tb for 36h, resolution 36G
  vpnt=tb36h36;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES36_TB36H);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES36_TB36H\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb36h36[scan=0][pixel=0]: %9.2f\n", tb36h36[0][0]);

  // read array: tb for 36v, resolution 36G
  vpnt=tb36v36;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES36_TB36V);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES36_TB36V\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb36v36[scan=0][pixel=0]: %9.2f\n", tb36v36[0][0]);

  // read array: tb for 89h, resolution 36G
  vpnt=tb89h36;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES36_TB89H);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES36_TB89H\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb89h36[scan=0][pixel=0]: %9.2f\n", tb89h36[0][0]);

  // read array: tb for 89v, resolution 36G
  vpnt=tb89v36;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_RES36_TB89V);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_RES36_TB89V\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb89v36[scan=0][pixel=0]: %9.2f\n", tb89v36[0][0]);

  // read array: tb for 89ah
  vpnt=tb89ah;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_TB89AH);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_TB89AH\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb89ah[scan=0][pixel=0]: %9.2f\n", tb89ah[0][0]);

  // read array: tb for 89av
  vpnt=tb89av;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_TB89AV);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_TB89AV\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb89av[scan=0][pixel=0]: %9.2f\n", tb89av[0][0]);

  // read array: tb for 89bh
  vpnt=tb89bh;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_TB89BH);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_TB89BH\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb89bh[scan=0][pixel=0]: %9.2f\n", tb89bh[0][0]);

  // read array: tb for 89bv
  vpnt=tb89bv;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_TB89BV);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_TB89BV\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("tb89bv[scan=0][pixel=0]: %9.2f\n", tb89bv[0][0]);

  // read array: pixel data quality for low
  vpnt=pdqlo;
  ret=AMTK_get_SwathInt(hnd,(int **)&vpnt,1,num,AM2_PIX_QUAL_LO);
  if(ret<0){
    printf("AMTK_get_SwathInt error: AM2_PIX_QUAL_LO\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  for(j=0;j<num;++j){
    for(i=0;i<AM2_DEF_SNUM_LO;++i){
      pdq06v[j][i]=pdqlo[j][i*16+ 1]*10+pdqlo[j][i*16+ 0];
      pdq06h[j][i]=pdqlo[j][i*16+ 3]*10+pdqlo[j][i*16+ 2];
      pdq07v[j][i]=pdqlo[j][i*16+ 5]*10+pdqlo[j][i*16+ 4];
      pdq07h[j][i]=pdqlo[j][i*16+ 7]*10+pdqlo[j][i*16+ 6];
    }
  }
  printf("pdq06h[scan=0][pixel=0]): %3d\n", pdq06h[0][0]);
  printf("pdq06v[scan=0][pixel=0]): %3d\n", pdq06v[0][0]);
  printf("pdq07h[scan=0][pixel=0]): %3d\n", pdq07h[0][0]);
  printf("pdq07v[scan=0][pixel=0]): %3d\n", pdq07v[0][0]);

  // read array: pixel data quality for high
  vpnt=pdqhi;
  ret=AMTK_get_SwathInt(hnd,(int **)&vpnt,1,num,AM2_PIX_QUAL_HI);
  if(ret<0){
    printf("AMTK_get_SwathInt error: AM2_PIX_QUAL_HI\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  for(j=0;j<num;++j){
    for(i=1;i<AM2_DEF_SNUM_HI;++i){
      pdq89ah[j][i]=pdqhi[j][i*8+0];
      pdq89av[j][i]=pdqhi[j][i*8+1];
      pdq89bh[j][i]=pdqhi[j][i*8+2];
      pdq89bv[j][i]=pdqhi[j][i*8+3];
    }
  }
  printf("pdq89ah[scan=0][pixel=0]): %3d\n", pdq89ah[0][0]);
  printf("pdq89av[scan=0][pixel=0]): %3d\n", pdq89av[0][0]);
  printf("pdq89ah[scan=0][pixel=0]): %3d\n", pdq89ah[0][0]);
  printf("pdq89av[scan=0][pixel=0]): %3d\n", pdq89av[0][0]);

  // read array: land ocean flag for low
  vpnt=loflo;
  ret=AMTK_get_SwathInt(hnd,(int **)&vpnt,1,num,AM2_LOF_RES_LO);
  if(ret<0){
    printf("AMTK_get_SwathInt error: AM2_LOF_RES_LO\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  for(j=0;j<num;++j){
    for(i=0;i<AM2_DEF_SNUM_LO;++i){
      lof06[j][i]=loflo[num*0+j][i];
      lof10[j][i]=loflo[num*1+j][i];
      lof23[j][i]=loflo[num*2+j][i];
      lof36[j][i]=loflo[num*3+j][i];
    }
  }
  printf("lof06[scan=0][pixel=0]: %3d\n", lof06[0][0]);
  printf("lof10[scan=0][pixel=0]: %3d\n", lof10[0][0]);
  printf("lof23[scan=0][pixel=0]: %3d\n", lof23[0][0]);
  printf("lof36[scan=0][pixel=0]: %3d\n", lof36[0][0]);

  // read array: land ocean flag for high
  vpnt=lofhi;
  ret=AMTK_get_SwathInt(hnd,(int **)&vpnt,1,num,AM2_LOF_RES_HI);
  if(ret<0){
    printf("AMTK_get_SwathInt error: AM2_LOF_RES_HI\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  for(j=0;j<num;++j){
    for(i=0;i<AM2_DEF_SNUM_HI;++i){
      lof89a[j][i]=lofhi[num*0+j][i];
      lof89b[j][i]=lofhi[num*1+j][i];
    }
  }
  printf("lof89a[scan=0][pixel=0]: %3d\n", lof89a[0][0]);
  printf("lof89b[scan=0][pixel=0]: %3d\n", lof89b[0][0]);

  // read array: earth incidence
  vpnt=ear_in;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_EARTH_INC);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_EARTH_INC\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("ear_in[scan=0][pixel=0]: %9.2f\n",ear_in[0][0]);

  // read array: earth incidence
  vpnt=ear_az;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_EARTH_AZ);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_EARTH_AZ\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("ear_az[scan=0][pixel=0]: %9.2f\n",ear_az[0][0]);

  // close
  ret=AMTK_closeH5(hnd);
}
