#include <stdio.h>
#include <stdlib.h>
#include "hdf5.h"
#include "amsr2time.h"

// fixed value
#define LMT 2200 // limit of NumberOfScans
#define AM2_DEF_SNUM_HI 486 // high resolution pixel width
#define AM2_DEF_SNUM_LO 243 // low  resolution pixel width

int main(int argc, char *argv[]){
  // interface variable
  int i,j;          // loop variable
  herr_t ret;       // return status
  char *buf = NULL; // text buffer
  char *fn  = NULL; // filename
  hid_t fhnd;       // file handle
  hid_t ahnd;       // attribute handle
  hid_t atyp;       // attribute type
  hid_t dhnd;       // dataset handle
  hid_t shnd;       // dataspace handle

  // meta data
  char *geo = NULL; // GeophysicalName
  char *gid = NULL; // GranuleID
  char *tm1 = NULL; // ObservationStartDateTime
  char *tm2 = NULL; // EquatorCrossingDateTime
  char *tm3 = NULL; // ObservationEndDateTime
  int num;          // NumberOfScans
  int ovr;          // OverlapScans
  double prm1[7];   // CoRegistrationParameterA1 -> 6G,7G,10G,18G,23G,36G,MEAN
  double prm2[7];   // CoRegistrationParameterA2 -> 6G,7G,10G,18G,23G,36G,MEAN

  // array buffer
  float sca; // scale
  double r8d1[LMT]; // array buffer for double D1
  unsigned char i1d2hi[LMT][AM2_DEF_SNUM_HI]; // array buffer for unsigned char D2 high

  // array data
  AM2_COMMON_SCANTIME st[LMT]; // scantime
  float lat89ar[LMT][AM2_DEF_SNUM_HI]; // lat for 89a altitude revised
  float lat89br[LMT][AM2_DEF_SNUM_HI]; // lat for 89b altitude revised
  float latlr  [LMT][AM2_DEF_SNUM_LO]; // lat for low resolution
  float lon89ar[LMT][AM2_DEF_SNUM_HI]; // lon for 89a altitude revised
  float lon89br[LMT][AM2_DEF_SNUM_HI]; // lon for 89b altitude revised
  float lonlr  [LMT][AM2_DEF_SNUM_LO]; // lon for low resolution
  float tb06h06 [LMT][AM2_DEF_SNUM_LO]; // tb for 06h, resolution 06G
  float tb06v06 [LMT][AM2_DEF_SNUM_LO]; // tb for 06v, resolution 06G
  float tb07h06 [LMT][AM2_DEF_SNUM_LO]; // tb for 07h, resolution 06G
  float tb07v06 [LMT][AM2_DEF_SNUM_LO]; // tb for 07v, resolution 06G
  float tb10h10 [LMT][AM2_DEF_SNUM_LO]; // tb for 10h, resolution 10G
  float tb10v10 [LMT][AM2_DEF_SNUM_LO]; // tb for 10v, resolution 10G
  float tb18h23 [LMT][AM2_DEF_SNUM_LO]; // tb for 18h, resolution 23G
  float tb18v23 [LMT][AM2_DEF_SNUM_LO]; // tb for 18v, resolution 23G
  float tb23h23 [LMT][AM2_DEF_SNUM_LO]; // tb for 23h, resolution 23G
  float tb23v23 [LMT][AM2_DEF_SNUM_LO]; // tb for 23v, resolution 23G
  float tb36h36 [LMT][AM2_DEF_SNUM_LO]; // tb for 36h, resolution 36G
  float tb36v36 [LMT][AM2_DEF_SNUM_LO]; // tb for 36v, resolution 36G
  float tb89h36 [LMT][AM2_DEF_SNUM_LO]; // tb for 89h, resolution 36G
  float tb89v36 [LMT][AM2_DEF_SNUM_LO]; // tb for 89v, resolution 36G
  float tb89ah[LMT][AM2_DEF_SNUM_HI]; // tb for 89ah
  float tb89av[LMT][AM2_DEF_SNUM_HI]; // tb for 89av
  float tb89bh[LMT][AM2_DEF_SNUM_HI]; // tb for 89bh
  float tb89bv[LMT][AM2_DEF_SNUM_HI]; // tb for 89bv
  unsigned char pdq06h [LMT][AM2_DEF_SNUM_LO]; // pixel data quality for 06h
  unsigned char pdq06v [LMT][AM2_DEF_SNUM_LO]; // pixel data quality for 06v
  unsigned char pdq07h [LMT][AM2_DEF_SNUM_LO]; // pixel data quality for 07h
  unsigned char pdq07v [LMT][AM2_DEF_SNUM_LO]; // pixel data quality for 07v
  unsigned char pdq89ah[LMT][AM2_DEF_SNUM_HI]; // pixel data quality for 89ah
  unsigned char pdq89av[LMT][AM2_DEF_SNUM_HI]; // pixel data quality for 89av
  unsigned char pdq89bh[LMT][AM2_DEF_SNUM_HI]; // pixel data quality for 89bh
  unsigned char pdq89bv[LMT][AM2_DEF_SNUM_HI]; // pixel data quality for 89bv
  unsigned char loflo [LMT*4][AM2_DEF_SNUM_LO]; // land ocean flag for low
  unsigned char lof06 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 06 
  unsigned char lof10 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 10 
  unsigned char lof23 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 23 
  unsigned char lof36 [LMT  ][AM2_DEF_SNUM_LO]; // land ocean flag for 36 
  unsigned char lofhi [LMT*2][AM2_DEF_SNUM_HI]; // land ocean flag for high
  unsigned char lof89a[LMT  ][AM2_DEF_SNUM_HI]; // land ocean flag for 89a
  unsigned char lof89b[LMT  ][AM2_DEF_SNUM_HI]; // land ocean flag for 89b
  float ear_in[LMT][AM2_DEF_SNUM_LO]; // earth incidence
  float ear_az[LMT][AM2_DEF_SNUM_LO]; // earth azimuth

  // argument check
  if(argc < 2){
    printf("argument: AMSR2 L1B DATA FILE (HDF5)\n");
    exit(1);
  }

  // set filename
  fn = argv[1];
  printf("input file: %s\n", fn);

  // hdf5 initialize
  ret = H5open();
  if(ret < 0){
    printf("h5open error: %d\n" ,ret);
    exit(1);
  }

  // open
  fhnd = H5Fopen(fn, H5F_ACC_RDONLY, H5P_DEFAULT);
  if(fhnd < 0){
    printf("H5Fopen error: %s\n", fn);
    exit(1);
  }

  // read meta: GeophysicalName
  ahnd = H5Aopen(fhnd, "GeophysicalName", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &geo);
  if(ret < 0){
    printf("H5Aread error: GeophysicalName\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("GeophysicalName: %s\n", geo);

  // read meta: GranuleID
  ahnd = H5Aopen(fhnd, "GranuleID", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &gid);
  if(ret < 0){
    printf("H5Aread error: GranuleID\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("GranuleID: %s\n", gid);

  // read meta: ObservationStartDateTime
  ahnd = H5Aopen(fhnd, "ObservationStartDateTime", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &tm1);
  if(ret < 0){
    printf("H5Aread error: ObservationStartDateTime\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("ObservationStartDateTime: %s\n", tm1);

  // read meta: EquatorCrossingDateTime 
  ahnd = H5Aopen(fhnd, "EquatorCrossingDateTime", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &tm2);
  if(ret < 0){
    printf("H5Aread error: EquatorCrossingDateTime\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("EquatorCrossingDateTime: %s\n", tm2);

  // read meta: ObservationEndDateTime
  ahnd = H5Aopen(fhnd, "ObservationEndDateTime", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &tm3);
  if(ret < 0){
    printf("H5Aread error: ObservationEndDateTime\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("ObservationEndDateTime: %s\n", tm3);

  // read meta: NumberOfScans
  ahnd = H5Aopen(fhnd, "NumberOfScans", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &buf);
  if(ret < 0){
    printf("H5Aread error: NumberOfScans\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  num = atoi(buf);
  printf("NumberOfScans: %d\n", num);

  // check limit
  if(num > LMT){
    printf("limit error: limit of NumberOfScans = %d\n", LMT);
    exit(1);
  }
  else{
    printf("limit of NumberOfScans = %d\n", LMT);
  }

  // read meta: OverlapScans
  ahnd = H5Aopen(fhnd, "OverlapScans", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &buf);
  if(ret < 0){
    printf("H5Aread error: OverlapScans\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  ovr = atoi(buf);
  printf("OverlapScans: %d\n", ovr);

  // read meta: CoRegistrationParameterA1
  ahnd = H5Aopen(fhnd, "CoRegistrationParameterA1", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &buf);
  if(ret < 0){
    printf("H5Aread error: CoRegistrationParameterA1\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  sscanf(buf, "6G-%lf,7G-%lf,10G-%lf,18G-%lf,23G-%lf,36G-%lf"
  , &prm1[0]
  , &prm1[1]
  , &prm1[2]
  , &prm1[3]
  , &prm1[4]
  , &prm1[5]
  );
  prm1[6] = (prm1[0] + prm1[1] + prm1[2] + prm1[3] + prm1[4] + prm1[5]) / 6.0;
  printf("CoRegistrationParameterA1: "
  "6G-%8.5lf,7G-%8.5lf,10G-%8.5lf,18G-%8.5lf,23G-%8.5lf,36G-%8.5lf\n"
  , prm1[0]
  , prm1[1]
  , prm1[2]
  , prm1[3]
  , prm1[4]
  , prm1[5]
  );

  // read meta: CoRegistrationParameterA2
  ahnd = H5Aopen(fhnd, "CoRegistrationParameterA2", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &buf);
  if(ret < 0){
    printf("H5Aread error: CoRegistrationParameterA2\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  sscanf(buf, "6G-%lf,7G-%lf,10G-%lf,18G-%lf,23G-%lf,36G-%lf"
  , &prm2[0]
  , &prm2[1]
  , &prm2[2]
  , &prm2[3]
  , &prm2[4]
  , &prm2[5]
  );
  prm2[6] = (prm2[0] + prm2[1] + prm2[2] + prm2[3] + prm2[4] + prm2[5]) / 6.0;
  printf("CoRegistrationParameterA2: "
  "6G-%8.5lf,7G-%8.5lf,10G-%8.5lf,18G-%8.5lf,23G-%8.5lf,36G-%8.5lf\n"
  , prm2[0]
  , prm2[1]
  , prm2[2]
  , prm2[3]
  , prm2[4]
  , prm2[5]
  );

  // read array: scantime
  dhnd = H5Dopen(fhnd, "Scan Time", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, r8d1);
  if(ret < 0){
    printf("H5Dread error: Scan Time\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap
  for(j = 0; j < num; ++j){
    r8d1[j] = r8d1[j + ovr];
  }
  for(j = num; j < LMT; ++j){
    r8d1[j] = 0;
  }
  // convert
  amsr2time_(&num, r8d1, st);
  // sample display
  printf("time[scan=0]: %04d/%02d/%02d %02d:%02d:%02d\n"
  , st[0].year
  , st[0].month
  , st[0].day
  , st[0].hour
  , st[0].minute
  , st[0].second
  );

  // read array: latlon for 89a altitude revised
  // read lat
  dhnd = H5Dopen(fhnd ,"Latitude of Observation Point for 89A", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lat89ar);
  if(ret < 0){
    printf("H5Dread error: Latitude of Observation Point for 89A\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // read lon
  dhnd = H5Dopen(fhnd ,"Longitude of Observation Point for 89A", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lon89ar);
  if(ret < 0){
    printf("H5Dread error: Longitude of Observation Point for 89A\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lat89ar[j][i] = lat89ar[j+ovr][i];
      lon89ar[j][i] = lon89ar[j+ovr][i];
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lat89ar[j][i] = 0;
      lon89ar[j][i] = 0;
    }
  }
  // sample display
  printf("latlon89ar[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat89ar[0][0], lon89ar[0][0]);

  // read array: latlon for 89b altitude revised
  // read lat
  dhnd = H5Dopen(fhnd ,"Latitude of Observation Point for 89B", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lat89br);
  if(ret < 0){
    printf("H5Dread error: Latitude of Observation Point for 89B\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // read lon
  dhnd = H5Dopen(fhnd ,"Longitude of Observation Point for 89B", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lon89br);
  if(ret < 0){
    printf("H5Dread error: Longitude of Observation Point for 89B\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lat89br[j][i] = lat89br[j+ovr][i];
      lon89br[j][i] = lon89br[j+ovr][i];
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lat89br[j][i] = 0;
      lon89br[j][i] = 0;
    }
  }
  // sample display
  printf("latlon89br[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat89br[0][0], lon89br[0][0]);

  // read array: latlon for low resolution
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      latlr[j][i] = lat89ar[j][i * 2];
      lonlr[j][i] = lon89ar[j][i * 2];
    }
  }
  printf("latlonlr[scan=0][pixel=0]: (%9.4f,%9.4f)\n", latlr[0][0], lonlr[0][0]);

  // read array: tb for 06h, resolution 06G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res06,6.9GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb06h06);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res06,6.9GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb06h06[j][i] = tb06h06[j+ovr][i];
      if(tb06h06[j][i] < 65534) tb06h06[j][i] = tb06h06[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb06h06[j][i] = 0;
    }
  }
  // sample display
  printf("tb06h06[scan=0][pixel=0]: %9.2f\n", tb06h06[0][0]);

  // read array: tb for 06v, resolution 06G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res06,6.9GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb06v06);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res06,6.9GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb06v06[j][i] = tb06v06[j+ovr][i];
      if(tb06v06[j][i] < 65534) tb06v06[j][i] = tb06v06[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb06v06[j][i] = 0;
    }
  }
  // sample display
  printf("tb06v06[scan=0][pixel=0]: %9.2f\n", tb06v06[0][0]);

  // read array: tb for 07h, resolution 06G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res06,7.3GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb07h06);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res06,7.3GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb07h06[j][i] = tb07h06[j+ovr][i];
      if(tb07h06[j][i] < 65534) tb07h06[j][i] = tb07h06[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb07h06[j][i] = 0;
    }
  }
  // sample display
  printf("tb07h06[scan=0][pixel=0]: %9.2f\n", tb07h06[0][0]);

  // read array: tb for 07v, resolution 06G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res06,7.3GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb07v06);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res06,7.3GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb07v06[j][i] = tb07v06[j+ovr][i];
      if(tb07v06[j][i] < 65534) tb07v06[j][i] = tb07v06[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb07v06[j][i] = 0;
    }
  }
  // sample display
  printf("tb07v06[scan=0][pixel=0]: %9.2f\n", tb07v06[0][0]);

  // read array: tb for 10h, resolution 10G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res10,10.7GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb10h10);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res10,10.7GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb10h10[j][i] = tb10h10[j+ovr][i];
      if(tb10h10[j][i] < 65534) tb10h10[j][i] = tb10h10[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb10h10[j][i] = 0;
    }
  }
  // sample display
  printf("tb10h10[scan=0][pixel=0]: %9.2f\n", tb10h10[0][0]);

  // read array: tb for 10v, resolution 10G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res10,10.7GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb10v10);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res10,10.7GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb10v10[j][i] = tb10v10[j+ovr][i];
      if(tb10v10[j][i] < 65534) tb10v10[j][i] = tb10v10[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb10v10[j][i] = 0;
    }
  }
  // sample display
  printf("tb10v10[scan=0][pixel=0]: %9.2f\n", tb10v10[0][0]);

  // read array: tb for 18h, resolution 23G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res23,18.7GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb18h23);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res23,18.7GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb18h23[j][i] = tb18h23[j+ovr][i];
      if(tb18h23[j][i] < 65534) tb18h23[j][i] = tb18h23[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb18h23[j][i] = 0;
    }
  }
  // sample display
  printf("tb18h23[scan=0][pixel=0]: %9.2f\n", tb18h23[0][0]);

  // read array: tb for 18v, resolution 23G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res23,18.7GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb18v23);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res23,18.7GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb18v23[j][i] = tb18v23[j+ovr][i];
      if(tb18v23[j][i] < 65534) tb18v23[j][i] = tb18v23[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb18v23[j][i] = 0;
    }
  }
  // sample display
  printf("tb18v23[scan=0][pixel=0]: %9.2f\n", tb18v23[0][0]);

  // read array: tb for 23h, resolution 23G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res23,23.8GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb23h23);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res23,23.8GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb23h23[j][i] = tb23h23[j+ovr][i];
      if(tb23h23[j][i] < 65534) tb23h23[j][i] = tb23h23[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb23h23[j][i] = 0;
    }
  }
  // sample display
  printf("tb23h23[scan=0][pixel=0]: %9.2f\n", tb23h23[0][0]);

  // read array: tb for 23v, resolution 23G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res23,23.8GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb23v23);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res23,23.8GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb23v23[j][i] = tb23v23[j+ovr][i];
      if(tb23v23[j][i] < 65534) tb23v23[j][i] = tb23v23[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb23v23[j][i] = 0;
    }
  }
  // sample display
  printf("tb23v23[scan=0][pixel=0]: %9.2f\n", tb23v23[0][0]);

  // read array: tb for 36h, resolution 36G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res36,36.5GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb36h36);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res36,36.5GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb36h36[j][i] = tb36h36[j+ovr][i];
      if(tb36h36[j][i] < 65534) tb36h36[j][i] = tb36h36[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb36h36[j][i] = 0;
    }
  }
  // sample display
  printf("tb36h36[scan=0][pixel=0]: %9.2f\n", tb36h36[0][0]);

  // read array: tb for 36v, resolution 36G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res36,36.5GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb36v36);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res36,36.5GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb36v36[j][i] = tb36v36[j+ovr][i];
      if(tb36v36[j][i] < 65534) tb36v36[j][i] = tb36v36[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb36v36[j][i] = 0;
    }
  }
  // sample display
  printf("tb36v36[scan=0][pixel=0]: %9.2f\n", tb36v36[0][0]);

  // read array: tb for 89h, resolution 36G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res36,89.0GHz,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb89h36);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res36,89.0GHz,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb89h36[j][i] = tb89h36[j+ovr][i];
      if(tb89h36[j][i] < 65534) tb89h36[j][i] = tb89h36[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb89h36[j][i] = 0;
    }
  }
  // sample display
  printf("tb89h36[scan=0][pixel=0]: %9.2f\n", tb89h36[0][0]);

  // read array: tb for 89v, resolution 36G
  dhnd = H5Dopen(fhnd, "Brightness Temperature (res36,89.0GHz,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb89v36);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (res36,89.0GHz,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb89v36[j][i] = tb89v36[j+ovr][i];
      if(tb89v36[j][i] < 65534) tb89v36[j][i] = tb89v36[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      tb89v36[j][i] = 0;
    }
  }
  // sample display
  printf("tb89v36[scan=0][pixel=0]: %9.2f\n", tb89v36[0][0]);

  // read array: tb for 89ah
  dhnd = H5Dopen(fhnd, "Brightness Temperature (original,89GHz-A,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb89ah);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (original,89GHz-A,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89ah[j][i] = tb89ah[j+ovr][i];
      if(tb89ah[j][i] < 65534) tb89ah[j][i] = tb89ah[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89ah[j][i] = 0;
    }
  }
  // sample display
  printf("tb89ah[scan=0][pixel=0]: %9.2f\n", tb89ah[0][0]);

  // read array: tb for 89av
  dhnd = H5Dopen(fhnd, "Brightness Temperature (original,89GHz-A,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb89av);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (original,89GHz-A,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89av[j][i] = tb89av[j+ovr][i];
      if(tb89av[j][i] < 65534) tb89av[j][i] = tb89av[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89av[j][i] = 0;
    }
  }
  // sample display
  printf("tb89av[scan=0][pixel=0]: %9.2f\n", tb89av[0][0]);

  // read array: tb for 89bh
  dhnd = H5Dopen(fhnd, "Brightness Temperature (original,89GHz-B,H)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb89bh);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (original,89GHz-B,H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89bh[j][i] = tb89bh[j+ovr][i];
      if(tb89bh[j][i] < 65534) tb89bh[j][i] = tb89bh[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89bh[j][i] = 0;
    }
  }
  // sample display
  printf("tb89bh[scan=0][pixel=0]: %9.2f\n", tb89bh[0][0]);

  // read array: tb for 89bv
  dhnd = H5Dopen(fhnd, "Brightness Temperature (original,89GHz-B,V)", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tb89bv);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (original,89GHz-B,V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89bv[j][i] = tb89bv[j+ovr][i];
      if(tb89bv[j][i] < 65534) tb89bv[j][i] = tb89bv[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      tb89bv[j][i] = 0;
    }
  }
  // sample display
  printf("tb89bv[scan=0][pixel=0]: %9.2f\n", tb89bv[0][0]);

  // read array: pixel data quality for low
  dhnd = H5Dopen(fhnd, "Pixel Data Quality 6 to 36", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, i1d2hi);
  if(ret < 0){
    printf("H5Dread error: Pixel Data Quality 6 to 36\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & separate
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      pdq06v[j][i]=0;
      if((i1d2hi[j+ovr][i*2+0] &   1) != 0) pdq06v[j][i]=1;
      if((i1d2hi[j+ovr][i*2+0] &   2) != 0) pdq06v[j][i]=10;
      if(((i1d2hi[j+ovr][i*2+0] &   1) != 0) && ((i1d2hi[j+ovr][i*2+0] &   2) != 0)) pdq06v[j][i]=11;
      pdq06h[j][i]=0;
      if((i1d2hi[j+ovr][i*2+0] &   4) != 0) pdq06h[j][i]=1;
      if((i1d2hi[j+ovr][i*2+0] &   8) != 0) pdq06h[j][i]=10;
      if(((i1d2hi[j+ovr][i*2+0] &   4) != 0) && ((i1d2hi[j+ovr][i*2+0] &   8) != 0)) pdq06h[j][i]=11;
      pdq07v[j][i]=0;
      if((i1d2hi[j+ovr][i*2+0] &  16) != 0) pdq07v[j][i]=1;
      if((i1d2hi[j+ovr][i*2+0] &  32) != 0) pdq07v[j][i]=10;
      if(((i1d2hi[j+ovr][i*2+0] &  16) != 0) && ((i1d2hi[j+ovr][i*2+0] &  32) != 0)) pdq07v[j][i]=11;
      pdq07h[j][i]=0;
      if((i1d2hi[j+ovr][i*2+0] &  64) != 0) pdq07h[j][i]=1;
      if((i1d2hi[j+ovr][i*2+0] & 128) != 0) pdq07h[j][i]=10;
      if(((i1d2hi[j+ovr][i*2+0] &  64) != 0) && ((i1d2hi[j+ovr][i*2+0] & 128) != 0)) pdq07h[j][i]=11;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      pdq06h[j][i]=0;
      pdq06v[j][i]=0;
      pdq07h[j][i]=0;
      pdq07v[j][i]=0;
    }
  }
  // sample display
  printf("pdq06h[scan=0][pixel=0]): %3d\n", pdq06h[0][0]);
  printf("pdq06v[scan=0][pixel=0]): %3d\n", pdq06v[0][0]);
  printf("pdq07h[scan=0][pixel=0]): %3d\n", pdq07h[0][0]);
  printf("pdq07v[scan=0][pixel=0]): %3d\n", pdq07v[0][0]);

  // read array: pixel data quality for high
  dhnd = H5Dopen(fhnd, "Pixel Data Quality 89", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, i1d2hi);
  if(ret < 0){
    printf("H5Dread error: Pixel Data Quality 89\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & separate
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      pdq89ah[j][i]=0; if((i1d2hi[j+ovr][i] & 1) != 0) pdq89ah[j][i]=1;
      pdq89av[j][i]=0; if((i1d2hi[j+ovr][i] & 2) != 0) pdq89av[j][i]=1;
      pdq89ah[j][i]=0; if((i1d2hi[j+ovr][i] & 4) != 0) pdq89ah[j][i]=1;
      pdq89av[j][i]=0; if((i1d2hi[j+ovr][i] & 8) != 0) pdq89av[j][i]=1;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      pdq89ah[j][i]=0;
      pdq89av[j][i]=0;
      pdq89ah[j][i]=0;
      pdq89av[j][i]=0;
    }
  }
  // sample display
  printf("pdq89ah[scan=0][pixel=0]): %3d\n", pdq89ah[0][0]);
  printf("pdq89av[scan=0][pixel=0]): %3d\n", pdq89av[0][0]);
  printf("pdq89ah[scan=0][pixel=0]): %3d\n", pdq89ah[0][0]);
  printf("pdq89av[scan=0][pixel=0]): %3d\n", pdq89av[0][0]);

  // read array: land ocean flag for low
  dhnd = H5Dopen(fhnd, "Land_Ocean Flag 6 to 36", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, loflo);
  if(ret < 0){
    printf("H5Dread error: Land_Ocean Flag 6 to 36\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // separate
  for(j = 0; j < num+ovr*2; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      lof06[j][i]=loflo[(num+ovr*2)*0+j][i];
      lof10[j][i]=loflo[(num+ovr*2)*1+j][i];
      lof23[j][i]=loflo[(num+ovr*2)*2+j][i];
      lof36[j][i]=loflo[(num+ovr*2)*3+j][i];
    }
  }
  // cutoff overlap
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      lof06[j][i]=lof06[j+ovr][i];
      lof10[j][i]=lof10[j+ovr][i];
      lof23[j][i]=lof23[j+ovr][i];
      lof36[j][i]=lof36[j+ovr][i];
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      lof06[j][i]=0;
      lof10[j][i]=0;
      lof23[j][i]=0;
      lof36[j][i]=0;
    }
  }
  // sample display
  printf("lof06[scan=0][pixel=0]: %3d\n", lof06[0][0]);
  printf("lof10[scan=0][pixel=0]: %3d\n", lof10[0][0]);
  printf("lof23[scan=0][pixel=0]: %3d\n", lof23[0][0]);
  printf("lof36[scan=0][pixel=0]: %3d\n", lof36[0][0]);

  // read array: land ocean flag for high
  dhnd = H5Dopen(fhnd, "Land_Ocean Flag 89", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, lofhi);
  if(ret < 0){
    printf("H5Dread error: Land_Ocean Flag 89\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // separate
  for(j = 0; j < num+ovr*2; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lof89a[j][i]=lofhi[(num+ovr*2)*0+j][i];
      lof89b[j][i]=lofhi[(num+ovr*2)*1+j][i];
    }
  }
  // cutoff overlap
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lof89a[j][i]=lof89a[j+ovr][i];
      lof89b[j][i]=lof89b[j+ovr][i];
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      lof89a[j][i]=0;
      lof89b[j][i]=0;
    }
  }
  // sample display
  printf("lof89a[scan=0][pixel=0]: %3d\n", lof89a[0][0]);
  printf("lof89b[scan=0][pixel=0]: %3d\n", lof89b[0][0]);

  // read array: earth incidence
  dhnd = H5Dopen(fhnd, "Earth Incidence", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, ear_in);
  if(ret < 0){
    printf("H5Dread error: Earth Incidence\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      ear_in[j][i] = ear_in[j+ovr][i];
      if(ear_in[j][i] > -32767) ear_in[j][i] = ear_in[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      ear_in[j][i] = 0;
    }
  }
  // sample display
  printf("ear_in[scan=0][pixel=0]: %9.2f\n",ear_in[0][0]);

  // read array: earth azimuth
  dhnd = H5Dopen(fhnd, "Earth Azimuth", H5P_DEFAULT);
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT); // get scale
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);       // get scale
  ret = H5Aclose(ahnd);                              // get scale
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, ear_az);
  if(ret < 0){
    printf("H5Dread error: Earth Azimuth\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // cutoff overlap & change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      ear_az[j][i] = ear_az[j+ovr][i];
      if(ear_az[j][i] > -32767) ear_az[j][i] = ear_az[j][i] * sca;
    }
  }
  for(j = num; j < LMT; ++j){
    for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
      ear_az[j][i] = 0;
    }
  }
  // sample display
  printf("ear_az[scan=0][pixel=0]: %9.2f\n",ear_az[0][0]);

  // close
  ret = H5Fclose(fhnd);
  ret = H5close();
}
