#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "AMTK.h"

// fixed value
#define LMT 2200 // limit of NumberOfScans
#define MES "AMSR2 L2 HIGH RESOLUTION DATA FILE (HDF5) [PRC]"

int main(int argc, char *argv[]){
  // interface variable
  int i,j;       // loop variable
  int ret;       // return status
  char buf[512]; // text buffer
  void *vpnt;    // pointer to void
  char *fn;      // filename
  hid_t hnd;     // file handle

  // meta data
  char geo[512];  // GeophysicalName
  char gid[512];  // GranuleID
  char tm1[512];  // ObservationStartDateTime
  char tm2[512];  // EquatorCrossingDateTime
  char tm3[512];  // ObservationEndDateTime
  int num;        // NumberOfScans
  int ovr;        // OverlapScans

  // array data
  AM2_COMMON_SCANTIME st[LMT]; // scantime
  AM2_COMMON_LATLON ll89a[LMT][AM2_DEF_SNUM_HI]; // latlon for 89a
  AM2_COMMON_LATLON ll89b[LMT][AM2_DEF_SNUM_HI]; // latlon for 89b
  float geo1_89a[LMT][AM2_DEF_SNUM_HI]; // geophysical data layer 1 for 89a
  float geo1_89b[LMT][AM2_DEF_SNUM_HI]; // geophysical data layer 1 for 89b
  unsigned char pdq1_89a[LMT][AM2_DEF_SNUM_HI]; // pixel data quality layer 1 for 89a
  unsigned char pdq1_89b[LMT][AM2_DEF_SNUM_HI]; // pixel data quality layer 1 for 89b

  // argument check
  if(argc < 2){
    printf("argument: %s\n", MES);
    exit(1);
  }

  // set filename
  fn = argv[1];
  printf("input file: %s\n", fn);

  // open
  hnd=AMTK_openH5(fn);
  if(hnd<0){
    printf("AMTK_openH5 error: %s\n", fn);
    printf("amtk status: %d\n", hnd);
    exit(1);
  }

  // read meta: GeophysicalName
  vpnt=geo;
  ret=AMTK_getMetaDataName(hnd,"GeophysicalName",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: GeophysicalName\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("GeophysicalName: %s\n",geo);

  // read meta: GranuleID
  vpnt=gid;
  ret=AMTK_getMetaDataName(hnd,"GranuleID",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: GranuleID\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("GranuleID: %s\n",gid);

  // check geophysical type
  if(strncmp(gid+29,"PRC",3)!=0){
    printf("wrong input file: this program is for %s\n", MES);
    exit(1);
  }

  // read meta: ObservationStartDateTime
  vpnt=tm1;
  ret=AMTK_getMetaDataName(hnd,"ObservationStartDateTime",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: ObservationStartDateTime\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("ObservationStartDateTime: %s\n",tm1);

  // read meta: EquatorCrossingDateTime
  vpnt=tm2;
  ret=AMTK_getMetaDataName(hnd,"EquatorCrossingDateTime",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: EquatorCrossingDateTime\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("EquatorCrossingDateTime: %s\n",tm2);

  // read meta: ObservationEndDateTime
  vpnt=tm3;
  ret=AMTK_getMetaDataName(hnd,"ObservationEndDateTime",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: ObservationEndDateTime\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("ObservationEndDateTime: %s\n",tm3);

  // read meta: NumberOfScans
  vpnt=buf;
  ret=AMTK_getMetaDataName(hnd,"NumberOfScans",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: NumberOfScans\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  num=atoi(buf);
  printf("NumberOfScans: %d\n",num);

  // check limit
  if(num>LMT){
    printf("limit error: limit of NumberOfScans = %d\n",LMT);
    exit(1);
  }
  else{
    printf("limit of NumberOfScans = %d\n",LMT);
  }

  // read meta: OverlapScans
  vpnt=buf;
  ret=AMTK_getMetaDataName(hnd,"OverlapScans",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: OverlapScans\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  ovr=atoi(buf);
  printf("OverlapScans: %d\n",ovr);

  // read array: scantime
  vpnt=st;
  ret=AMTK_getScanTime(hnd,1,num,(AM2_COMMON_SCANTIME **)&vpnt);
  if(ret<0){
    printf("AMTK_getScanTime error.\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("time[scan=0]: %04d/%02d/%02d %02d:%02d:%02d\n"
  , st[0].year
  , st[0].month
  , st[0].day
  , st[0].hour
  , st[0].minute
  , st[0].second
  );

  // read array: latlon for 89a
  vpnt=ll89a;
  ret=AMTK_getLatLon(hnd,(AM2_COMMON_LATLON **)&vpnt,1,num,AM2_LATLON_L2_89A);
  if(ret<0){
    printf("AMTK_getLatLon error: AM2_LATLON_L2_89A\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("latlon89a[scan=0][pixel=0]: (%9.4f,%9.4f)\n", ll89a[0][0].lat, ll89a[0][0].lon);

  // read array: latlon for 89b
  vpnt=ll89b;
  ret=AMTK_getLatLon(hnd,(AM2_COMMON_LATLON **)&vpnt,1,num,AM2_LATLON_L2_89B);
  if(ret<0){
    printf("AMTK_getLatLon error: AM2_LATLON_L2_89B\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("latlon89b[scan=0][pixel=0]: (%9.4f,%9.4f)\n", ll89b[0][0].lat, ll89b[0][0].lon);

  // read array: geophysical data for 1 layer for 89a
  vpnt=geo1_89a;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_SWATHA_GEO1);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_SWATHA_GEO1\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }

  // read array: geophysical data for 1 layer for 89b
  vpnt=geo1_89b;
  ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_SWATHB_GEO1);
  if(ret<0){
    printf("AMTK_get_SwathFloat error: AM2_SWATHB_GEO1\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }

  // read array: pixel data quality for 1 layer for 89a
  vpnt=pdq1_89a;
  ret=AMTK_get_SwathUChar(hnd,(unsigned char **)&vpnt,1,num,AM2_PIX_QUAL_A);
  if(ret<0){
    printf("AMTK_get_SwathUChar error: AM2_PIX_QUAL_A\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }

  // read array: pixel data quality for 1 layer for 89b
  vpnt=pdq1_89b;
  ret=AMTK_get_SwathUChar(hnd,(unsigned char **)&vpnt,1,num,AM2_PIX_QUAL_B);
  if(ret<0){
    printf("AMTK_get_SwathUChar error: AM2_PIX_QUAL_B\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }

  // display sample
  printf("geo1_89a[scan=0][pixel=0]: %8.1f [mm/h] (PDQ:%3d)\n",geo1_89a[0][0],pdq1_89a[0][0]);
  printf("geo1_89b[scan=0][pixel=0]: %8.1f [mm/h] (PDQ:%3d)\n",geo1_89b[0][0],pdq1_89b[0][0]);

  // close
  ret=AMTK_closeH5(hnd);
}
