      program main
      implicit none
C include
      include 'AMTK_f.h'
C intrinsic function
      integer(4) iargc ! pgf need explicitly declaration
C fixed value
      integer(4),parameter::LMT=2200 ! limit of NumberOfScans
      character(len=*),parameter::MES=
     +'AMSR2 L2 HIGH RESOLUTION DATA FILE (HDF5) [PRC]'
C interface variable
      integer(4) i,j         ! loop variable
      integer(4) ret         ! return status
      character(len=512) buf ! text buffer
      character(len=512) fn  ! filename
      integer(4) hnd         ! file handle
C meta data
      character(len=512) geo  ! GeophysicalName
      character(len=512) gid  ! GranuleID
      character(len=512) tm1  ! ObservationStartDateTime
      character(len=512) tm2  ! EquatorCrossingDateTime
      character(len=512) tm3  ! ObservationEndDateTime
      integer(4) num          ! NumberOfScans
      integer(4) ovr          ! OverlapScans
C array data
      type(AM2_COMMON_SCANTIME) st(LMT) ! scantime
      type(AM2_COMMON_LATLON) ll89a(AM2_DEF_SNUM_HI,LMT) ! latlon for 89a
      type(AM2_COMMON_LATLON) ll89b(AM2_DEF_SNUM_HI,LMT) ! latlon for 89b
      real(4) geo1_89a(AM2_DEF_SNUM_HI,LMT) ! geophysical data layer 1 for 89a
      real(4) geo1_89b(AM2_DEF_SNUM_HI,LMT) ! geophysical data layer 1 for 89b
      integer(4) pdq1_89a(AM2_DEF_SNUM_HI,LMT) ! pixel data quality layer 1 for 89a
      integer(4) pdq1_89b(AM2_DEF_SNUM_HI,LMT) ! pixel data quality layer 1 for 89b
      integer(1) pdqtmp(AM2_DEF_SNUM_HI,LMT) ! pixel data quality temporary
C argument check
      if(iargc().lt.1)then
        write(*,'(a)')'argument: '//MES
        call exit(1)
      endif
C set filename
      call getarg(1,fn)
      write(*,'(a,a)')'input file: ',fn(1:len_trim(fn))
C open
      hnd=AMTK_openH5(fn)
      if(hnd.lt.0)then
        write(*,'(a,a)')'AMTK_openH5 error: ',fn(1:len_trim(fn))
        write(*,'(a,i12)')'amtk status: ',hnd
        call exit(1)
      endif
C read meta: GeophysicalName
      ret=AMTK_getMetaDataName(hnd,'GeophysicalName',geo)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getMetaDataName error: GeophysicalName'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'GeophysicalName: ',geo(1:len_trim(geo))
C read meta: GranuleID
      ret=AMTK_getMetaDataName(hnd,'GranuleID',gid)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getMetaDataName error: GranuleID'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'GranuleID: ',gid(1:len_trim(gid))
C check geophysical type
      if(gid(30:32).ne.'PRC')then
        write(*,'(a)')'wrong input file: this program is for '//MES
        call exit(1)
      endif
C read meta: ObservationStartDateTime
      ret=AMTK_getMetaDataName(hnd,'ObservationStartDateTime',tm1)
      if(ret.lt.0)then
        write(*,'(a,a)')'AMTK_getMetaDataName error: '
     +  ,'ObservationStartDateTime'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'ObservationStartDateTime: ',tm1(1:len_trim(tm1))
C read meta: EquatorCrossingDateTime
      ret=AMTK_getMetaDataName(hnd,'EquatorCrossingDateTime',tm2)
      if(ret.lt.0)then
        write(*,'(a,a)')'AMTK_getMetaDataName error: '
     +  ,'EquatorCrossingDateTime'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'EquatorCrossingDateTime: ',tm2(1:len_trim(tm2))
C read meta: ObservationEndDateTime
      ret=AMTK_getMetaDataName(hnd,'ObservationEndDateTime',tm3)
      if(ret.lt.0)then
        write(*,'(a,a)')'AMTK_getMetaDataName error: '
     +  ,'ObservationEndDateTime'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'ObservationEndDateTime: ',tm3(1:len_trim(tm3))
C read meta: NumberOfScans
      ret=AMTK_getMetaDataName(hnd,'NumberOfScans',buf)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getMetaDataName error: NumberOfScans'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      read(buf(1:ret),*)num
      write(*,'(a,i12)')'NumberOfScans: ',num
C check limit
      if(num.gt.LMT)then
        write(*,'(a,i6)')'limit error: limit of NumberOfScans = ',LMT
        call exit(1)
      else
        write(*,'(a,i6)')'limit of NumberOfScans = ',LMT
      endif
C read meta: OverlapScans
      ret=AMTK_getMetaDataName(hnd,'OverlapScans',buf)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getMetaDataName error: OverlapScans'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      read(buf(1:ret),*)ovr
      write(*,'(a,i12)')'OverlapScans: ',ovr
C read array: scantime
      ret=AMTK_getScanTime(hnd,1,num,st)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getScanTime error.'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,i4.4,"/",i2.2,"/",i2.2," ",i2.2,":",i2.2,":",i2.2)')
     +'time(scan=1): '
     +,st(1)%year
     +,st(1)%month
     +,st(1)%day
     +,st(1)%hour
     +,st(1)%minute
     +,st(1)%second
C read array: latlon for 89a
      ret=AMTK_getLatLon(hnd,ll89a,1,num,AM2_LATLON_L2_89A)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getLatLon error: AM2_LATLON_L2_89A'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,"(",f9.4,",",f9.4,")")')'latlon89a(pixel=1,scan=1): '
     +,ll89a(1,1)%lat,ll89a(1,1)%lon
C read array: latlon for 89b
      ret=AMTK_getLatLon(hnd,ll89b,1,num,AM2_LATLON_L2_89B)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getLatLon error: AM2_LATLON_L2_89B'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,"(",f9.4,",",f9.4,")")')'latlon89b(pixel=1,scan=1): '
     +,ll89b(1,1)%lat,ll89b(1,1)%lon
C read array: geophysical data for 1 layer for 89a
      ret=AMTK_get_SwathFloat(hnd,geo1_89a,1,num,AM2_SWATHA_GEO1)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_get_SwathFloat error: AM2_SWATHA_GEO1'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
C read array: geophysical data for 1 layer for 89b
      ret=AMTK_get_SwathFloat(hnd,geo1_89b,1,num,AM2_SWATHB_GEO1)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_get_SwathFloat error: AM2_SWATHB_GEO1'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
C read array: pixel data quality for 1 layer for 89a
      ! read
      ret=AMTK_get_SwathUChar(hnd,pdqtmp,1,num,AM2_PIX_QUAL_A)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_get_SwathUChar error: AM2_PIX_QUAL_A'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      ! convert signed to unsigned
      do j=1,num
        do i=1,AM2_DEF_SNUM_HI
          pdq1_89a(i,j)=pdqtmp(i,j)
          if(pdq1_89a(i,j).ge.0)then
            pdq1_89a(i,j)=pdq1_89a(i,j)
          else
            pdq1_89a(i,j)=pdq1_89a(i,j)+256
          endif
        enddo
      enddo
C read array: pixel data quality for 1 layer for 89b
      ! read
      ret=AMTK_get_SwathUChar(hnd,pdqtmp,1,num,AM2_PIX_QUAL_B)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_get_SwathUChar error: AM2_PIX_QUAL_B'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      ! convert signed to unsigned
      do j=1,num
        do i=1,AM2_DEF_SNUM_HI
          pdq1_89b(i,j)=pdqtmp(i,j)
          if(pdq1_89b(i,j).ge.0)then
            pdq1_89b(i,j)=pdq1_89b(i,j)
          else
            pdq1_89b(i,j)=pdq1_89b(i,j)+256
          endif
        enddo
      enddo
C display sample
      write(*,'(a,f8.1,a,i3,a)')'geo1_89a(pixel=1,scan=1): '
     +,geo1_89a(1,1),' [mm/h] (PDQ:',pdq1_89a(1,1),')'
      write(*,'(a,f8.1,a,i3,a)')'geo1_89b(pixel=1,scan=1): '
     +,geo1_89b(1,1),' [mm/h] (PDQ:',pdq1_89b(1,1),')'
C close
      ret=AMTK_closeH5(hnd)
      end
