#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "hdf5.h"
#include "amsr2time.h"

// fixed value
#define LMT 2200 // limit of NumberOfScans
#define AM2_DEF_SNUM_HI 486 // high resolution pixel width
#define AM2_DEF_SNUM_LO 243 // low  resolution pixel width
#define MES "AMSR2 L2 HIGH RESOLUTION DATA FILE (HDF5) [PRC]"

int main(int argc, char *argv[]){
  // interface variable
  int i,j;          // loop variable
  herr_t ret;       // return status
  char *buf = NULL; // text buffer
  char *fn  = NULL; // filename
  hid_t fhnd;       // file handle
  hid_t ahnd;       // attribute handle
  hid_t atyp;       // attribute type
  hid_t dhnd;       // dataset handle

  // meta data
  char *geo = NULL; // GeophysicalName
  char *gid = NULL; // GranuleID
  char *tm1 = NULL; // ObservationStartDateTime
  char *tm2 = NULL; // EquatorCrossingDateTime
  char *tm3 = NULL; // ObservationEndDateTime
  int num;          // NumberOfScans
  int ovr;          // OverlapScans

  // array buffer
  float sca; // scale
  double r8d1[LMT]; // array buffer for double D1

  // array data
  AM2_COMMON_SCANTIME st[LMT]; // scantime
  float lat89a[LMT][AM2_DEF_SNUM_HI]; // lat for 89a
  float lat89b[LMT][AM2_DEF_SNUM_HI]; // lat for 89b
  float lon89a[LMT][AM2_DEF_SNUM_HI]; // lon for 89a
  float lon89b[LMT][AM2_DEF_SNUM_HI]; // lon for 89b
  float geo1_89a[LMT][AM2_DEF_SNUM_HI]; // geophysical data layer 1 for 89a
  float geo1_89b[LMT][AM2_DEF_SNUM_HI]; // geophysical data layer 1 for 89b
  unsigned char pdq1_89a[LMT][AM2_DEF_SNUM_HI]; // pixel data quality layer 1 for 89a
  unsigned char pdq1_89b[LMT][AM2_DEF_SNUM_HI]; // pixel data quality layer 1 for 89b

  // argument check
  if(argc < 2){
    printf("argument: %s\n", MES);
    exit(1);
  }

  // set filename
  fn = argv[1];
  printf("input file: %s\n", fn);

  // hdf5 initialize
  ret = H5open();
  if(ret < 0){
    printf("h5open error: %d\n" ,ret);
    exit(1);
  }

  // open
  fhnd = H5Fopen(fn, H5F_ACC_RDONLY, H5P_DEFAULT);
  if(fhnd < 0){
    printf("H5Fopen error: %s\n", fn);
    exit(1);
  }

  // read meta: GeophysicalName
  ahnd = H5Aopen(fhnd, "GeophysicalName", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &geo);
  if(ret < 0){
    printf("H5Aread error: GeophysicalName\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("GeophysicalName: %s\n", geo);

  // read meta: GranuleID
  ahnd = H5Aopen(fhnd, "GranuleID", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &gid);
  if(ret < 0){
    printf("H5Aread error: GranuleID\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("GranuleID: %s\n", gid);

  // check geophysical type
  if(strncmp(gid+29,"PRC",3)!=0){
    printf("wrong input file: this program is for %s\n", MES);
    exit(1);
  }

  // read meta: ObservationStartDateTime
  ahnd = H5Aopen(fhnd, "ObservationStartDateTime", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &tm1);
  if(ret < 0){
    printf("H5Aread error: ObservationStartDateTime\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("ObservationStartDateTime: %s\n", tm1);

  // read meta: EquatorCrossingDateTime 
  ahnd = H5Aopen(fhnd, "EquatorCrossingDateTime", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &tm2);
  if(ret < 0){
    printf("H5Aread error: EquatorCrossingDateTime\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("EquatorCrossingDateTime: %s\n", tm2);

  // read meta: ObservationEndDateTime
  ahnd = H5Aopen(fhnd, "ObservationEndDateTime", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &tm3);
  if(ret < 0){
    printf("H5Aread error: ObservationEndDateTime\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("ObservationEndDateTime: %s\n", tm3);

  // read meta: NumberOfScans
  ahnd = H5Aopen(fhnd, "NumberOfScans", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &buf);
  if(ret < 0){
    printf("H5Aread error: NumberOfScans\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  num = atoi(buf);
  printf("NumberOfScans: %d\n", num);

  // check limit
  if(num > LMT){
    printf("limit error: limit of NumberOfScans = %d\n", LMT);
    exit(1);
  }
  else{
    printf("limit of NumberOfScans = %d\n", LMT);
  }

  // read meta: OverlapScans
  ahnd = H5Aopen(fhnd, "OverlapScans", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &buf);
  if(ret < 0){
    printf("H5Aread error: OverlapScans\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  ovr = atoi(buf);
  printf("OverlapScans: %d\n", ovr);

  // read array: scantime
  dhnd = H5Dopen(fhnd, "Scan Time", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, r8d1);
  if(ret < 0){
    printf("H5Dread error: Scan Time\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // convert
  amsr2time_(&num, r8d1, st);
  // sample display
  printf("time[scan=0]: %04d/%02d/%02d %02d:%02d:%02d\n"
  , st[0].year
  , st[0].month
  , st[0].day
  , st[0].hour
  , st[0].minute
  , st[0].second
  );

  // read array: latlon for 89a
  // read lat
  dhnd = H5Dopen(fhnd ,"Latitude of Observation Point for 89A", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lat89a);
  if(ret < 0){
    printf("H5Dread error: Latitude of Observation Point for 89A\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // read lon
  dhnd = H5Dopen(fhnd ,"Longitude of Observation Point for 89A", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lon89a);
  if(ret < 0){
    printf("H5Dread error: Longitude of Observation Point for 89A\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // sample display
  printf("latlon89a[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat89a[0][0], lon89a[0][0]);

  // read array: latlon for 89b
  // read lat
  dhnd = H5Dopen(fhnd ,"Latitude of Observation Point for 89B", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lat89b);
  if(ret < 0){
    printf("H5Dread error: Latitude of Observation Point for 89B\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // read lon
  dhnd = H5Dopen(fhnd ,"Longitude of Observation Point for 89B", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lon89b);
  if(ret < 0){
    printf("H5Dread error: Longitude of Observation Point for 89B\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // sample display
  printf("latlon89b[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat89b[0][0], lon89b[0][0]);

  // read array: geophysical data for 1 layer for 89a
  dhnd = H5Dopen(fhnd, "Geophysical Data for 89A", H5P_DEFAULT);
  // get scale
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT);
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);
  ret = H5Aclose(ahnd);
  // read
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, geo1_89a);
  if(ret < 0){
    printf("H5Dread error: Geophysical Data for 89A\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      if(geo1_89a[j][i] > -32767) geo1_89a[j][i] = geo1_89a[j][i] * sca;
    }
  }

  // read array: geophysical data for 1 layer for 89b
  dhnd = H5Dopen(fhnd, "Geophysical Data for 89B", H5P_DEFAULT);
  // get scale
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT);
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);
  ret = H5Aclose(ahnd);
  // read
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, geo1_89b);
  if(ret < 0){
    printf("H5Dread error: Geophysical Data for 89B\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // change scale
  for(j = 0; j < num; ++j){
    for(i = 0; i < AM2_DEF_SNUM_HI; ++i){
      if(geo1_89b[j][i] > -32767) geo1_89b[j][i] = geo1_89b[j][i] * sca;
    }
  }

  // read array: pixel data quality for 1 layer for 89a
  dhnd = H5Dopen(fhnd, "Pixel Data Quality for 89A", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, pdq1_89a);
  if(ret < 0){
    printf("H5Dread error: Pixel Data Quality for 89A\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);

  // read array: pixel data quality for 1 layer for 89b
  dhnd = H5Dopen(fhnd, "Pixel Data Quality for 89B", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, pdq1_89b);
  if(ret < 0){
    printf("H5Dread error: Pixel Data Quality for 89B\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);

  // display sample
  printf("geo1_89a[scan=0][pixel=0]: %8.1f [mm/h] (PDQ:%3d)\n",geo1_89a[0][0],pdq1_89a[0][0]);
  printf("geo1_89b[scan=0][pixel=0]: %8.1f [mm/h] (PDQ:%3d)\n",geo1_89b[0][0],pdq1_89b[0][0]);

  // close
  ret = H5Fclose(fhnd);
  ret = H5close();
}
