      program main
      use hdf5
      implicit none
C include
      include 'amsr2time_f.h'
C intrinsic function
      integer(4) iargc ! pgf need explicitly declaration
C fixed value
      integer(4),parameter::LMT=2200 ! limit of NumberOfScans
      integer(4),parameter::AM2_DEF_SNUM_HI=486 ! high resolution pixel width
      integer(4),parameter::AM2_DEF_SNUM_LO=243 ! low  resolution pixel width
      character(len=*),parameter::MES=
     +'AMSR2 L2 HIGH RESOLUTION DATA FILE (HDF5) [PRC]'
C interface variable
      integer(4) i,j          ! loop variable
      integer(4) ret          ! return status
      character(len=512) buf  ! text buffer
      character(len=512) fn   ! filename
      integer(HID_T) fhnd     ! file handle
      integer(HID_T) ahnd     ! attribute handle
      integer(HID_T) atyp     ! attribute type
      integer(HID_T) dhnd     ! dataset handle
      integer(HID_T) shnd     ! dataspace handle
      integer(HSIZE_T) sz1(3) ! array size 1
      integer(HSIZE_T) sz2(3) ! array size 2
C meta data
C HDF5 FORTRAN LIBRARY CAN'T RETRIEVE VARIABLE STRING !
C calculate NumberOfScans from array size and OverlapScans
C use fixed value for OverlapScans
C     character(len=512) geo  ! GeophysicalName
      character(len=512) gid  ! GranuleID
C     character(len=512) tm1  ! ObservationStartDateTime
C     character(len=512) tm2  ! EquatorCrossingDateTime
C     character(len=512) tm3  ! ObservationEndDateTime
      integer(4) num          ! NumberOfScans
      integer(4) ovr          ! OverlapScans
      parameter(ovr=0)
C array buffer
      real(4) sca ! scale
      real(8) r8d1(LMT) ! array buffer for real(8) D1
C array data
      type(AM2_COMMON_SCANTIME) st(LMT) ! scantime
      real(4) lat89a(AM2_DEF_SNUM_HI,LMT)  ! lat for 89a
      real(4) lat89b(AM2_DEF_SNUM_HI,LMT)  ! lat for 89b
      real(4) lon89a(AM2_DEF_SNUM_HI,LMT)  ! lon for 89a
      real(4) lon89b(AM2_DEF_SNUM_HI,LMT)  ! lon for 89b
      real(4) geo1_89a(AM2_DEF_SNUM_HI,LMT) ! geophysical data layer 1 for 89a
      real(4) geo1_89b(AM2_DEF_SNUM_HI,LMT) ! geophysical data layer 1 for 89b
      integer(4) pdq1_89a(AM2_DEF_SNUM_HI,LMT) ! pixel data quality layer 1 for 89a
      integer(4) pdq1_89b(AM2_DEF_SNUM_HI,LMT) ! pixel data quality layer 1 for 89b
C argument check
      if(iargc().lt.1)then
        write(*,'(a)')'argument: '//MES
        call exit(1)
      endif
C set filename
      call getarg(1,fn)
      write(*,'(a,a)')'input file: ',fn(1:len_trim(fn))
C hdf5 initialize
      call H5open_f(ret)
      if(ret.lt.0)then
        write(*,'(a,i12)')'h5open_f error: ',ret
        call exit(1)
      endif
C open
      call H5Fopen_f(fn,H5F_ACC_RDONLY_F,fhnd,ret,H5P_DEFAULT_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Fopen error: ',fn(1:len_trim(fn))
        call exit(1)
      endif
C HDF5 FORTRAN LIBRARY CAN'T RETRIEVE VARIABLE STRING !
C retrieve GranuleID from filename
      i=index(fn,'/',.true.)
      gid=fn(i+1:i+41)
      write(*,'(a,a)')'GranuleID(RETRIEVE FROM FILENAME): '
     +,gid(1:len_trim(gid))
C check geophysical type by filename
      if(gid(30:32).ne.'PRC')then
        write(*,'(a)')'wrong input file: this program is for '//MES
        call exit(1)
      endif
C HDF5 FORTRAN LIBRARY CAN'T RETRIEVE VARIABLE STRING !
C calculate NumberOfScans from array size and OverlapScans
      call H5Dopen_f(fhnd,'Scan Time',dhnd,ret)
      call H5Dget_space_f(dhnd,shnd,ret)
      call H5Sget_simple_extent_dims_f(shnd,sz1,sz2,ret)
      if(ret.lt.0)then
        write(*,'(a)')'H5Sget_simple_extent_dims error: Scan Time'
        call exit(1)
      endif
      call H5Sclose_f(shnd,ret)
      call H5Dclose_f(dhnd,ret)
      num=sz1(1)-ovr*2
      write(*,'(a,i12)')'NumberOfScans(RETRIEVE BY ARRAY SIZE): ',num
      write(*,'(a,i12)')'OverlapScans(FIXED VALUE): ',ovr
C check limit
      if(num.gt.LMT)then
        write(*,'(a,i6)')'limit error: limit of NumberOfScans = ',LMT
        call exit(1)
      else
        write(*,'(a,i6)')'limit of NumberOfScans = ',LMT
      endif
C read array: scantime
      ! read
      call H5Dopen_f(fhnd,'Scan Time',dhnd,ret)
      sz1(1)=LMT
      call H5Dread_f(dhnd
     +,H5T_NATIVE_DOUBLE,r8d1,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: ','Scan Time'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
      ! convert
      call amsr2time(num,r8d1,st)
      ! sample display
      write(*,'(a,i4.4,"/",i2.2,"/",i2.2," ",i2.2,":",i2.2,":",i2.2)')
     +'time(scan=1): '
     +,st(1)%year
     +,st(1)%month
     +,st(1)%day
     +,st(1)%hour
     +,st(1)%minute
     +,st(1)%second
C read array: latlon for 89a
      ! read lat
      call H5Dopen_f(fhnd
     +,'Latitude of Observation Point for 89A',dhnd,ret)
      sz1(1)=LMT
      sz1(2)=AM2_DEF_SNUM_HI
      call H5Dread_f(dhnd
     +,H5T_NATIVE_REAL,lat89a,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: '
     +  ,'Latitude of Observation Point for 89A'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
      ! read lon
      call H5Dopen_f(fhnd
     +,'Longitude of Observation Point for 89A',dhnd,ret)
      sz1(1)=LMT
      sz1(2)=AM2_DEF_SNUM_HI
      call H5Dread_f(dhnd
     +,H5T_NATIVE_REAL,lon89a,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: '
     +  ,'Longitude of Observation Point for 89A'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
      ! sample display
      write(*,'(a,"(",f9.4,",",f9.4,")")')'latlon89a(pixel=1,scan=1): '
     +,lat89a(1,1),lon89a(1,1)
C read array: latlon for 89b
      ! read lat
      call H5Dopen_f(fhnd
     +,'Latitude of Observation Point for 89B',dhnd,ret)
      sz1(1)=LMT
      sz1(2)=AM2_DEF_SNUM_HI
      call H5Dread_f(dhnd
     +,H5T_NATIVE_REAL,lat89b,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: '
     +  ,'Latitude of Observation Point for 89B'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
      ! read lon
      call H5Dopen_f(fhnd
     +,'Longitude of Observation Point for 89B',dhnd,ret)
      sz1(1)=LMT
      sz1(2)=AM2_DEF_SNUM_HI
      call H5Dread_f(dhnd
     +,H5T_NATIVE_REAL,lon89b,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: '
     +  ,'Longitude of Observation Point for 89B'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
      ! sample display
      write(*,'(a,"(",f9.4,",",f9.4,")")')'latlon89b(pixel=1,scan=1): '
     +,lat89b(1,1),lon89b(1,1)
C read array: geophysical data for 1 layer for 89a
      call H5Dopen_f(fhnd,'Geophysical Data for 89A',dhnd,ret)
      ! get scale
      call H5Aopen_f(dhnd,'SCALE FACTOR',ahnd,ret)
      call H5Aread_f(ahnd,H5T_NATIVE_REAL,sca,sz1,ret)
      call H5Aclose_f(ahnd,ret)
      ! read
      sz1(1)=LMT
      sz1(2)=AM2_DEF_SNUM_HI
      call H5Dread_f(dhnd
     +,H5T_NATIVE_REAL,geo1_89a,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: '
     +  ,'Geophysical Data for 89A'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
      ! change scale
      do j=1,num
        do i=1,AM2_DEF_SNUM_HI
          if(geo1_89a(i,j).gt.-32767)geo1_89a(i,j)=geo1_89a(i,j)*sca
        enddo
      enddo
C read array: geophysical data for 1 layer for 89b
      call H5Dopen_f(fhnd,'Geophysical Data for 89B',dhnd,ret)
      ! get scale
      call H5Aopen_f(dhnd,'SCALE FACTOR',ahnd,ret)
      call H5Aread_f(ahnd,H5T_NATIVE_REAL,sca,sz1,ret)
      call H5Aclose_f(ahnd,ret)
      ! read
      sz1(1)=LMT
      sz1(2)=AM2_DEF_SNUM_HI
      call H5Dread_f(dhnd
     +,H5T_NATIVE_REAL,geo1_89b,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: '
     +  ,'Geophysical Data for 89B'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
      ! change scale
      do j=1,num
        do i=1,AM2_DEF_SNUM_HI
          if(geo1_89b(i,j).gt.-32767)geo1_89b(i,j)=geo1_89b(i,j)*sca
        enddo
      enddo
C read array: pixel data quality for 1 layer for 89a
      call H5Dopen_f(fhnd,'Pixel Data Quality for 89A',dhnd,ret)
      sz1(1)=LMT
      sz1(2)=AM2_DEF_SNUM_HI
      call H5Dread_f(dhnd
     +,H5T_NATIVE_INTEGER,pdq1_89a,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: '
     +  ,'Pixel Data Quality for 89A'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
C read array: pixel data quality for 1 layer for 89b
      call H5Dopen_f(fhnd,'Pixel Data Quality for 89B',dhnd,ret)
      sz1(1)=LMT
      sz1(2)=AM2_DEF_SNUM_HI
      call H5Dread_f(dhnd
     +,H5T_NATIVE_INTEGER,pdq1_89b,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: '
     +  ,'Pixel Data Quality for 89B'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
C display sample
      write(*,'(a,f8.1,a,i3,a)')'geo1_89a(pixel=1,scan=1): '
     +,geo1_89a(1,1),' [mm/h] (PDQ:',pdq1_89a(1,1),')'
      write(*,'(a,f8.1,a,i3,a)')'geo1_89b(pixel=1,scan=1): '
     +,geo1_89b(1,1),' [mm/h] (PDQ:',pdq1_89b(1,1),')'
C close
      call H5Fclose_f(fhnd,ret)
      call H5close_f(ret)
      end
