/*  This sample program can be used for SST Ver.4 products but may have problem   *
 *  to read SST Ver.3 or older.                                                   *
 *  You can use the program to read Ver.4 or older products of other geophysical  *
 *  parameters (except SST).                                                      */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "AMTK.h"

// fixed value
#define LMT 2200 // limit of NumberOfScans
#define MES "AMSR2 L2 LOW RESOLUTION DATA FILE (HDF5) [CLW SIC SMC SND SST SSW TPW ASW LST PWL TSI]"

int main(int argc, char *argv[]){
  // interface variable
  int i,j;       // loop variable
  int ret;       // return status
  char buf[512]; // text buffer
  void *vpnt;    // pointer to void
  char *fn;      // filename
  hid_t hnd;     // file handle

  // meta data
  char geo[512];  // GeophysicalName
  char gid[512];  // GranuleID
  char tm1[512];  // ObservationStartDateTime
  char tm2[512];  // EquatorCrossingDateTime
  char tm3[512];  // ObservationEndDateTime
  int num;        // NumberOfScans
  int ovr;        // OverlapScans

  // array data
  AM2_COMMON_SCANTIME st[LMT]; // scantime
  AM2_COMMON_LATLON ll[LMT][AM2_DEF_SNUM_LO]; // latlon
  float geo1[LMT][AM2_DEF_SNUM_LO]; // geophysical data layer 1
  float geo2[LMT][AM2_DEF_SNUM_LO]; // geophysical data layer 2
  float geo3[LMT][AM2_DEF_SNUM_LO]; // geophysical data layer 3
  unsigned char pdq1[LMT][AM2_DEF_SNUM_LO]; // pixel data quality layer 1
  unsigned char pdq2[LMT][AM2_DEF_SNUM_LO]; // pixel data quality layer 2
  unsigned char pdq3[LMT][AM2_DEF_SNUM_LO]; // pixel data quality layer 3
  unsigned char pdqtmp[LMT*3][AM2_DEF_SNUM_LO]; // pixel data quality temporary

  // argument check
  if(argc < 2){
    printf("argument: %s\n", MES);
    exit(1);
  }

  // set filename
  fn = argv[1];
  printf("input file: %s\n", fn);

  // open
  hnd=AMTK_openH5(fn);
  if(hnd<0){
    printf("AMTK_openH5 error: %s\n", fn);
    printf("amtk status: %d\n", hnd);
    exit(1);
  }

  // read meta: GeophysicalName
  vpnt=geo;
  ret=AMTK_getMetaDataName(hnd,"GeophysicalName",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: GeophysicalName\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("GeophysicalName: %s\n",geo);

  // read meta: GranuleID
  vpnt=gid;
  ret=AMTK_getMetaDataName(hnd,"GranuleID",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: GranuleID\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("GranuleID: %s\n",gid);

  // check geophysical type
  if(strncmp(gid+29,"CLW",3)!=0
  && strncmp(gid+29,"SIC",3)!=0
  && strncmp(gid+29,"SMC",3)!=0
  && strncmp(gid+29,"SND",3)!=0
  && strncmp(gid+29,"SST",3)!=0
  && strncmp(gid+29,"SSW",3)!=0
  && strncmp(gid+29,"TPW",3)!=0
  && strncmp(gid+29,"ASW",3)!=0
  && strncmp(gid+29,"LST",3)!=0
  && strncmp(gid+29,"PWL",3)!=0
  && strncmp(gid+29,"TSI",3)!=0
  ){
    printf("wrong input file: this program is for %s\n", MES);
    exit(1);
  }

  // read meta: ObservationStartDateTime
  vpnt=tm1;
  ret=AMTK_getMetaDataName(hnd,"ObservationStartDateTime",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: ObservationStartDateTime\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("ObservationStartDateTime: %s\n",tm1);

  // read meta: EquatorCrossingDateTime
  vpnt=tm2;
  ret=AMTK_getMetaDataName(hnd,"EquatorCrossingDateTime",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: EquatorCrossingDateTime\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("EquatorCrossingDateTime: %s\n",tm2);

  // read meta: ObservationEndDateTime
  vpnt=tm3;
  ret=AMTK_getMetaDataName(hnd,"ObservationEndDateTime",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: ObservationEndDateTime\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("ObservationEndDateTime: %s\n",tm3);

  // read meta: NumberOfScans
  vpnt=buf;
  ret=AMTK_getMetaDataName(hnd,"NumberOfScans",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: NumberOfScans\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  num=atoi(buf);
  printf("NumberOfScans: %d\n",num);

  // check limit
  if(num>LMT){
    printf("limit error: limit of NumberOfScans = %d\n",LMT);
    exit(1);
  }
  else{
    printf("limit of NumberOfScans = %d\n",LMT);
  }

  // read meta: OverlapScans
  vpnt=buf;
  ret=AMTK_getMetaDataName(hnd,"OverlapScans",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: OverlapScans\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  ovr=atoi(buf);
  printf("OverlapScans: %d\n",ovr);

  // read array: scantime
  vpnt=st;
  ret=AMTK_getScanTime(hnd,1,num,(AM2_COMMON_SCANTIME **)&vpnt);
  if(ret<0){
    printf("AMTK_getScanTime error.\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("time[scan=0]: %04d/%02d/%02d %02d:%02d:%02d\n"
  , st[0].year
  , st[0].month
  , st[0].day
  , st[0].hour
  , st[0].minute
  , st[0].second
  );

  // read array: latlon
  vpnt=ll;
  ret=AMTK_getLatLon(hnd,(AM2_COMMON_LATLON **)&vpnt,1,num,AM2_LATLON_L2_LO);
  if(ret<0){
    printf("AMTK_getLatLon error: AM2_LATLON_L2_LO\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("latlon[scan=0][pixel=0]: (%9.4f,%9.4f)\n", ll[0][0].lat, ll[0][0].lon);

  // read array: geophysical data for 1 layer
  if(strncmp(gid+29,"SND",3)!=0 && strncmp(gid+29,"SST",3)!=0 && strncmp(gid+29,"TSI",3)!=0){
    vpnt=geo1;
    ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_SWATH_GEO1);
    if(ret<0){
      printf("AMTK_get_SwathFloat error: AM2_SWATH_GEO1\n");
      printf("amtk status: %d\n",ret);
      exit(1);
    }
  }

  // read array: geophysical data for 2 layer
  if(strncmp(gid+29,"SND",3)==0 || strncmp(gid+29,"TSI",3)==0){
    // layer 1
    vpnt=geo1;
    ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_SWATH_GEO1);
    if(ret<0){
      printf("AMTK_get_SwathFloat error: AM2_SWATH_GEO1\n");
      printf("amtk status: %d\n",ret);
      exit(1);
    }
    // layer 2
    vpnt=geo2;
    ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_SWATH_GEO2);
    if(ret<0){
      printf("AMTK_get_SwathFloat error: AM2_SWATH_GEO2\n");
      printf("amtk status: %d\n",ret);
      exit(1);
    }
  }

  // read array: geophysical data for 3 layer
  if(strncmp(gid+29,"SST",3)==0){
    // layer 1
    vpnt=geo1;
    ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_SWATH_GEO1);
    if(ret<0){
      printf("AMTK_get_SwathFloat error: AM2_SWATH_GEO1\n");
      printf("amtk status: %d\n",ret);
      exit(1);
    }
    // layer 2
    vpnt=geo2;
    ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_SWATH_GEO2);
    if(ret<0){
      printf("AMTK_get_SwathFloat error: AM2_SWATH_GEO2\n");
      printf("amtk status: %d\n",ret);
      exit(1);
    }
    // layer 3
    vpnt=geo3;
    ret=AMTK_get_SwathFloat(hnd,(float **)&vpnt,1,num,AM2_SWATH_GEO3);
    if(ret<0){
      printf("AMTK_get_SwathFloat error: AM2_SWATH_GE3\n");
      printf("amtk status: %d\n",ret);
      exit(1);
    }
  }

 // read array: pixel data quality for 1 layer
  if(strncmp(gid+29,"SND",3)!=0 && strncmp(gid+29,"SST",3)!=0 && strncmp(gid+29,"TSI",3)!=0){
    vpnt=pdq1;
    ret=AMTK_get_SwathUChar(hnd,(unsigned char **)&vpnt,1,num,AM2_PIX_QUAL);
    if(ret<0){
      printf("AMTK_get_SwathUChar error: AM2_PIX_QUAL\n");
      printf("amtk status: %d\n",ret);
      exit(1);
    }
  }

  // read array: pixel data quality for 2 layer
  if(strncmp(gid+29,"SND",3)==0 || strncmp(gid+29,"TSI",3)==0){
    // read
    vpnt=pdqtmp;
    ret=AMTK_get_SwathUChar(hnd,(unsigned char **)&vpnt,1,num,AM2_PIX_QUAL);
    if(ret<0){
      printf("AMTK_get_SwathUChar error: AM2_PIX_QUAL\n");
      printf("amtk status: %d\n",ret);
      exit(1);
    }
    // separate
    for(j=0;j<num;++j){
      for(i=0;i<AM2_DEF_SNUM_LO;++i){
        pdq1[j][i]=pdqtmp[num*0+j][i];
        pdq2[j][i]=pdqtmp[num*1+j][i];
      }
    }
  }

  // read array: pixel data quality for 3 layer
  if(strncmp(gid+29,"SST",3)==0){
    // read
    vpnt=pdqtmp;
    ret=AMTK_get_SwathUChar(hnd,(unsigned char **)&vpnt,1,num,AM2_PIX_QUAL);
    if(ret<0){
      printf("AMTK_get_SwathUChar error: AM2_PIX_QUAL\n");
      printf("amtk status: %d\n",ret);
      exit(1);
    }
    // separate
    for(j=0;j<num;++j){
      for(i=0;i<AM2_DEF_SNUM_LO;++i){
        pdq1[j][i]=pdqtmp[num*0+j][i];
        pdq2[j][i]=pdqtmp[num*1+j][i];
        pdq3[j][i]=pdqtmp[num*2+j][i];
      }
    }
  }

  // display sample
  if(strncmp(gid+29,"CLW",3)==0){
    printf("geo1[scan=0][pixel=0]: %10.3f [Kg/m2] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"SIC",3)==0){
    printf("geo1[scan=0][pixel=0]: %8.1f [%%] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"SMC",3)==0){
    printf("geo1[scan=0][pixel=0]: %8.1f [%%] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"SND",3)==0){
    printf("geo1[scan=0][pixel=0]: %8.1f [cm] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
    printf("geo2[scan=0][pixel=0]: %8.1f [cm] (PDQ:%3d)\n",geo2[0][0],pdq2[0][0]);
  }
  if(strncmp(gid+29,"SST",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
    printf("geo2[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo2[0][0],pdq2[0][0]);
    printf("geo3[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo3[0][0],pdq3[0][0]);
  }
  if(strncmp(gid+29,"SSW",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [m/s] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"TPW",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [Kg/m2] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"ASW",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [m/s] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"LST",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"PWL",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"TSI",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
    printf("geo2[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo2[0][0],pdq2[0][0]);
  }

  // close
  ret=AMTK_closeH5(hnd);
}
