C   This sample program can be used for SST Ver.4 products but may have
C   problem to read SST Ver.3 or older.
C   You can use the program to read Ver.4 or older products of other
C   geophysical parameters (except SST).


      program main
      implicit none
C include
      include 'AMTK_f.h'
C intrinsic function
      integer(4) iargc ! pgf need explicitly declaration
C fixed value
      integer(4),parameter::LMT=2200 ! limit of NumberOfScans
      character(len=*),parameter::MES=
     +'AMSR2 L2 LOW RESOLUTION DATA FILE'
     +//' (HDF5) [CLW SIC SMC SND SST SSW TPW ASW LST PWL TSI]'
C interface variable
      integer(4) i,j         ! loop variable
      integer(4) ret         ! return status
      character(len=512) buf ! text buffer
      character(len=512) fn  ! filename
      integer(4) hnd         ! file handle
C meta data
      character(len=512) geo  ! GeophysicalName
      character(len=512) gid  ! GranuleID
      character(len=512) tm1  ! ObservationStartDateTime
      character(len=512) tm2  ! EquatorCrossingDateTime
      character(len=512) tm3  ! ObservationEndDateTime
      integer(4) num          ! NumberOfScans
      integer(4) ovr          ! OverlapScans
C array data
      type(AM2_COMMON_SCANTIME) st(LMT) ! scantime
      type(AM2_COMMON_LATLON) ll(AM2_DEF_SNUM_LO,LMT) ! latlon
      real(4) geo1(AM2_DEF_SNUM_LO,LMT) ! geophysical data layer 1
      real(4) geo2(AM2_DEF_SNUM_LO,LMT) ! geophysical data layer 2
      real(4) geo3(AM2_DEF_SNUM_LO,LMT) ! geophysical data layer 3
      integer(4) pdq1(AM2_DEF_SNUM_LO,LMT) ! pixel data quality layer 1
      integer(4) pdq2(AM2_DEF_SNUM_LO,LMT) ! pixel data quality layer 2
      integer(4) pdq3(AM2_DEF_SNUM_LO,LMT) ! pixel data quality layer 3
      integer(1) pdqtmp(AM2_DEF_SNUM_LO,LMT*3) ! pixel data quality temporary
C argument check
      if(iargc().lt.1)then
        write(*,'(a)')'argument: '//MES
        call exit(1)
      endif
C set filename
      call getarg(1,fn)
      write(*,'(a,a)')'input file: ',fn(1:len_trim(fn))
C open
      hnd=AMTK_openH5(fn)
      if(hnd.lt.0)then
        write(*,'(a,a)')'AMTK_openH5 error: ',fn(1:len_trim(fn))
        write(*,'(a,i12)')'amtk status: ',hnd
        call exit(1)
      endif
C read meta: GeophysicalName
      ret=AMTK_getMetaDataName(hnd,'GeophysicalName',geo)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getMetaDataName error: GeophysicalName'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'GeophysicalName: ',geo(1:len_trim(geo))
C read meta: GranuleID
      ret=AMTK_getMetaDataName(hnd,'GranuleID',gid)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getMetaDataName error: GranuleID'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'GranuleID: ',gid(1:len_trim(gid))
C check geophysical type
      if(  gid(30:32).ne.'CLW'
     +.and.gid(30:32).ne.'SIC'
     +.and.gid(30:32).ne.'SMC'
     +.and.gid(30:32).ne.'SND'
     +.and.gid(30:32).ne.'SST'
     +.and.gid(30:32).ne.'SSW'
     +.and.gid(30:32).ne.'TPW'
     +.and.gid(30:32).ne.'ASW'
     +.and.gid(30:32).ne.'LST'
     +.and.gid(30:32).ne.'PWL'
     +.and.gid(30:32).ne.'TSI')then
        write(*,'(a)')'wrong input file: this program is for '//MES
        call exit(1)
      endif
C read meta: ObservationStartDateTime
      ret=AMTK_getMetaDataName(hnd,'ObservationStartDateTime',tm1)
      if(ret.lt.0)then
        write(*,'(a,a)')'AMTK_getMetaDataName error: '
     +  ,'ObservationStartDateTime'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'ObservationStartDateTime: ',tm1(1:len_trim(tm1))
C read meta: EquatorCrossingDateTime
      ret=AMTK_getMetaDataName(hnd,'EquatorCrossingDateTime',tm2)
      if(ret.lt.0)then
        write(*,'(a,a)')'AMTK_getMetaDataName error: '
     +  ,'EquatorCrossingDateTime'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'EquatorCrossingDateTime: ',tm2(1:len_trim(tm2))
C read meta: ObservationEndDateTime
      ret=AMTK_getMetaDataName(hnd,'ObservationEndDateTime',tm3)
      if(ret.lt.0)then
        write(*,'(a,a)')'AMTK_getMetaDataName error: '
     +  ,'ObservationEndDateTime'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'ObservationEndDateTime: ',tm3(1:len_trim(tm3))
C read meta: NumberOfScans
      ret=AMTK_getMetaDataName(hnd,'NumberOfScans',buf)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getMetaDataName error: NumberOfScans'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      read(buf(1:ret),*)num
      write(*,'(a,i12)')'NumberOfScans: ',num
C check limit
      if(num.gt.LMT)then
        write(*,'(a,i6)')'limit error: limit of NumberOfScans = ',LMT
        call exit(1)
      else
        write(*,'(a,i6)')'limit of NumberOfScans = ',LMT
      endif
C read meta: OverlapScans
      ret=AMTK_getMetaDataName(hnd,'OverlapScans',buf)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getMetaDataName error: OverlapScans'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      read(buf(1:ret),*)ovr
      write(*,'(a,i12)')'OverlapScans: ',ovr
C read array: scantime
      ret=AMTK_getScanTime(hnd,1,num,st)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getScanTime error.'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,i4.4,"/",i2.2,"/",i2.2," ",i2.2,":",i2.2,":",i2.2)')
     +'time(scan=1): '
     +,st(1)%year
     +,st(1)%month
     +,st(1)%day
     +,st(1)%hour
     +,st(1)%minute
     +,st(1)%second
C read array: latlon
      ret=AMTK_getLatLon(hnd,ll,1,num,AM2_LATLON_L2_LO)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getLatLon error: AM2_LATLON_L2_LO'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,"(",f9.4,",",f9.4,")")')'latlon(pixel=1,scan=1): '
     +,ll(1,1)%lat,ll(1,1)%lon
C read array: geophysical data for 1 layer
      if((gid(30:32).ne.'SND').and.(gid(30:32).ne.'SST')
     +   .and.(gid(30:32).ne.'TSI'))then
        ret=AMTK_get_SwathFloat(hnd,geo1,1,num,AM2_SWATH_GEO1)
        if(ret.lt.0)then
          write(*,'(a)')'AMTK_get_SwathFloat error: AM2_SWATH_GEO1'
          write(*,'(a,i12)')'amtk status: ',ret
          call exit(1)
        endif
      endif
C read array: geophysical data for 2 layer
      if((gid(30:32).eq.'SND').or.(gid(30:32).eq.'TSI'))then
        ! layer 1
        ret=AMTK_get_SwathFloat(hnd,geo1,1,num,AM2_SWATH_GEO1)
        if(ret.lt.0)then
          write(*,'(a)')'AMTK_get_SwathFloat error: AM2_SWATH_GEO1'
          write(*,'(a,i12)')'amtk status: ',ret
          call exit(1)
        endif
        ! layer 2
        ret=AMTK_get_SwathFloat(hnd,geo2,1,num,AM2_SWATH_GEO2)
        if(ret.lt.0)then
          write(*,'(a)')'AMTK_get_SwathFloat error: AM2_SWATH_GEO2'
          write(*,'(a,i12)')'amtk status: ',ret
          call exit(1)
        endif
      endif
C read array: geophysical data for 3 layer
      if((gid(30:32).eq.'SST'))then
        ! layer 1
        ret=AMTK_get_SwathFloat(hnd,geo1,1,num,AM2_SWATH_GEO1)
        if(ret.lt.0)then
          write(*,'(a)')'AMTK_get_SwathFloat error: AM2_SWATH_GEO1'
          write(*,'(a,i12)')'amtk status: ',ret
          call exit(1)
        endif
        ! layer 2
        ret=AMTK_get_SwathFloat(hnd,geo2,1,num,AM2_SWATH_GEO2)
        if(ret.lt.0)then
          write(*,'(a)')'AMTK_get_SwathFloat error: AM2_SWATH_GEO2'
          write(*,'(a,i12)')'amtk status: ',ret
          call exit(1)
        endif
        ! layer 3
        ret=AMTK_get_SwathFloat(hnd,geo3,1,num,AM2_SWATH_GEO3)
        if(ret.lt.0)then
          write(*,'(a)')'AMTK_get_SwathFloat error: AM2_SWATH_GEO3'
          write(*,'(a,i12)')'amtk status: ',ret
          call exit(1)
        endif
      endif
C read array: pixel data quality for 1 layer
      if((gid(30:32).ne.'SND').and.(gid(30:32).ne.'SST')
     +    .and.(gid(30:32).ne.'TSI'))then
        ! read
        ret=AMTK_get_SwathUChar(hnd,pdqtmp,1,num,AM2_PIX_QUAL)
        if(ret.lt.0)then
          write(*,'(a)')'AMTK_get_SwathUChar error: AM2_PIX_QUAL'
          write(*,'(a,i12)')'amtk status: ',ret
          call exit(1)
        endif
        ! convert signed to unsigned
        do j=1,num
          do i=1,AM2_DEF_SNUM_LO
            pdq1(i,j)=pdqtmp(i,j)
            if(pdq1(i,j).ge.0)then
              pdq1(i,j)=pdq1(i,j)
            else
              pdq1(i,j)=pdq1(i,j)+256
            endif
          enddo
        enddo
      endif
C read array: pixel data quality for 2 layer
      if((gid(30:32).eq.'SND').or.(gid(30:32).eq.'TSI'))then
        ! read
        ret=AMTK_get_SwathUChar(hnd,pdqtmp,1,num,AM2_PIX_QUAL)
        if(ret.lt.0)then
          write(*,'(a)')'AMTK_get_SwathUChar error: AM2_PIX_QUAL'
          write(*,'(a,i12)')'amtk status: ',ret
          call exit(1)
        endif
        ! separate & convert signed to unsigned
        do j=1,num
          do i=1,AM2_DEF_SNUM_LO
            pdq1(i,j)=pdqtmp(i,num*0+j)
            pdq2(i,j)=pdqtmp(i,num*1+j)
            if(pdq1(i,j).ge.0)then
              pdq1(i,j)=pdq1(i,j)
            else
              pdq1(i,j)=pdq1(i,j)+256
            endif
            if(pdq2(i,j).ge.0)then
              pdq2(i,j)=pdq2(i,j)
            else
              pdq2(i,j)=pdq2(i,j)+256
            endif
          enddo
        enddo
      endif
C read array: pixel data quality for 3 layer
      if((gid(30:32).eq.'SST'))then
        ! read
        ret=AMTK_get_SwathUChar(hnd,pdqtmp,1,num,AM2_PIX_QUAL)
        if(ret.lt.0)then
          write(*,'(a)')'AMTK_get_SwathUChar error: AM2_PIX_QUAL'
          write(*,'(a,i12)')'amtk status: ',ret
          call exit(1)
        endif
        ! separate & convert signed to unsigned
        do j=1,num
          do i=1,AM2_DEF_SNUM_LO
            pdq1(i,j)=pdqtmp(i,num*0+j)
            pdq2(i,j)=pdqtmp(i,num*1+j)
            pdq3(i,j)=pdqtmp(i,num*2+j)
            if(pdq1(i,j).ge.0)then
              pdq1(i,j)=pdq1(i,j)
            else
              pdq1(i,j)=pdq1(i,j)+256
            endif
            if(pdq2(i,j).ge.0)then
              pdq2(i,j)=pdq2(i,j)
            else
              pdq2(i,j)=pdq2(i,j)+256
            endif
            if(pdq3(i,j).ge.0)then
              pdq3(i,j)=pdq3(i,j)
            else
              pdq3(i,j)=pdq3(i,j)+256
            endif
          enddo
        enddo
      endif
C display sample
      if(gid(30:32).eq.'CLW')then
        write(*,'(a,f10.3,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [Kg/m2] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'SIC')then
        write(*,'(a,f8.1,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [%] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'SMC')then
        write(*,'(a,f8.1,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [%] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'SND')then
        write(*,'(a,f8.1,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [cm] (PDQ:',pdq1(1,1),')'
        write(*,'(a,f8.1,a,i3,a)')'geo2(pixel=1,scan=1): '
     +  ,geo2(1,1),' [cm] (PDQ:',pdq2(1,1),')'
      endif
      if(gid(30:32).eq.'SST')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [C] (PDQ:',pdq1(1,1),')'
        write(*,'(a,f9.2,a,i3,a)')'geo2(pixel=1,scan=1): '
     +  ,geo2(1,1),' [C] (PDQ:',pdq2(1,1),')'
        write(*,'(a,f9.2,a,i3,a)')'geo3(pixel=1,scan=1): '
     +  ,geo3(1,1),' [C] (PDQ:',pdq3(1,1),')'
      endif
      if(gid(30:32).eq.'SSW')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [m/s] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'TPW')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [Kg/m2] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'ASW')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [m/s] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'LST')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [C] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'PWL')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [C] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'TSI')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [C] (PDQ:',pdq1(1,1),')'
        write(*,'(a,f9.2,a,i3,a)')'geo2(pixel=1,scan=1): '
     +  ,geo2(1,1),' [C] (PDQ:',pdq2(1,1),')'
      endif
C close
      ret=AMTK_closeH5(hnd)
      end
