/*  This sample program can be used for SST Ver.4 products but may have problem   *
 *  to read SST Ver.3 or older.                                                   *
 *  You can use the program to read Ver.4 or older products of other geophysical  *
 *  parameters (except SST).                                                      */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "hdf5.h"
#include "amsr2time.h"

// fixed value
#define LMT 2200 // limit of NumberOfScans
#define AM2_DEF_SNUM_HI 486 // high resolution pixel width
#define AM2_DEF_SNUM_LO 243 // low  resolution pixel width
#define MES "AMSR2 L2 LOW RESOLUTION DATA FILE (HDF5) [CLW SIC SMC SND SST SSW TPW ASW LST PWL TSI]"

int main(int argc, char *argv[]){
  // interface variable
  int i,j;          // loop variable
  herr_t ret;       // return status
  char *buf = NULL; // text buffer
  char *fn  = NULL; // filename
  hid_t fhnd;       // file handle
  hid_t ahnd;       // attribute handle
  hid_t atyp;       // attribute type
  hid_t dhnd;       // dataset handle

  // meta data
  char *geo = NULL; // GeophysicalName
  char *gid = NULL; // GranuleID
  char *tm1 = NULL; // ObservationStartDateTime
  char *tm2 = NULL; // EquatorCrossingDateTime
  char *tm3 = NULL; // ObservationEndDateTime
  int num;          // NumberOfScans
  int ovr;          // OverlapScans

  // array buffer
  float sca; // scale
  double r8d1[LMT]; // array buffer for double D1

  // array data
  AM2_COMMON_SCANTIME st[LMT]; // scantime
  float lat[LMT][AM2_DEF_SNUM_LO]; // lat
  float lon[LMT][AM2_DEF_SNUM_LO]; // lon
  float geo1[LMT][AM2_DEF_SNUM_LO]; // geophysical data layer 1
  float geo2[LMT][AM2_DEF_SNUM_LO]; // geophysical data layer 2
  float geo3[LMT][AM2_DEF_SNUM_LO]; // geophysical data layer 3
  float geotmp[LMT*3][AM2_DEF_SNUM_LO]; // geophysical data temporary
  unsigned char pdq1[LMT][AM2_DEF_SNUM_LO]; // pixel data quality layer 1
  unsigned char pdq2[LMT][AM2_DEF_SNUM_LO]; // pixel data quality layer 2
  unsigned char pdq3[LMT][AM2_DEF_SNUM_LO]; // pixel data quality layer 3
  unsigned char pdqtmp[LMT*3][AM2_DEF_SNUM_LO]; // pixel data quality temporary

  // argument check
  if(argc < 2){
    printf("argument: %s\n", MES);
    exit(1);
  }

  // set filename
  fn = argv[1];
  printf("input file: %s\n", fn);

  // hdf5 initialize
  ret = H5open();
  if(ret < 0){
    printf("h5open error: %d\n" ,ret);
    exit(1);
  }

  // open
  fhnd = H5Fopen(fn, H5F_ACC_RDONLY, H5P_DEFAULT);
  if(fhnd < 0){
    printf("H5Fopen error: %s\n", fn);
    exit(1);
  }

  // read meta: GeophysicalName
  ahnd = H5Aopen(fhnd, "GeophysicalName", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &geo);
  if(ret < 0){
    printf("H5Aread error: GeophysicalName\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("GeophysicalName: %s\n", geo);

  // read meta: GranuleID
  ahnd = H5Aopen(fhnd, "GranuleID", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &gid);
  if(ret < 0){
    printf("H5Aread error: GranuleID\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("GranuleID: %s\n", gid);

  // check geophysical type
  if(strncmp(gid+29,"CLW",3)!=0
  && strncmp(gid+29,"SIC",3)!=0
  && strncmp(gid+29,"SMC",3)!=0
  && strncmp(gid+29,"SND",3)!=0
  && strncmp(gid+29,"SST",3)!=0
  && strncmp(gid+29,"SSW",3)!=0
  && strncmp(gid+29,"TPW",3)!=0
  && strncmp(gid+29,"ASW",3)!=0
  && strncmp(gid+29,"LST",3)!=0
  && strncmp(gid+29,"PWL",3)!=0
  && strncmp(gid+29,"TSI",3)!=0
  ){
    printf("wrong input file: this program is for %s\n", MES);
    exit(1);
  }

  // read meta: ObservationStartDateTime
  ahnd = H5Aopen(fhnd, "ObservationStartDateTime", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &tm1);
  if(ret < 0){
    printf("H5Aread error: ObservationStartDateTime\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("ObservationStartDateTime: %s\n", tm1);

  // read meta: EquatorCrossingDateTime 
  ahnd = H5Aopen(fhnd, "EquatorCrossingDateTime", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &tm2);
  if(ret < 0){
    printf("H5Aread error: EquatorCrossingDateTime\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("EquatorCrossingDateTime: %s\n", tm2);

  // read meta: ObservationEndDateTime
  ahnd = H5Aopen(fhnd, "ObservationEndDateTime", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &tm3);
  if(ret < 0){
    printf("H5Aread error: ObservationEndDateTime\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("ObservationEndDateTime: %s\n", tm3);

  // read meta: NumberOfScans
  ahnd = H5Aopen(fhnd, "NumberOfScans", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &buf);
  if(ret < 0){
    printf("H5Aread error: NumberOfScans\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  num = atoi(buf);
  printf("NumberOfScans: %d\n", num);

  // check limit
  if(num > LMT){
    printf("limit error: limit of NumberOfScans = %d\n", LMT);
    exit(1);
  }
  else{
    printf("limit of NumberOfScans = %d\n", LMT);
  }

  // read meta: OverlapScans
  ahnd = H5Aopen(fhnd, "OverlapScans", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &buf);
  if(ret < 0){
    printf("H5Aread error: OverlapScans\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  ovr = atoi(buf);
  printf("OverlapScans: %d\n", ovr);

  // read array: scantime
  dhnd = H5Dopen(fhnd, "Scan Time", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_DOUBLE, H5S_ALL, H5S_ALL, H5P_DEFAULT, r8d1);
  if(ret < 0){
    printf("H5Dread error: Scan Time\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // convert
  amsr2time_(&num, r8d1, st);
  // sample display
  printf("time[scan=0]: %04d/%02d/%02d %02d:%02d:%02d\n"
  , st[0].year
  , st[0].month
  , st[0].day
  , st[0].hour
  , st[0].minute
  , st[0].second
  );

  // read array: latlon
  // read lat
  dhnd = H5Dopen(fhnd ,"Latitude of Observation Point", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lat);
  if(ret < 0){
    printf("H5Dread error: Latitude of Observation Point\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // read lon
  dhnd = H5Dopen(fhnd ,"Longitude of Observation Point", H5P_DEFAULT);
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, lon);
  if(ret < 0){
    printf("H5Dread error: Longitude of Observation Point\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // sample display
  printf("latlon[scan=0][pixel=0]: (%9.4f,%9.4f)\n", lat[0][0], lon[0][0]);

  // read array: geophysical data for 1 layer
  if(strncmp(gid+29,"SND",3)!=0 && strncmp(gid+29,"SST",3)!=0 && strncmp(gid+29,"TSI",3)!=0){
    dhnd = H5Dopen(fhnd, "Geophysical Data", H5P_DEFAULT);
    // get scale
    ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT);
    ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);
    ret = H5Aclose(ahnd);
    // read
    ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, geo1);
    if(ret < 0){
      printf("H5Dread error: Geophysical Data\n");
      exit(1);
    }
    ret = H5Dclose(dhnd);
    // change scale
    for(j = 0; j < num; ++j){
      for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
        if(geo1[j][i] > -32767) geo1[j][i] = geo1[j][i] * sca;
      }
    }
  }

  // read array: geophysical data for 2 layer
  if(strncmp(gid+29,"SND",3)==0 || strncmp(gid+29,"TSI",3)==0){
    dhnd = H5Dopen(fhnd, "Geophysical Data", H5P_DEFAULT);
    // get scale
    ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT);
    ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);
    ret = H5Aclose(ahnd);
    // read
    ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, geotmp);
    if(ret < 0){
      printf("H5Dread error: Geophysical Data\n");
      exit(1);
    }
    ret = H5Dclose(dhnd);
    // separate & change scale
    for(j = 0; j < num; ++j){
      for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
        geo1[j][i] = geotmp[j*2][i*2+0];
        geo2[j][i] = geotmp[j*2][i*2+1];
        if(geo1[j][i] > -32767) geo1[j][i] = geo1[j][i] * sca;
        if(geo2[j][i] > -32767) geo2[j][i] = geo2[j][i] * sca;
      }
    }
  }

  // read array: geophysical data for 3 layer
  if(strncmp(gid+29,"SST",3)==0){
    dhnd = H5Dopen(fhnd, "Geophysical Data", H5P_DEFAULT);
    // get scale
    ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT);
    ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);
    ret = H5Aclose(ahnd);
    // read
    ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, geotmp);
    if(ret < 0){
      printf("H5Dread error: Geophysical Data\n");
      exit(1);
    }
    ret = H5Dclose(dhnd);
    // separate & change scale
    for(j = 0; j < num; ++j){
      for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
        geo1[j][i] = geotmp[j*3][i*3+0];
        geo2[j][i] = geotmp[j*3][i*3+1];
        geo3[j][i] = geotmp[j*3][i*3+2];
        if(geo1[j][i] > -32767) geo1[j][i] = geo1[j][i] * sca;
        if(geo2[j][i] > -32767) geo2[j][i] = geo2[j][i] * sca;
        if(geo3[j][i] > -32767) geo3[j][i] = geo3[j][i] * sca;
      }
    }
  }

  // read array: pixel data quality for 1 layer
  if(strncmp(gid+29,"SND",3)!=0 && strncmp(gid+29,"SST",3)!=0 && strncmp(gid+29,"TSI",3)!=0){
    dhnd = H5Dopen(fhnd, "Pixel Data Quality", H5P_DEFAULT);
    ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, pdq1);
    if(ret < 0){
      printf("H5Dread error: Pixel Data Quality\n");
      exit(1);
    }
    ret = H5Dclose(dhnd);
  }

  // read array: pixel data quality for 2 layer
  if(strncmp(gid+29,"SND",3)==0 || strncmp(gid+29,"TSI",3)==0){
    // read
    dhnd = H5Dopen(fhnd, "Pixel Data Quality", H5P_DEFAULT);
    ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, pdqtmp);
    if(ret < 0){
      printf("H5Dread error: Pixel Data Quality\n");
      exit(1);
    }
    ret = H5Dclose(dhnd);
    // separate
    for(j = 0; j < num; ++j){
      for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
        pdq1[j][i] = pdqtmp[num*0+j][i];
        pdq2[j][i] = pdqtmp[num*1+j][i];
      }
    }
  }

  // read array: pixel data quality for 3 layer
  if(strncmp(gid+29,"SST",3)==0){
    // read
    dhnd = H5Dopen(fhnd, "Pixel Data Quality", H5P_DEFAULT);
    ret = H5Dread(dhnd, H5T_NATIVE_UCHAR, H5S_ALL, H5S_ALL, H5P_DEFAULT, pdqtmp);
    if(ret < 0){
      printf("H5Dread error: Pixel Data Quality\n");
      exit(1);
    }
    ret = H5Dclose(dhnd);
    // separate
    for(j = 0; j < num; ++j){
      for(i = 0; i < AM2_DEF_SNUM_LO; ++i){
        pdq1[j][i] = pdqtmp[num*0+j][i];
        pdq2[j][i] = pdqtmp[num*1+j][i];
        pdq3[j][i] = pdqtmp[num*2+j][i];
      }
    }
  }

  // display sample
  if(strncmp(gid+29,"CLW",3)==0){
    printf("geo1[scan=0][pixel=0]: %10.3f [Kg/m2] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"SIC",3)==0){
    printf("geo1[scan=0][pixel=0]: %8.1f [%%] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"SMC",3)==0){
    printf("geo1[scan=0][pixel=0]: %8.1f [%%] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"SND",3)==0){
    printf("geo1[scan=0][pixel=0]: %8.1f [cm] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
    printf("geo2[scan=0][pixel=0]: %8.1f [cm] (PDQ:%3d)\n",geo2[0][0],pdq2[0][0]);
  }
  if(strncmp(gid+29,"SST",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
    printf("geo2[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo2[0][0],pdq2[0][0]);
    printf("geo3[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo3[0][0],pdq3[0][0]);
  }
  if(strncmp(gid+29,"SSW",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [m/s] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"TPW",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [Kg/m2] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"ASW",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [m/s] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"LST",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"PWL",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
  }
  if(strncmp(gid+29,"TSI",3)==0){
    printf("geo1[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo1[0][0],pdq1[0][0]);
    printf("geo2[scan=0][pixel=0]: %9.2f [C] (PDQ:%3d)\n",geo2[0][0],pdq2[0][0]);
  }

  // close
  ret = H5Fclose(fhnd);
  ret = H5close();
}
