C   This sample program can be used for SST Ver.4 products but may have
C   problem to read SST Ver.3 or older.
C   You can use the program to read Ver.4 or older products of other
C   geophysical parameters (except SST).

      program main
      use hdf5
      implicit none
C include
      include 'amsr2time_f.h'
C intrinsic function
      integer(4) iargc ! pgf need explicitly declaration
C fixed value
      integer(4),parameter::LMT=2200 ! limit of NumberOfScans
      integer(4),parameter::AM2_DEF_SNUM_HI=486 ! high resolution pixel width
      integer(4),parameter::AM2_DEF_SNUM_LO=243 ! low  resolution pixel width
      character(len=*),parameter::MES=
     +'AMSR2 L2 LOW RESOLUTION DATA FILE'
     +//' (HDF5) [CLW SIC SMC SND SST SSW TPW ASW LST PWL TSI]'
C interface variable
      integer(4) i,j          ! loop variable
      integer(4) ret          ! return status
      character(len=512) buf  ! text buffer
      character(len=512) fn   ! filename
      integer(HID_T) fhnd     ! file handle
      integer(HID_T) ahnd     ! attribute handle
      integer(HID_T) atyp     ! attribute type
      integer(HID_T) dhnd     ! dataset handle
      integer(HID_T) shnd     ! dataspace handle
      integer(HSIZE_T) sz1(3) ! array size 1
      integer(HSIZE_T) sz2(3) ! array size 2
C meta data
C HDF5 FORTRAN LIBRARY CAN'T RETRIEVE VARIABLE STRING !
C calculate NumberOfScans from array size and OverlapScans
C use fixed value for OverlapScans
C retrieve GranuleID from filename
C     character(len=512) geo  ! GeophysicalName
      character(len=512) gid  ! GranuleID
C     character(len=512) tm1  ! ObservationStartDateTime
C     character(len=512) tm2  ! EquatorCrossingDateTime
C     character(len=512) tm3  ! ObservationEndDateTime
      integer(4) num          ! NumberOfScans
      integer(4) ovr          ! OverlapScans
      parameter(ovr=0)
C array buffer
      real(4) sca ! scale
      real(8) r8d1(LMT) ! array buffer for real(8) D1
C array data
      type(AM2_COMMON_SCANTIME) st(LMT) ! scantime
      real(4) lat(AM2_DEF_SNUM_LO,LMT)  ! lat
      real(4) lon(AM2_DEF_SNUM_LO,LMT)  ! lon
      real(4) geo1(AM2_DEF_SNUM_LO,LMT) ! geophysical data layer 1
      real(4) geo2(AM2_DEF_SNUM_LO,LMT) ! geophysical data layer 2
      real(4) geo3(AM2_DEF_SNUM_LO,LMT) ! geophysical data layer 3
      real(4) geotmp(AM2_DEF_SNUM_LO,LMT*3) ! geophysical data temporary
      integer(4) pdq1(AM2_DEF_SNUM_LO,LMT) ! pixel data quality layer 1
      integer(4) pdq2(AM2_DEF_SNUM_LO,LMT) ! pixel data quality layer 2
      integer(4) pdq3(AM2_DEF_SNUM_LO,LMT) ! pixel data quality layer 3
      integer(4) pdqtmp(AM2_DEF_SNUM_LO,LMT*3) ! pixel data quality temporary
C argument check
      if(iargc().lt.1)then
        write(*,'(a)')'argument: '//MES
        call exit(1)
      endif
C set filename
      call getarg(1,fn)
      write(*,'(a,a)')'input file: ',fn(1:len_trim(fn))
C hdf5 initialize
      call H5open_f(ret)
      if(ret.lt.0)then
        write(*,'(a,i12)')'h5open_f error: ',ret
        call exit(1)
      endif
C open
      call H5Fopen_f(fn,H5F_ACC_RDONLY_F,fhnd,ret,H5P_DEFAULT_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Fopen error: ',fn(1:len_trim(fn))
        call exit(1)
      endif
C HDF5 FORTRAN LIBRARY CAN'T RETRIEVE VARIABLE STRING !
C retrieve GranuleID from filename
      i=index(fn,'/',.true.)
      gid=fn(i+1:i+41)
      write(*,'(a,a)')'GranuleID(RETRIEVE FROM FILENAME): '
     +,gid(1:len_trim(gid))
C check geophysical type by filename
      if(  gid(30:32).ne.'CLW'
     +.and.gid(30:32).ne.'SIC'
     +.and.gid(30:32).ne.'SMC'
     +.and.gid(30:32).ne.'SND'
     +.and.gid(30:32).ne.'SST'
     +.and.gid(30:32).ne.'SSW'
     +.and.gid(30:32).ne.'TPW'
     +.and.gid(30:32).ne.'ASW'
     +.and.gid(30:32).ne.'LST'
     +.and.gid(30:32).ne.'PWL'
     +.and.gid(30:32).ne.'TSI')then
        write(*,'(a)')'wrong input file: this program is for '//MES
        call exit(1)
      endif
C HDF5 FORTRAN LIBRARY CAN'T RETRIEVE VARIABLE STRING !
C calculate NumberOfScans from array size and OverlapScans
      call H5Dopen_f(fhnd,'Scan Time',dhnd,ret)
      call H5Dget_space_f(dhnd,shnd,ret)
      call H5Sget_simple_extent_dims_f(shnd,sz1,sz2,ret)
      if(ret.lt.0)then
        write(*,'(a)')'H5Sget_simple_extent_dims error: Scan Time'
        call exit(1)
      endif
      call H5Sclose_f(shnd,ret)
      call H5Dclose_f(dhnd,ret)
      num=sz1(1)-ovr*2
      write(*,'(a,i12)')'NumberOfScans(RETRIEVE BY ARRAY SIZE): ',num
      write(*,'(a,i12)')'OverlapScans(FIXED VALUE): ',ovr
C check limit
      if(num.gt.LMT)then
        write(*,'(a,i6)')'limit error: limit of NumberOfScans = ',LMT
        call exit(1)
      else
        write(*,'(a,i6)')'limit of NumberOfScans = ',LMT
      endif
C read array: scantime
      ! read
      call H5Dopen_f(fhnd,'Scan Time',dhnd,ret)
      sz1(1)=LMT
      call H5Dread_f(dhnd
     +,H5T_NATIVE_DOUBLE,r8d1,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: ','Scan Time'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
      ! convert
      call amsr2time(num,r8d1,st)
      ! sample display
      write(*,'(a,i4.4,"/",i2.2,"/",i2.2," ",i2.2,":",i2.2,":",i2.2)')
     +'time(scan=1): '
     +,st(1)%year
     +,st(1)%month
     +,st(1)%day
     +,st(1)%hour
     +,st(1)%minute
     +,st(1)%second
C read array: latlon
      ! read lat
      call H5Dopen_f(fhnd,'Latitude of Observation Point',dhnd,ret)
      sz1(1)=LMT
      sz1(2)=AM2_DEF_SNUM_LO
      call H5Dread_f(dhnd
     +,H5T_NATIVE_REAL,lat,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: '
     +  ,'Latitude of Observation Point'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
      ! read lon
      call H5Dopen_f(fhnd,'Longitude of Observation Point',dhnd,ret)
      sz1(1)=LMT
      sz1(2)=AM2_DEF_SNUM_LO
      call H5Dread_f(dhnd
     +,H5T_NATIVE_REAL,lon,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: '
     +  ,'Longitude of Observation Point'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
      ! sample display
      write(*,'(a,"(",f9.4,",",f9.4,")")')'latlon(pixel=1,scan=1): '
     +,lat(1,1),lon(1,1)
C read array: geophysical data for 1 layer
      if((gid(30:32).ne.'SND').and.(gid(30:32).ne.'SST')
     +   .and.(gid(30:32).ne.'TSI'))then
        call H5Dopen_f(fhnd,'Geophysical Data',dhnd,ret)
        ! get scale
        call H5Aopen_f(dhnd,'SCALE FACTOR',ahnd,ret)
        call H5Aread_f(ahnd,H5T_NATIVE_REAL,sca,sz1,ret)
        call H5Aclose_f(ahnd,ret)
        ! read
        sz1(1)=LMT
        sz1(2)=AM2_DEF_SNUM_LO
        call H5Dread_f(dhnd
     +  ,H5T_NATIVE_REAL,geo1,sz1,ret,H5S_ALL_F,H5S_ALL_F)
        if(ret.lt.0)then
          write(*,'(a,a)')'H5Dread error: '
     +    ,'Geophysical Data'
          call exit(1)
        endif
        call H5Dclose_f(dhnd,ret)
        ! change scale
        do j=1,num
          do i=1,AM2_DEF_SNUM_LO
            if(geo1(i,j).gt.-32767)geo1(i,j)=geo1(i,j)*sca
          enddo
        enddo
      endif
C read array: geophysical data for 2 layer
      if((gid(30:32).eq.'SND').or.(gid(30:32).eq.'TSI'))then
        call H5Dopen_f(fhnd,'Geophysical Data',dhnd,ret)
        ! get scale
        call H5Aopen_f(dhnd,'SCALE FACTOR',ahnd,ret)
        call H5Aread_f(ahnd,H5T_NATIVE_REAL,sca,sz1,ret)
        call H5Aclose_f(ahnd,ret)
        ! read
        sz1(1)=LMT*2
        sz1(2)=AM2_DEF_SNUM_LO
        call H5Dread_f(dhnd
     +  ,H5T_NATIVE_REAL,geotmp,sz1,ret,H5S_ALL_F,H5S_ALL_F)
        if(ret.lt.0)then
          write(*,'(a,a)')'H5Dread error: '
     +    ,'Geophysical Data'
          call exit(1)
        endif
        call H5Dclose_f(dhnd,ret)
        ! separate & change scale
        do j=1,num
          do i=1,AM2_DEF_SNUM_LO
            geo1(i,j)=geotmp((i-1)*2+1,(j-1)*2+1)
            geo2(i,j)=geotmp((i-1)*2+2,(j-1)*2+1)
            if(geo1(i,j).gt.-32767)geo1(i,j)=geo1(i,j)*sca
            if(geo2(i,j).gt.-32767)geo2(i,j)=geo2(i,j)*sca
          enddo
        enddo
      endif
C read array: geophysical data for 3 layer
      if((gid(30:32).eq.'SST'))then
        call H5Dopen_f(fhnd,'Geophysical Data',dhnd,ret)
        ! get scale
        call H5Aopen_f(dhnd,'SCALE FACTOR',ahnd,ret)
        call H5Aread_f(ahnd,H5T_NATIVE_REAL,sca,sz1,ret)
        call H5Aclose_f(ahnd,ret)
        ! read
        sz1(1)=LMT*3
        sz1(2)=AM2_DEF_SNUM_LO
        call H5Dread_f(dhnd
     +  ,H5T_NATIVE_REAL,geotmp,sz1,ret,H5S_ALL_F,H5S_ALL_F)
        if(ret.lt.0)then
          write(*,'(a,a)')'H5Dread error: '
     +    ,'Geophysical Data'
          call exit(1)
        endif
        call H5Dclose_f(dhnd,ret)
        ! separate & change scale
        do j=1,num
          do i=1,AM2_DEF_SNUM_LO
            geo1(i,j)=geotmp((i-1)*3+1,(j-1)*3+1)
            geo2(i,j)=geotmp((i-1)*3+2,(j-1)*3+1)
            geo3(i,j)=geotmp((i-1)*3+3,(j-1)*3+1)
            if(geo1(i,j).gt.-32767)geo1(i,j)=geo1(i,j)*sca
            if(geo2(i,j).gt.-32767)geo2(i,j)=geo2(i,j)*sca
            if(geo3(i,j).gt.-32767)geo3(i,j)=geo3(i,j)*sca
          enddo
        enddo
      endif
C read array: pixel data quality for 1 layer
      if((gid(30:32).ne.'SND').and.(gid(30:32).ne.'SST')
     +    .and.(gid(30:32).ne.'TSI'))then
        call H5Dopen_f(fhnd,'Pixel Data Quality',dhnd,ret)
        sz1(1)=LMT
        sz1(2)=AM2_DEF_SNUM_LO
        call H5Dread_f(dhnd
     +  ,H5T_NATIVE_INTEGER,pdq1,sz1,ret,H5S_ALL_F,H5S_ALL_F)
        if(ret.lt.0)then
          write(*,'(a,a)')'H5Dread error: '
     +    ,'Pixel Data Quality'
          call exit(1)
        endif
        call H5Dclose_f(dhnd,ret)
      endif
C read array: pixel data quality for 2 layer
      if((gid(30:32).eq.'SND').or.(gid(30:32).eq.'TSI'))then
        ! read
        call H5Dopen_f(fhnd,'Pixel Data Quality',dhnd,ret)
        sz1(1)=LMT*2
        sz1(2)=AM2_DEF_SNUM_LO
        call H5Dread_f(dhnd
     +  ,H5T_NATIVE_INTEGER,pdqtmp,sz1,ret,H5S_ALL_F,H5S_ALL_F)
        if(ret.lt.0)then
          write(*,'(a,a)')'H5Dread error: '
     +    ,'Pixel Data Quality'
          call exit(1)
        endif
        call H5Dclose_f(dhnd,ret)
        ! separate
        do j=1,num
          do i=1,AM2_DEF_SNUM_LO
            pdq1(i,j)=pdqtmp(i,num*0+j)
            pdq2(i,j)=pdqtmp(i,num*1+j)
          enddo
        enddo
      endif
C read array: pixel data quality for 3 layer
      if((gid(30:32).eq.'SST'))then
        ! read
        call H5Dopen_f(fhnd,'Pixel Data Quality',dhnd,ret)
        sz1(1)=LMT*3
        sz1(2)=AM2_DEF_SNUM_LO
        call H5Dread_f(dhnd
     +  ,H5T_NATIVE_INTEGER,pdqtmp,sz1,ret,H5S_ALL_F,H5S_ALL_F)
        if(ret.lt.0)then
          write(*,'(a,a)')'H5Dread error: '
     +    ,'Pixel Data Quality'
          call exit(1)
        endif
        call H5Dclose_f(dhnd,ret)
        ! separate
        do j=1,num
          do i=1,AM2_DEF_SNUM_LO
            pdq1(i,j)=pdqtmp(i,num*0+j)
            pdq2(i,j)=pdqtmp(i,num*1+j)
            pdq2(i,j)=pdqtmp(i,num*3+j)
          enddo
        enddo
      endif

C display sample
      if(gid(30:32).eq.'CLW')then
        write(*,'(a,f10.3,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [Kg/m2] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'SIC')then
        write(*,'(a,f8.1,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [%] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'SMC')then
        write(*,'(a,f8.1,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [%] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'SND')then
        write(*,'(a,f8.1,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [cm] (PDQ:',pdq1(1,1),')'
        write(*,'(a,f8.1,a,i3,a)')'geo2(pixel=1,scan=1): '
     +  ,geo2(1,1),' [cm] (PDQ:',pdq2(1,1),')'
      endif
      if(gid(30:32).eq.'SST')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [C] (PDQ:',pdq1(1,1),')'
        write(*,'(a,f9.2,a,i3,a)')'geo2(pixel=1,scan=1): '
     +  ,geo2(1,1),' [C] (PDQ:',pdq2(1,1),')'
        write(*,'(a,f9.2,a,i3,a)')'geo3(pixel=1,scan=1): '
     +  ,geo3(1,1),' [C] (PDQ:',pdq3(1,1),')'
      endif
      if(gid(30:32).eq.'SSW')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [m/s] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'TPW')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [Kg/m2] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'ASW')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [m/s] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'LST')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [C] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'PWL')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [C] (PDQ:',pdq1(1,1),')'
      endif
      if(gid(30:32).eq.'TSI')then
        write(*,'(a,f9.2,a,i3,a)')'geo1(pixel=1,scan=1): '
     +  ,geo1(1,1),' [C] (PDQ:',pdq1(1,1),')'
        write(*,'(a,f9.2,a,i3,a)')'geo2(pixel=1,scan=1): '
     +  ,geo2(1,1),' [C] (PDQ:',pdq2(1,1),')'
      endif
C close
      call H5Fclose_f(fhnd,ret)
      call H5close_f(ret)
      end
