#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "AMTK.h"

// fixed value
#define MES "AMSR2 L3 BRIGHTNESS TEMPERATURE DATA FILE (HDF5)"

int main(int argc, char *argv[]){
  // interface variable
  int i,j;       // loop variable
  int ret;       // return status
  char buf[512]; // text buffer
  void *vpnt;    // pointer to void
  char *fn;      // filename
  hid_t hnd;     // file handle
  int siz[3];    // array size
  int x;         // grid size x
  int y;         // grid size y

  // variable for sample display
  float v; // grid value
  char c;  // grid character for display
  int ex;  // grid extract count for x
  int ey;  // grid extract count for y

  // meta data
  char geo[512];  // GeophysicalName
  char gid[512];  // GranuleID

  // array data
  float *tbH; // brightness temperature for horizontal
  float *tbV; // brightness temperature for vertical

  // argument check
  if(argc < 2){
    printf("argument: %s\n", MES);
    exit(1);
  }

  // set filename
  fn = argv[1];
  printf("input file: %s\n", fn);

  // open
  hnd=AMTK_openH5(fn);
  if(hnd<0){
    printf("AMTK_openH5 error: %s\n", fn);
    printf("amtk status: %d\n", hnd);
    exit(1);
  }

  // read meta: GeophysicalName
  vpnt=geo;
  ret=AMTK_getMetaDataName(hnd,"GeophysicalName",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: GeophysicalName\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("GeophysicalName: %s\n",geo);

  // read meta: GranuleID
  vpnt=gid;
  ret=AMTK_getMetaDataName(hnd,"GranuleID",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: GranuleID\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("GranuleID: %s\n",gid);

  // check geophysical type
  if(strncmp(gid+29,"T06",3)!=0
  && strncmp(gid+29,"T07",3)!=0
  && strncmp(gid+29,"T10",3)!=0
  && strncmp(gid+29,"T18",3)!=0
  && strncmp(gid+29,"T23",3)!=0
  && strncmp(gid+29,"T36",3)!=0
  && strncmp(gid+29,"T89",3)!=0
  ){
    printf("wrong input file: this program is for %s\n", MES);
    exit(1);
  }

  // get grid size
  ret=AMTK_getDimSize(hnd,AM2_GRID_TBH,siz);
  if(ret<0){
    printf("AMTK_getDimSize error: AM2_GRID_TBH\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  x=siz[1];
  y=siz[0];
  printf("grid size x: %d\n", x);
  printf("grid size y: %d\n", y);

  // memory allocate
  tbH=malloc(sizeof(float)*x*y);
  if(tbH==NULL){
    printf("memory allocate error: tbH\n");
    exit(1);
  }
  tbV=malloc(sizeof(float)*x*y);
  if(tbV==NULL){
    printf("memory allocate error: tbV\n");
    exit(1);
  }

  // read horizontal
  vpnt=tbH;
  ret=AMTK_get_GridFloat(hnd,(float **)&vpnt,AM2_GRID_TBH);
  if(ret<0){
    printf("AMTK_get_GridFloat error: AM2_GRID_TBH\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }

  // read vertical
  vpnt=tbV;
  ret=AMTK_get_GridFloat(hnd,(float **)&vpnt,AM2_GRID_TBV);
  if(ret<0){
    printf("AMTK_get_GridFloat error: AM2_GRID_TBV\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }

  //// display sample

  // set extract count
  if(strncmp(gid+20,"EQ",2)==0){
    if(gid[32]=='L'){
      ex=20;
      ey=40;
    }
    else{
      ex=50;
      ey=100;
    }
  }
  else{
    if(gid[32]=='L'){
      ex=5;
      ey=10;
    }
    else{
      ex=10;
      ey=20;
    }
  }

  // horizontal
  printf("\n");
  printf("ASCII ART OF HORIZONTAL BRIGHTNESS TEMPERATURE (X/%dGRID Y/%dGRID)\n",ex,ey);
  putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
  for(j=0;j<y;j=j+ey){
    putchar('|');
    for(i=0;i<x;i=i+ex){
      v=tbH[x*j+i];
           if(v==65535)      c='#';
      else if(v==65534)      c=' ';
      else if( 50<=v&&v<100) c='1';
      else if(100<=v&&v<150) c='2';
      else if(150<=v&&v<200) c='3';
      else if(200<=v&&v<250) c='4';
      else if(250<=v&&v<300) c='5';
      else if(300<=v&&v<350) c='6';
      else                   c='*';
      putchar(c);
    }
    putchar('|');
    putchar('\n');
  }
  putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');

  // vertical
  printf("\n");
  printf("ASCII ART OF VERTICAL BRIGHTNESS TEMPERATURE (X/%dGRID Y/%dGRID)\n",ex,ey);
  putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
  for(j=0;j<y;j=j+ey){
    putchar('|');
    for(i=0;i<x;i=i+ex){
      v=tbV[x*j+i];
           if(v==65535)      c='#';
      else if(v==65534)      c=' ';
      else if( 50<=v&&v<100) c='1';
      else if(100<=v&&v<150) c='2';
      else if(150<=v&&v<200) c='3';
      else if(200<=v&&v<250) c='4';
      else if(250<=v&&v<300) c='5';
      else if(300<=v&&v<350) c='6';
      else                   c='*';
      putchar(c);
    }
    putchar('|');
    putchar('\n');
  }
  putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');

  // legend
  printf("\n");
  printf("[#]:missing\n");
  printf("[ ]:out of observation\n");
  printf("[1]: 50-100K\n");
  printf("[2]:100-150K\n");
  printf("[3]:150-200K\n");
  printf("[4]:200-250K\n");
  printf("[5]:250-300K\n");
  printf("[6]:300-350K\n");
  printf("[*]:other\n");

  // memory free
  free(tbH);
  free(tbV);

  // close
  ret=AMTK_closeH5(hnd);
}
