      program main
      implicit none
C include
      include 'AMTK_f.h'
C intrinsic function
      integer(4) iargc ! pgf need explicitly declaration
C fixed value
      character(len=*),parameter::MES=
     +'AMSR2 L3 BRIGHTNESS TEMPERATURE DATA FILE (HDF5)'
C interface variable
      integer(4) i,j         ! loop variable
      integer(4) ret         ! return status
      character(len=512) buf ! text buffer
      character(len=512) fn  ! filename
      integer(4) hnd         ! file handle
      integer(4) siz(3)      ! array size
      integer(4) x           ! grid size x
      integer(4) y           ! grid size y
C variable for sample display
      real(4) v     ! grid value
      character c   ! grid character for display
      integer(4) ex ! grid extract count for x
      integer(4) ey ! grid extract count for y
C meta data
      character(len=512) geo  ! GeophysicalName
      character(len=512) gid  ! GranuleID
C array data
      real(4),allocatable::tbH(:,:) ! brightness temperature for horizontal
      real(4),allocatable::tbV(:,:) ! brightness temperature for vertical
C argument check
      if(iargc().lt.1)then
        write(*,'(a)')'argument: '//MES
        call exit(1)
      endif
C set filename
      call getarg(1,fn)
      write(*,'(a,a)')'input file: ',fn(1:len_trim(fn))
C open
      hnd=AMTK_openH5(fn)
      if(hnd.lt.0)then
        write(*,'(a,a)')'AMTK_openH5 error: ',fn(1:len_trim(fn))
        write(*,'(a,i12)')'amtk status: ',hnd
        call exit(1)
      endif
C read meta: GeophysicalName
      ret=AMTK_getMetaDataName(hnd,'GeophysicalName',geo)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getMetaDataName error: GeophysicalName'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'GeophysicalName: ',geo(1:len_trim(geo))
C read meta: GranuleID
      ret=AMTK_getMetaDataName(hnd,'GranuleID',gid)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getMetaDataName error: GranuleID'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'GranuleID: ',gid(1:len_trim(gid))
C check geophysical type
      if(  gid(30:32).ne.'T06'
     +.and.gid(30:32).ne.'T07'
     +.and.gid(30:32).ne.'T10'
     +.and.gid(30:32).ne.'T18'
     +.and.gid(30:32).ne.'T23'
     +.and.gid(30:32).ne.'T36'
     +.and.gid(30:32).ne.'T89')then
        write(*,'(a)')'wrong input file: this program is for '//MES
        call exit(1)
      endif
C get grid size
      ret=AMTK_getDimSize(hnd,AM2_GRID_TBH,siz);
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getDimSize error: AM2_GRID_TBH'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      x=siz(2);
      y=siz(1);
      write(*,'(a,i12)')'grid size x: ',x
      write(*,'(a,i12)')'grid size y: ',y
C memory allocate
      allocate(tbH(x,y),stat=ret)
      if(ret.ne.0)then
        write(*,'(a)')'memory allocate error: tbH'
        call exit(1)
      endif
      allocate(tbV(x,y),stat=ret)
      if(ret.ne.0)then
        write(*,'(a)')'memory allocate error: tbV'
        call exit(1)
      endif
C read horizontal
      ret=AMTK_get_GridFloat(hnd,tbH,AM2_GRID_TBH)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_get_GridFloat error: AM2_GRID_TBH'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
C read vertical
      ret=AMTK_get_GridFloat(hnd,tbV,AM2_GRID_TBV)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_get_GridFloat error: AM2_GRID_TBV'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
CC display sample
C set extract count
      if(gid(21:22).eq.'EQ')then
        if(gid(33:33).eq.'L')then
          ex=20
          ey=40
        else
          ex=50
          ey=100
        endif
      else
        if(gid(33:33).eq.'L')then
          ex=5
          ey=10
        else
          ex=10
          ey=20
        endif
      endif
C horizontal
      write(*,'(a)')' '
      write(*,'(a$)')'ASCII ART OF HORIZONTAL BRIGHTNESS TEMPERATURE'
      write(*,'(a,i3,a,i3,a)')' (X/',ex,'GRID Y/',ey,'GRID)'
      write(*,'(a$)')'+'
      do i=1,x,ex;write(*,'(a$)')'-';enddo
      write(*,'(a$)')'+'
      write(*,'(a)')' '
      do j=1,y,ey
        write(*,'(a$)')'|'
        do i=1,x,ex
          v=tbH(i,j);
               if(v.eq.65535)then           ;c='#'
          else if(v.eq.65534)then           ;c=' '
          else if( 50.le.v.and.v.lt.100)then;c='1'
          else if(100.le.v.and.v.lt.150)then;c='2'
          else if(150.le.v.and.v.lt.200)then;c='3'
          else if(200.le.v.and.v.lt.250)then;c='4'
          else if(250.le.v.and.v.lt.300)then;c='5'
          else if(300.le.v.and.v.lt.350)then;c='6'
          else                              ;c='*'
          endif
          write(*,'(a$)')c
        enddo
        write(*,'(a$)')'|'
        write(*,'(a)')' '
      enddo
      write(*,'(a$)')'+'
      do i=1,x,ex;write(*,'(a$)')'-';enddo
      write(*,'(a$)')'+'
      write(*,'(a)')' '
C vertical
      write(*,'(a)')' '
      write(*,'(a$)')'ASCII ART OF VERTICAL BRIGHTNESS TEMPERATURE'
      write(*,'(a,i3,a,i3,a)')' (X/',ex,'GRID Y/',ey,'GRID)'
      write(*,'(a$)')'+'
      do i=1,x,ex;write(*,'(a$)')'-';enddo
      write(*,'(a$)')'+'
      write(*,'(a)')' '
      do j=1,y,ey
        write(*,'(a$)')'|'
        do i=1,x,ex
          v=tbV(i,j);
               if(v.eq.65535)then           ;c='#'
          else if(v.eq.65534)then           ;c=' '
          else if( 50.le.v.and.v.lt.100)then;c='1'
          else if(100.le.v.and.v.lt.150)then;c='2'
          else if(150.le.v.and.v.lt.200)then;c='3'
          else if(200.le.v.and.v.lt.250)then;c='4'
          else if(250.le.v.and.v.lt.300)then;c='5'
          else if(300.le.v.and.v.lt.350)then;c='6'
          else                              ;c='*'
          endif
          write(*,'(a$)')c
        enddo
        write(*,'(a$)')'|'
        write(*,'(a)')' '
      enddo
      write(*,'(a$)')'+'
      do i=1,x,ex;write(*,'(a$)')'-';enddo
      write(*,'(a$)')'+'
      write(*,'(a)')' '
C legend
      write(*,'(a)')' '
      write(*,'(a)')'[#]:missing'
      write(*,'(a)')'[ ]:out of observation'
      write(*,'(a)')'[1]: 50-100K'
      write(*,'(a)')'[2]:100-150K'
      write(*,'(a)')'[3]:150-200K'
      write(*,'(a)')'[4]:200-250K'
      write(*,'(a)')'[5]:250-300K'
      write(*,'(a)')'[6]:300-350K'
      write(*,'(a)')'[*]:other'
C memory free
      deallocate(tbH)
      deallocate(tbV)
C close
      ret=AMTK_closeH5(hnd)
      end
