#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "hdf5.h"

// fixed value
#define MES "AMSR2 L3 BRIGHTNESS TEMPERATURE DATA FILE (HDF5)"

int main(int argc, char *argv[]){
  // interface variable
  int i,j;          // loop variable
  herr_t ret;       // return status
  char *buf = NULL; // text buffer
  char *fn  = NULL; // filename
  hid_t fhnd;       // file handle
  hid_t ahnd;       // attribute handle
  hid_t atyp;       // attribute type
  hid_t dhnd;       // dataset handle
  hid_t shnd;       // dataspace handle
  hsize_t sz1[3];   // array size 1
  hsize_t sz2[3];   // array size 2
  int x;            // grid size x
  int y;            // grid size y

  // variable for sample display
  float v; // grid value
  char c;  // grid character for display
  int ex;  // grid extract count for x
  int ey;  // grid extract count for y

  // meta data
  char *geo = NULL; // GeophysicalName
  char *gid = NULL; // GranuleID

  // array buffer
  float sca; // scale

  // array data
  float *tbH; // brightness temperature for horizontal
  float *tbV; // brightness temperature for vertical

  // argument check
  if(argc < 2){
    printf("argument: %s\n", MES);
    exit(1);
  }

  // set filename
  fn = argv[1];
  printf("input file: %s\n", fn);

  // hdf5 initialize
  ret = H5open();
  if(ret < 0){
    printf("h5open error: %d\n" ,ret);
    exit(1);
  }

  // open
  fhnd = H5Fopen(fn, H5F_ACC_RDONLY, H5P_DEFAULT);
  if(fhnd < 0){
    printf("H5Fopen error: %s\n", fn);
    exit(1);
  }

  // read meta: GeophysicalName
  ahnd = H5Aopen(fhnd, "GeophysicalName", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &geo);
  if(ret < 0){
    printf("H5Aread error: GeophysicalName\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("GeophysicalName: %s\n", geo);

  // read meta: GranuleID
  ahnd = H5Aopen(fhnd, "GranuleID", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &gid);
  if(ret < 0){
    printf("H5Aread error: GranuleID\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("GranuleID: %s\n", gid);

  // check geophysical type
  if(strncmp(gid+29,"T06",3)!=0
  && strncmp(gid+29,"T07",3)!=0
  && strncmp(gid+29,"T10",3)!=0
  && strncmp(gid+29,"T18",3)!=0
  && strncmp(gid+29,"T23",3)!=0
  && strncmp(gid+29,"T36",3)!=0
  && strncmp(gid+29,"T89",3)!=0
  ){
    printf("wrong input file: this program is for %s\n", MES);
    exit(1);
  }

  // get grid size
  dhnd = H5Dopen(fhnd ,"Brightness Temperature (H)", H5P_DEFAULT);
  shnd = H5Dget_space(dhnd);
  ret = H5Sget_simple_extent_dims(shnd,sz1,sz2);
  if(ret < 0){
    printf("H5Sget_simple_extent_dims error: Brightness Temperature (H)\n");
    exit(1);
  }
  ret = H5Sclose(shnd);
  ret = H5Dclose(dhnd);
  x=sz1[1];
  y=sz1[0];
  printf("grid size x: %d\n", x);
  printf("grid size y: %d\n", y);

  // memory allocate
  tbH=malloc(sizeof(float)*x*y);
  if(tbH==NULL){
    printf("memory allocate error: tbH\n");
    exit(1);
  }
  tbV=malloc(sizeof(float)*x*y);
  if(tbV==NULL){
    printf("memory allocate error: tbV\n");
    exit(1);
  }

  // read horizontal
  dhnd = H5Dopen(fhnd, "Brightness Temperature (H)", H5P_DEFAULT);
  // get scale
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT);
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);
  ret = H5Aclose(ahnd);
  // read
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tbH);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (H)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // change scale
  for(j = 0; j < y; ++j){
    for(i = 0; i < x; ++i){
      if(tbH[x*j+i] < 65534) tbH[x*j+i] = tbH[x*j+i] * sca;
    }
  }

  // read vertical
  dhnd = H5Dopen(fhnd, "Brightness Temperature (V)", H5P_DEFAULT);
  // get scale
  ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT);
  ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);
  ret = H5Aclose(ahnd);
  // read
  ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, tbV);
  if(ret < 0){
    printf("H5Dread error: Brightness Temperature (V)\n");
    exit(1);
  }
  ret = H5Dclose(dhnd);
  // change scale
  for(j = 0; j < y; ++j){
    for(i = 0; i < x; ++i){
      if(tbV[x*j+i] < 65534) tbV[x*j+i] = tbV[x*j+i] * sca;
    }
  }

  //// display sample

  // set extract count
  if(strncmp(gid+20,"EQ",2)==0){
    if(gid[32]=='L'){
      ex=20;
      ey=40;
    }
    else{
      ex=50;
      ey=100;
    }
  }
  else{
    if(gid[32]=='L'){
      ex=5;
      ey=10;
    }
    else{
      ex=10;
      ey=20;
    }
  }

  // horizontal
  printf("\n");
  printf("ASCII ART OF HORIZONTAL BRIGHTNESS TEMPERATURE (X/%dGRID Y/%dGRID)\n",ex,ey);
  putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
  for(j=0;j<y;j=j+ey){
    putchar('|');
    for(i=0;i<x;i=i+ex){
      v=tbH[x*j+i];
           if(v==65535)      c='#';
      else if(v==65534)      c=' ';
      else if( 50<=v&&v<100) c='1';
      else if(100<=v&&v<150) c='2';
      else if(150<=v&&v<200) c='3';
      else if(200<=v&&v<250) c='4';
      else if(250<=v&&v<300) c='5';
      else if(300<=v&&v<350) c='6';
      else                   c='*';
      putchar(c);
    }
    putchar('|');
    putchar('\n');
  }
  putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');

  // vertical
  printf("\n");
  printf("ASCII ART OF VERTICAL BRIGHTNESS TEMPERATURE (X/%dGRID Y/%dGRID)\n",ex,ey);
  putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
  for(j=0;j<y;j=j+ey){
    putchar('|');
    for(i=0;i<x;i=i+ex){
      v=tbV[x*j+i];
           if(v==65535)      c='#';
      else if(v==65534)      c=' ';
      else if( 50<=v&&v<100) c='1';
      else if(100<=v&&v<150) c='2';
      else if(150<=v&&v<200) c='3';
      else if(200<=v&&v<250) c='4';
      else if(250<=v&&v<300) c='5';
      else if(300<=v&&v<350) c='6';
      else                   c='*';
      putchar(c);
    }
    putchar('|');
    putchar('\n');
  }
  putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');

  // legend
  printf("\n");
  printf("[#]:missing\n");
  printf("[ ]:out of observation\n");
  printf("[1]: 50-100K\n");
  printf("[2]:100-150K\n");
  printf("[3]:150-200K\n");
  printf("[4]:200-250K\n");
  printf("[5]:250-300K\n");
  printf("[6]:300-350K\n");
  printf("[*]:other\n");

  // memory free
  free(tbH);
  free(tbV);

  // close
  ret = H5Fclose(fhnd);
  ret = H5close();
}
