      program main
      use hdf5
      implicit none
C intrinsic function
      integer(4) iargc ! pgf need explicitly declaration
C fixed value
      character(len=*),parameter::MES=
     +'AMSR2 L3 BRIGHTNESS TEMPERATURE DATA FILE (HDF5)'
C interface variable
      integer(4) i,j          ! loop variable
      integer(4) ret          ! return status
      character(len=512) buf  ! text buffer
      character(len=512) fn   ! filename
      integer(HID_T) fhnd     ! file handle
      integer(HID_T) ahnd     ! attribute handle
      integer(HID_T) atyp     ! attribute type
      integer(HID_T) dhnd     ! dataset handle
      integer(HID_T) shnd     ! dataspace handle
      integer(HSIZE_T) sz1(3) ! array size 1
      integer(HSIZE_T) sz2(3) ! array size 2
      integer(4) x            ! grid size x
      integer(4) y            ! grid size y
C variable for sample display
      real(4) v     ! grid value
      character c   ! grid character for display
      integer(4) ex ! grid extract count for x
      integer(4) ey ! grid extract count for y
C meta data
C HDF5 FORTRAN LIBRARY CAN'T RETRIEVE VARIABLE STRING !
C retrieve GranuleID from filename
C     character(len=512) geo  ! GeophysicalName
      character(len=512) gid  ! GranuleID
C array buffer
      real(4) sca ! scale
C array data
      real(4),allocatable::tbH(:,:) ! brightness temperature for horizontal
      real(4),allocatable::tbV(:,:) ! brightness temperature for vertical
C argument check
      if(iargc().lt.1)then
        write(*,'(a)')'argument: '//MES
        call exit(1)
      endif
C set filename
      call getarg(1,fn)
      write(*,'(a,a)')'input file: ',fn(1:len_trim(fn))
C hdf5 initialize
      call H5open_f(ret)
      if(ret.lt.0)then
        write(*,'(a,i12)')'h5open_f error: ',ret
        call exit(1)
      endif
C open
      call H5Fopen_f(fn,H5F_ACC_RDONLY_F,fhnd,ret,H5P_DEFAULT_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Fopen error: ',fn(1:len_trim(fn))
        call exit(1)
      endif
C HDF5 FORTRAN LIBRARY CAN'T RETRIEVE VARIABLE STRING !
C retrieve GranuleID from filename
      i=index(fn,'/',.true.)
      gid=fn(i+1:i+41)
      write(*,'(a,a)')'GranuleID(RETRIEVE FROM FILENAME): '
     +,gid(1:len_trim(gid))
C check geophysical type
      if(  gid(30:32).ne.'T06'
     +.and.gid(30:32).ne.'T07'
     +.and.gid(30:32).ne.'T10'
     +.and.gid(30:32).ne.'T18'
     +.and.gid(30:32).ne.'T23'
     +.and.gid(30:32).ne.'T36'
     +.and.gid(30:32).ne.'T89')then
        write(*,'(a)')'wrong input file: this program is for '//MES
        call exit(1)
      endif
C get grid size
      call H5Dopen_f(fhnd,'Brightness Temperature (H)',dhnd,ret)
      call H5Dget_space_f(dhnd,shnd,ret)
      call H5Sget_simple_extent_dims_f(shnd,sz1,sz2,ret)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Sget_simple_extent_dims error: '
     +  ,'Brightness Temperature (H)'
        call exit(1)
      endif
      call H5Sclose_f(shnd,ret)
      call H5Dclose_f(dhnd,ret)
      x=sz1(1);
      y=sz1(2);
      write(*,'(a,i12)')'grid size x: ',x
      write(*,'(a,i12)')'grid size y: ',y
C memory allocate
      allocate(tbH(x,y),stat=ret)
      if(ret.ne.0)then
        write(*,'(a)')'memory allocate error: tbH'
        call exit(1)
      endif
      allocate(tbV(x,y),stat=ret)
      if(ret.ne.0)then
        write(*,'(a)')'memory allocate error: tbV'
        call exit(1)
      endif
C read horizontal
      call H5Dopen_f(fhnd,'Brightness Temperature (H)',dhnd,ret)
      ! get scale
      call H5Aopen_f(dhnd,'SCALE FACTOR',ahnd,ret)
      call H5Aread_f(ahnd,H5T_NATIVE_REAL,sca,sz1,ret)
      call H5Aclose_f(ahnd,ret)
      ! read
      sz1(1)=y
      sz1(2)=x
      call H5Dread_f(dhnd
     +,H5T_NATIVE_REAL,tbH,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: '
     +  ,'Brightness Temperature (H)'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
      ! change scale
      do j=1,y
        do i=1,x
          if(tbH(i,j).lt.65534)tbH(i,j)=tbH(i,j)*sca
        enddo
      enddo
C read vertical
      call H5Dopen_f(fhnd,'Brightness Temperature (V)',dhnd,ret)
      ! get scale
      call H5Aopen_f(dhnd,'SCALE FACTOR',ahnd,ret)
      call H5Aread_f(ahnd,H5T_NATIVE_REAL,sca,sz1,ret)
      call H5Aclose_f(ahnd,ret)
      ! read
      sz1(1)=y
      sz1(2)=x
      call H5Dread_f(dhnd
     +,H5T_NATIVE_REAL,tbV,sz1,ret,H5S_ALL_F,H5S_ALL_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Dread error: '
     +  ,'Brightness Temperature (V)'
        call exit(1)
      endif
      call H5Dclose_f(dhnd,ret)
      ! change scale
      do j=1,y
        do i=1,x
          if(tbV(i,j).lt.65534)tbV(i,j)=tbV(i,j)*sca
        enddo
      enddo
CC display sample
C set extract count
      if(gid(21:22).eq.'EQ')then
        if(gid(33:33).eq.'L')then
          ex=20
          ey=40
        else
          ex=50
          ey=100
        endif
      else
        if(gid(33:33).eq.'L')then
          ex=5
          ey=10
        else
          ex=10
          ey=20
        endif
      endif
C horizontal
      write(*,'(a)')' '
      write(*,'(a$)')'ASCII ART OF HORIZONTAL BRIGHTNESS TEMPERATURE'
      write(*,'(a,i3,a,i3,a)')' (X/',ex,'GRID Y/',ey,'GRID)'
      write(*,'(a$)')'+'
      do i=1,x,ex;write(*,'(a$)')'-';enddo
      write(*,'(a$)')'+'
      write(*,'(a)')' '
      do j=1,y,ey
        write(*,'(a$)')'|'
        do i=1,x,ex
          v=tbH(i,j);
               if(v.eq.65535)then           ;c='#'
          else if(v.eq.65534)then           ;c=' '
          else if( 50.le.v.and.v.lt.100)then;c='1'
          else if(100.le.v.and.v.lt.150)then;c='2'
          else if(150.le.v.and.v.lt.200)then;c='3'
          else if(200.le.v.and.v.lt.250)then;c='4'
          else if(250.le.v.and.v.lt.300)then;c='5'
          else if(300.le.v.and.v.lt.350)then;c='6'
          else                              ;c='*'
          endif
          write(*,'(a$)')c
        enddo
        write(*,'(a$)')'|'
        write(*,'(a)')' '
      enddo
      write(*,'(a$)')'+'
      do i=1,x,ex;write(*,'(a$)')'-';enddo
      write(*,'(a$)')'+'
      write(*,'(a)')' '
C vertical
      write(*,'(a)')' '
      write(*,'(a$)')'ASCII ART OF VERTICAL BRIGHTNESS TEMPERATURE'
      write(*,'(a,i3,a,i3,a)')' (X/',ex,'GRID Y/',ey,'GRID)'
      write(*,'(a$)')'+'
      do i=1,x,ex;write(*,'(a$)')'-';enddo
      write(*,'(a$)')'+'
      write(*,'(a)')' '
      do j=1,y,ey
        write(*,'(a$)')'|'
        do i=1,x,ex
          v=tbV(i,j);
               if(v.eq.65535)then           ;c='#'
          else if(v.eq.65534)then           ;c=' '
          else if( 50.le.v.and.v.lt.100)then;c='1'
          else if(100.le.v.and.v.lt.150)then;c='2'
          else if(150.le.v.and.v.lt.200)then;c='3'
          else if(200.le.v.and.v.lt.250)then;c='4'
          else if(250.le.v.and.v.lt.300)then;c='5'
          else if(300.le.v.and.v.lt.350)then;c='6'
          else                              ;c='*'
          endif
          write(*,'(a$)')c
        enddo
        write(*,'(a$)')'|'
        write(*,'(a)')' '
      enddo
      write(*,'(a$)')'+'
      do i=1,x,ex;write(*,'(a$)')'-';enddo
      write(*,'(a$)')'+'
      write(*,'(a)')' '
C legend
      write(*,'(a)')' '
      write(*,'(a)')'[#]:missing'
      write(*,'(a)')'[ ]:out of observation'
      write(*,'(a)')'[1]: 50-100K'
      write(*,'(a)')'[2]:100-150K'
      write(*,'(a)')'[3]:150-200K'
      write(*,'(a)')'[4]:200-250K'
      write(*,'(a)')'[5]:250-300K'
      write(*,'(a)')'[6]:300-350K'
      write(*,'(a)')'[*]:other'
C memory free
      deallocate(tbH)
      deallocate(tbV)
C close
      call H5Fclose_f(fhnd,ret)
      call H5close_f(ret)
      end
