/*  This sample program can be used for SST Ver.4 products but may have problem   *
 *  to read SST Ver.3 or older.                                                   *
 *  You can use the program to read Ver.4 or older products of other geophysical  *
 *  parameters (except SST).                                                      */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "AMTK.h"

// fixed value
#define MES "AMSR2 L3 GEOPHYSICAL DATA FILE (HDF5)"

int main(int argc, char *argv[]){
  // interface variable
  int i,j;       // loop variable
  int ret;       // return status
  char buf[512]; // text buffer
  void *vpnt;    // pointer to void
  char *fn;      // filename
  hid_t hnd;     // file handle
  int siz[3];    // array size
  int x;         // grid size x
  int y;         // grid size y

  // variable for sample display
  float r0,r1,r2,r3,r4,r5,r9; // range start & end value
  char *unit; // unit
  float v;    // grid value
  char c;     // grid character for display
  int ex;     // grid extract count for x
  int ey;     // grid extract count for y

  // meta data
  char geo[512];  // GeophysicalName
  char gid[512];  // GranuleID

  // array data
  float *geo1; // geophysical data layer 1
  float *geo2; // geophysical data layer 2
  float *geo3; // geophysical data layer 2

  // argument check
  if(argc < 2){
    printf("argument: %s\n", MES);
    exit(1);
  }

  // set filename
  fn = argv[1];
  printf("input file: %s\n", fn);

  // open
  hnd=AMTK_openH5(fn);
  if(hnd<0){
    printf("AMTK_openH5 error: %s\n", fn);
    printf("amtk status: %d\n", hnd);
    exit(1);
  }

  // read meta: GeophysicalName
  vpnt=geo;
  ret=AMTK_getMetaDataName(hnd,"GeophysicalName",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: GeophysicalName\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("GeophysicalName: %s\n",geo);

  // read meta: GranuleID
  vpnt=gid;
  ret=AMTK_getMetaDataName(hnd,"GranuleID",(char **)&vpnt);
  if(ret<0){
    printf("AMTK_getMetaDataName error: GranuleID\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  printf("GranuleID: %s\n",gid);

  // check geophysical type
  if(strncmp(gid+29,"CLW",3)!=0
  && strncmp(gid+29,"PRC",3)!=0
  && strncmp(gid+29,"SIC",3)!=0
  && strncmp(gid+29,"SMC",3)!=0
  && strncmp(gid+29,"SND",3)!=0
  && strncmp(gid+29,"SST",3)!=0
  && strncmp(gid+29,"SSW",3)!=0
  && strncmp(gid+29,"TPW",3)!=0
  && strncmp(gid+29,"ASW",3)!=0
  && strncmp(gid+29,"LST",3)!=0
  && strncmp(gid+29,"PWL",3)!=0
  && strncmp(gid+29,"TSI",3)!=0
  ){
    printf("wrong input file: this program is for %s\n", MES);
    exit(1);
  }

  // get grid size
  ret=AMTK_getDimSize(hnd,AM2_GRID_GEO1,siz);
  if(ret<0){
    printf("AMTK_getDimSize error: AM2_GRID_GEO1\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }
  x=siz[1];
  y=siz[0];
  printf("grid size x: %d\n", x);
  printf("grid size y: %d\n", y);

  // memory allocate layer 1
  geo1=malloc(sizeof(float)*x*y);
  if(geo1==NULL){
    printf("memory allocate error: geo1\n");
    exit(1);
  }

  // memory allocate layer 2
  if(strncmp(gid+29,"SND",3)==0 || strncmp(gid+29,"SST",3)==0 || strncmp(gid+29,"TSI",3)==0){
    geo2=malloc(sizeof(float)*x*y);
    if(geo2==NULL){
      printf("memory allocate error: geo2\n");
      exit(1);
    }
  }

  // memory allocate layer 3
  if(strncmp(gid+29,"SST",3)==0){
    geo3=malloc(sizeof(float)*x*y);
    if(geo3==NULL){
      printf("memory allocate error: geo3\n");
      exit(1);
    }
  }

  // read layer 1
  vpnt=geo1;
  ret=AMTK_get_GridFloat(hnd,(float **)&vpnt,AM2_GRID_GEO1);
  if(ret<0){
    printf("AMTK_get_GridFloat error: AM2_GRID_GEO1\n");
    printf("amtk status: %d\n",ret);
    exit(1);
  }

  // read layer 2
  if(strncmp(gid+29,"SND",3)==0 || strncmp(gid+29,"SST",3)==0 || strncmp(gid+29,"TSI",3)==0){
    vpnt=geo2;
    ret=AMTK_get_GridFloat(hnd,(float **)&vpnt,AM2_GRID_GEO2);
    if(ret<0){
      printf("AMTK_get_GridFloat error: AM2_GRID_GEO2\n");
      printf("amtk status: %d\n",ret);
      exit(1);
    }
  }

  // read layer 3
  if(strncmp(gid+29,"SST",3)==0){
    vpnt=geo3;
    ret=AMTK_get_GridFloat(hnd,(float **)&vpnt,AM2_GRID_GEO3);
    if(ret<0){
      printf("AMTK_get_GridFloat error: AM2_GRID_GEO3\n");
      printf("amtk status: %d\n",ret);
      exit(1);
    }
  }


  //// display sample

  // set extract count
  if(strncmp(gid+20,"EQ",2)==0){
    if(gid[32]=='L'){
      ex=20;
      ey=40;
    }
    else{
      ex=50;
      ey=100;
    }
  }
  else{
    if(gid[32]=='L'){
      if(strncmp(gid+29,"SIC",3)==0){
        ex=5;
        ey=10;
      }
      else{
        ex=7;
        ey=14;
      }
    }
    else{
      if(strncmp(gid+29,"SIC",3)==0){
        ex=10;
        ey=20;
      }
      else{
        ex=14;
        ey=28;
      }
    }
  }

  // set range value & unit
  if(strncmp(gid+29,"CLW",3)==0){
    r0=0.00F;
    r1=0.03F;
    r2=0.06F;
    r3=0.09F;
    r4=0.12F;
    r5=0.15F;
    r9=0.18F;
    unit="Kg/m2";
  }
  if(strncmp(gid+29,"PRC",3)==0){
    r0=0.0F;
    r1=0.5F;
    r2=1.0F;
    r3=1.5F;
    r4=2.0F;
    r5=2.5F;
    r9=3.0F;
    unit="mm/h";
  }
  if(strncmp(gid+29,"SIC",3)==0 || strncmp(gid+29,"TSI",3)==0){
    r0= 0.0F;
    r1=15.0F;
    r2=30.0F;
    r3=45.0F;
    r4=60.0F;
    r5=75.0F;
    r9=90.0F;
    unit="%";
  }
  if(strncmp(gid+29,"SMC",3)==0){
    r0= 0.0F;
    r1=10.0F;
    r2=20.0F;
    r3=30.0F;
    r4=40.0F;
    r5=50.0F;
    r9=60.0F;
    unit="%";
  }
  if(strncmp(gid+29,"SND",3)==0){
    r0= 0.0F;
    r1=10.0F;
    r2=20.0F;
    r3=30.0F;
    r4=40.0F;
    r5=50.0F;
    r9=60.0F;
    unit="cm";
  }
  if(strncmp(gid+29,"SST",3)==0){
    r0= 0.0F;
    r1= 5.0F;
    r2=10.0F;
    r3=15.0F;
    r4=20.0F;
    r5=25.0F;
    r9=30.0F;
    unit="C";
  }
  if(strncmp(gid+29,"SSW",3)==0){
    r0= 0.0F;
    r1= 3.0F;
    r2= 6.0F;
    r3= 9.0F;
    r4=12.0F;
    r5=15.0F;
    r9=18.0F;
    unit="m/s";
  }
  if(strncmp(gid+29,"TPW",3)==0 || strncmp(gid+29,"PWL",3)==0){
    r0= 0.0F;
    r1=10.0F;
    r2=20.0F;
    r3=30.0F;
    r4=40.0F;
    r5=50.0F;
    r9=60.0F;
    unit="Kg/m2";
  }
  if(strncmp(gid+29,"ASW",3)==0){
    r0= 0.0F;
    r1=10.0F;
    r2=20.0F;
    r3=30.0F;
    r4=40.0F;
    r5=50.0F;
    r9=60.0F;
    unit="m/s";
  }
  if(strncmp(gid+29,"LST",3)==0){
    r0= 0.0F;
    r1= 8.0F;
    r2=16.0F;
    r3=24.0F;
    r4=32.0F;
    r5=40.0F;
    r9=48.0F;
    unit="C";
  }

  // layer 1
  printf("\n");
  printf("ASCII ART OF GEOPHYSICAL DATA LAYER #1 (X/%dGRID Y/%dGRID)\n",ex,ey);
  putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
  for(j=0;j<y;j=j+ey){
    putchar('|');
    for(i=0;i<x;i=i+ex){
      v=geo1[x*j+i];
           if(v==-32768)   c='#';
      else if(v==-32767)   c=' ';
      else if(r0<=v&&v<r1) c='0';
      else if(r1<=v&&v<r2) c='1';
      else if(r2<=v&&v<r3) c='2';
      else if(r3<=v&&v<r4) c='3';
      else if(r4<=v&&v<r5) c='4';
      else if(r5<=v&&v<r9) c='5';
      else                 c='*';
      putchar(c);
    }
    putchar('|');
    putchar('\n');
  }
  putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');

  // layer 2
  if(strncmp(gid+29,"SND",3)==0 || strncmp(gid+29,"SST",3)==0 || strncmp(gid+29,"TSI",3)==0){
    printf("\n");
    printf("ASCII ART OF GEOPHYSICAL DATA LAYER #2 (X/%dGRID Y/%dGRID)\n",ex,ey);
    putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
    for(j=0;j<y;j=j+ey){
      putchar('|');
      for(i=0;i<x;i=i+ex){
        v=geo2[x*j+i];
             if(v==-32768)   c='#';
        else if(v==-32767)   c=' ';
        else if(r0<=v&&v<r1) c='0';
        else if(r1<=v&&v<r2) c='1';
        else if(r2<=v&&v<r3) c='2';
        else if(r3<=v&&v<r4) c='3';
        else if(r4<=v&&v<r5) c='4';
        else if(r5<=v&&v<r9) c='5';
        else                 c='*';
        putchar(c);
      }
      putchar('|');
      putchar('\n');
    }
    putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
  }

  // layer 3
  if(strncmp(gid+29,"SST",3)==0){
    printf("\n");
    printf("ASCII ART OF GEOPHYSICAL DATA LAYER #3 (X/%dGRID Y/%dGRID)\n",ex,ey);
    putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
    for(j=0;j<y;j=j+ey){
      putchar('|');
      for(i=0;i<x;i=i+ex){
        v=geo3[x*j+i];
             if(v==-32768)   c='#';
        else if(v==-32767)   c=' ';
        else if(r0<=v&&v<r1) c='0';
        else if(r1<=v&&v<r2) c='1';
        else if(r2<=v&&v<r3) c='2';
        else if(r3<=v&&v<r4) c='3';
        else if(r4<=v&&v<r5) c='4';
        else if(r5<=v&&v<r9) c='5';
        else                 c='*';
        putchar(c);
      }
      putchar('|');
      putchar('\n');
    }
    putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
  }

  // legend
  if(strncmp(gid+29,"TSI",3)==0){
    printf("\n");
    printf("LAYER #1(TSI)\n");
    printf("[#]:missing\n");
    printf("[ ]:out of observation\n");
    printf("[0]:Not thin ice or Sea water\n");
    printf("[1]:Thin ice\n");
    printf("[*]:other\n");
    printf("\n");
    printf("LAYER #2(SIC)\n");
    printf("[#]:missing\n");
    printf("[ ]:out of observation\n");
    printf("[0]: %6.3f - %6.3f %s\n",r0,r1,unit);
    printf("[1]: %6.3f - %6.3f %s\n",r1,r2,unit);
    printf("[2]: %6.3f - %6.3f %s\n",r2,r3,unit);
    printf("[3]: %6.3f - %6.3f %s\n",r3,r4,unit);
    printf("[4]: %6.3f - %6.3f %s\n",r4,r5,unit);
    printf("[5]: %6.3f - %6.3f %s\n",r5,r9,unit);
    printf("[*]:other\n");
  }
  else{
    printf("\n");
    printf("[#]:missing\n");
    printf("[ ]:out of observation\n");
    printf("[0]: %6.3f - %6.3f %s\n",r0,r1,unit);
    printf("[1]: %6.3f - %6.3f %s\n",r1,r2,unit);
    printf("[2]: %6.3f - %6.3f %s\n",r2,r3,unit);
    printf("[3]: %6.3f - %6.3f %s\n",r3,r4,unit);
    printf("[4]: %6.3f - %6.3f %s\n",r4,r5,unit);
    printf("[5]: %6.3f - %6.3f %s\n",r5,r9,unit);
    printf("[*]:other\n");
  }
  // memory free
  free(geo1);
  if(strncmp(gid+29,"SND",3)==0 || strncmp(gid+29,"SST",3)==0 || strncmp(gid+29,"TSI",3)==0){
    free(geo2);
  }
  if(strncmp(gid+29,"SST",3)==0){
    free(geo3);
  }
  // close
  ret=AMTK_closeH5(hnd);
}
