C   This sample program can be used for SST Ver.4 products but may have
C   problem to read SST Ver.3 or older.
C   You can use the program to read Ver.4 or older products of other
C   geophysical parameters (except SST).

      program main
      implicit none
C include
      include 'AMTK_f.h'
C intrinsic function
      integer(4) iargc ! pgf need explicitly declaration
C fixed value
      character(len=*),parameter::MES=
     +'AMSR2 L3 GEOPHYSICAL DATA FILE (HDF5)'
C interface variable
      integer(4) i,j         ! loop variable
      integer(4) ret         ! return status
      character(len=512) buf ! text buffer
      character(len=512) fn  ! filename
      integer(4) hnd         ! file handle
      integer(4) siz(3)      ! array size
      integer(4) x           ! grid size x
      integer(4) y           ! grid size y
C variable for sample display
      real(4) r0,r1,r2,r3,r4,r5,r9 ! range start & end value
      character(len=10) unit ! unit
      real(4) v              ! grid value
      character c            ! grid character for display
      integer(4) ex          ! grid extract count for x
      integer(4) ey          ! grid extract count for y
C meta data
      character(len=512) geo  ! GeophysicalName
      character(len=512) gid  ! GranuleID
C array data
      real(4),allocatable::geo1(:,:) ! geophysical data layer 1
      real(4),allocatable::geo2(:,:) ! geophysical data layer 2
      real(4),allocatable::geo3(:,:) ! geophysical data layer 3

C argument check
      if(iargc().lt.1)then
        write(*,'(a)')'argument: '//MES
        call exit(1)
      endif
C set filename
      call getarg(1,fn)
      write(*,'(a,a)')'input file: ',fn(1:len_trim(fn))
C open
      hnd=AMTK_openH5(fn)
      if(hnd.lt.0)then
        write(*,'(a,a)')'AMTK_openH5 error: ',fn(1:len_trim(fn))
        write(*,'(a,i12)')'amtk status: ',hnd
        call exit(1)
      endif
C read meta: GeophysicalName
      ret=AMTK_getMetaDataName(hnd,'GeophysicalName',geo)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getMetaDataName error: GeophysicalName'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'GeophysicalName: ',geo(1:len_trim(geo))
C read meta: GranuleID
      ret=AMTK_getMetaDataName(hnd,'GranuleID',gid)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getMetaDataName error: GranuleID'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      write(*,'(a,a)')'GranuleID: ',gid(1:len_trim(gid))
C check geophysical type
      if(  gid(30:32).ne.'CLW'
     +.and.gid(30:32).ne.'PRC'
     +.and.gid(30:32).ne.'SIC'
     +.and.gid(30:32).ne.'SMC'
     +.and.gid(30:32).ne.'SND'
     +.and.gid(30:32).ne.'SST'
     +.and.gid(30:32).ne.'SSW'
     +.and.gid(30:32).ne.'TPW'
     +.and.gid(30:32).ne.'ASW'
     +.and.gid(30:32).ne.'LST'
     +.and.gid(30:32).ne.'PWL'
     +.and.gid(30:32).ne.'TSI')then
        write(*,'(a)')'wrong input file: this program is for '//MES
        call exit(1)
      endif
C get grid size
      ret=AMTK_getDimSize(hnd,AM2_GRID_GEO1,siz);
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_getDimSize error: AM2_GRID_GEO1'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
      x=siz(2);
      y=siz(1);
      write(*,'(a,i12)')'grid size x: ',x
      write(*,'(a,i12)')'grid size y: ',y
C memory allocate layer 1
      allocate(geo1(x,y),stat=ret)
      if(ret.ne.0)then
        write(*,'(a)')'memory allocate error: geo1'
        call exit(1)
      endif
C memory allocate layer 2
      if((gid(30:32).eq.'SND').or.(gid(30:32).eq.'SST')
     +    .or.(gid(30:32).eq.'TSI'))then
        allocate(geo2(x,y),stat=ret)
        if(ret.ne.0)then
          write(*,'(a)')'memory allocate error: geo2'
          call exit(1)
        endif
      endif
C memory allocate layer 3
      if((gid(30:32).eq.'SST'))then
        allocate(geo3(x,y),stat=ret)
        if(ret.ne.0)then
          write(*,'(a)')'memory allocate error: geo3'
          call exit(1)
        endif
      endif
C read layer 1
      ret=AMTK_get_GridFloat(hnd,geo1,AM2_GRID_GEO1)
      if(ret.lt.0)then
        write(*,'(a)')'AMTK_get_GridFloat error: AM2_GRID_GEO1'
        write(*,'(a,i12)')'amtk status: ',ret
        call exit(1)
      endif
C read layer 2
      if((gid(30:32).eq.'SND').or.(gid(30:32).eq.'SST')
     +    .or.(gid(30:32).eq.'TSI'))then
        ret=AMTK_get_GridFloat(hnd,geo2,AM2_GRID_GEO2)
        if(ret.lt.0)then
          write(*,'(a)')'AMTK_get_GridFloat error: AM2_GRID_GEO2'
          write(*,'(a,i12)')'amtk status: ',ret
          call exit(1)
        endif
      endif
C read layer 3
      if((gid(30:32).eq.'SST'))then
        ret=AMTK_get_GridFloat(hnd,geo3,AM2_GRID_GEO3)
        if(ret.lt.0)then
          write(*,'(a)')'AMTK_get_GridFloat error: AM2_GRID_GEO3'
          write(*,'(a,i12)')'amtk status: ',ret
          call exit(1)
        endif
      endif
CC display sample
C set extract count
      if(gid(21:22).eq.'EQ')then
        if(gid(33:33).eq.'L')then
          ex=20
          ey=40
        else
          ex=50
          ey=100
        endif
      else
        if(gid(33:33).eq.'L')then
          if(gid(30:32).eq.'SIC')then
            ex=5
            ey=10
          else
            ex=7
            ey=14
          endif
        else
          if(gid(30:32).eq.'SIC')then
            ex=10
            ey=20
          else
            ex=14
            ey=28
          endif
        endif
      endif
C set range value & unit
      if(gid(30:32).eq.'CLW')then
        r0=0.00
        r1=0.03
        r2=0.06
        r3=0.09
        r4=0.12
        r5=0.15
        r9=0.18
        unit='Kg/m2'
      endif
      if(gid(30:32).eq.'PRC')then
        r0=0.0
        r1=0.5
        r2=1.0
        r3=1.5
        r4=2.0
        r5=2.5
        r9=3.0
        unit='mm/h'
      endif
      if(gid(30:32).eq.'SIC'.or.gid(30:32).eq.'TSI')then
        r0= 0.0
        r1=15.0
        r2=30.0
        r3=45.0
        r4=60.0
        r5=75.0
        r9=90.0
        unit='%'
      endif
      if(gid(30:32).eq.'SMC')then
        r0= 0.0
        r1=10.0
        r2=20.0
        r3=30.0
        r4=40.0
        r5=50.0
        r9=60.0
        unit='%'
      endif
      if(gid(30:32).eq.'SND')then
        r0= 0.0
        r1=10.0
        r2=20.0
        r3=30.0
        r4=40.0
        r5=50.0
        r9=60.0
        unit='cm'
      endif
      if(gid(30:32).eq.'SST')then
        r0= 0.0
        r1= 5.0
        r2=10.0
        r3=15.0
        r4=20.0
        r5=25.0
        r9=30.0
        unit='C'
      endif
      if(gid(30:32).eq.'SSW')then
        r0= 0.0
        r1= 3.0
        r2= 6.0
        r3= 9.0
        r4=12.0
        r5=15.0
        r9=18.0
        unit='m/s'
      endif
      if(gid(30:32).eq.'TPW'.or.gid(30:32).eq.'PWL')then
        r0= 0.0
        r1=10.0
        r2=20.0
        r3=30.0
        r4=40.0
        r5=50.0
        r9=60.0
        unit='Kg/m2'
      endif
      if(gid(30:32).eq.'ASW')then
        r0= 0.0
        r1=10.0
        r2=20.0
        r3=30.0
        r4=40.0
        r5=50.0
        r9=60.0
        unit='m/s'
      endif
      if(gid(30:32).eq.'LST')then
        r0= 0.0
        r1= 8.0
        r2=16.0
        r3=24.0
        r4=32.0
        r5=40.0
        r9=48.0
        unit='C'
      endif
C layer 1
      write(*,'(a)')' '
      write(*,'(a$)')'ASCII ART OF GEOPHYSICAL DATA LAYER #1'
      write(*,'(a,i3,a,i3,a)')' (X/',ex,'GRID Y/',ey,'GRID)'
      write(*,'(a$)')'+'
      do i=1,x,ex;write(*,'(a$)')'-';enddo
      write(*,'(a$)')'+'
      write(*,'(a)')' '
      do j=1,y,ey
        write(*,'(a$)')'|'
        do i=1,x,ex
          v=geo1(i,j);
               if(v.eq.-32768)then        ;c='#'
          else if(v.eq.-32767)then        ;c=' '
          else if(r0.le.v.and.v.lt.r1)then;c='0'
          else if(r1.le.v.and.v.lt.r2)then;c='1'
          else if(r2.le.v.and.v.lt.r3)then;c='2'
          else if(r3.le.v.and.v.lt.r4)then;c='3'
          else if(r4.le.v.and.v.lt.r5)then;c='4'
          else if(r5.le.v.and.v.lt.r9)then;c='5'
          else                            ;c='*'
          endif
          write(*,'(a$)')c
        enddo
        write(*,'(a$)')'|'
        write(*,'(a)')' '
      enddo
      write(*,'(a$)')'+'
      do i=1,x,ex;write(*,'(a$)')'-';enddo
      write(*,'(a$)')'+'
      write(*,'(a)')' '
C layer 2
      if((gid(30:32).eq.'SND').or.(gid(30:32).eq.'SST')
     +     .or.(gid(30:32).eq.'TSI'))then
        write(*,'(a)')' '
        write(*,'(a$)')'ASCII ART OF GEOPHYSICAL DATA LAYER #2'
        write(*,'(a,i3,a,i3,a)')' (X/',ex,'GRID Y/',ey,'GRID)'
        write(*,'(a$)')'+'
        do i=1,x,ex;write(*,'(a$)')'-';enddo
        write(*,'(a$)')'+'
        write(*,'(a)')' '
        do j=1,y,ey
          write(*,'(a$)')'|'
          do i=1,x,ex
            v=geo2(i,j);
                 if(v.eq.-32768)then        ;c='#'
            else if(v.eq.-32767)then        ;c=' '
            else if(r0.le.v.and.v.lt.r1)then;c='0'
            else if(r1.le.v.and.v.lt.r2)then;c='1'
            else if(r2.le.v.and.v.lt.r3)then;c='2'
            else if(r3.le.v.and.v.lt.r4)then;c='3'
            else if(r4.le.v.and.v.lt.r5)then;c='4'
            else if(r5.le.v.and.v.lt.r9)then;c='5'
            else                            ;c='*'
            endif
            write(*,'(a$)')c
          enddo
          write(*,'(a$)')'|'
          write(*,'(a)')' '
        enddo
        write(*,'(a$)')'+'
        do i=1,x,ex;write(*,'(a$)')'-';enddo
        write(*,'(a$)')'+'
        write(*,'(a)')' '
      endif
C layer 3
      if((gid(30:32).eq.'SST'))then
        write(*,'(a)')' '
        write(*,'(a$)')'ASCII ART OF GEOPHYSICAL DATA LAYER #3'
        write(*,'(a,i3,a,i3,a)')' (X/',ex,'GRID Y/',ey,'GRID)'
        write(*,'(a$)')'+'
        do i=1,x,ex;write(*,'(a$)')'-';enddo
        write(*,'(a$)')'+'
        write(*,'(a)')' '
        do j=1,y,ey
          write(*,'(a$)')'|'
          do i=1,x,ex
            v=geo3(i,j);
                 if(v.eq.-32768)then        ;c='#'
            else if(v.eq.-32767)then        ;c=' '
            else if(r0.le.v.and.v.lt.r1)then;c='0'
            else if(r1.le.v.and.v.lt.r2)then;c='1'
            else if(r2.le.v.and.v.lt.r3)then;c='2'
            else if(r3.le.v.and.v.lt.r4)then;c='3'
            else if(r4.le.v.and.v.lt.r5)then;c='4'
            else if(r5.le.v.and.v.lt.r9)then;c='5'
            else                            ;c='*'
            endif
            write(*,'(a$)')c
          enddo
          write(*,'(a$)')'|'
          write(*,'(a)')' '
        enddo
        write(*,'(a$)')'+'
        do i=1,x,ex;write(*,'(a$)')'-';enddo
        write(*,'(a$)')'+'
        write(*,'(a)')' '
      endif
C legend
      if((gid(30:32).eq.'TSI'))then
        write(*,'(a)')' '
        write(*,'(a)')'LAYER #1(TSI)'
        write(*,'(a)')'[#]:missing'
        write(*,'(a)')'[ ]:out of observation'
        write(*,'(a,f6.3,a,f6.3,a,a)')'[0]:Not thin ice or Sea water'
        write(*,'(a,f6.3,a,f6.3,a,a)')'[1]:Thin ice'
        write(*,'(a)')'[*]:other'
        write(*,'(a)')' '
        write(*,'(a)')'LAYER #2(SIC)'
        write(*,'(a)')'[#]:missing'
        write(*,'(a)')'[ ]:out of observation'
        write(*,'(a,f6.3,a,f6.3,a,a)')'[0]: ',r0,' - ',r1,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[1]: ',r1,' - ',r2,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[2]: ',r2,' - ',r3,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[3]: ',r3,' - ',r4,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[4]: ',r4,' - ',r5,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[5]: ',r5,' - ',r9,' ',unit
        write(*,'(a)')'[*]:other'
      else
        write(*,'(a)')' '
        write(*,'(a)')'[#]:missing'
        write(*,'(a)')'[ ]:out of observation'
        write(*,'(a,f6.3,a,f6.3,a,a)')'[0]: ',r0,' - ',r1,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[1]: ',r1,' - ',r2,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[2]: ',r2,' - ',r3,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[3]: ',r3,' - ',r4,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[4]: ',r4,' - ',r5,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[5]: ',r5,' - ',r9,' ',unit
        write(*,'(a)')'[*]:other'
      endif
C memory free
      deallocate(geo1)
      if((gid(30:32).eq.'SND').or.(gid(30:32).eq.'SST')
     +    .or.(gid(30:32).eq.'TSI'))then
        deallocate(geo2)
      endif
      if((gid(30:32).eq.'SST'))then
        deallocate(geo3)
      endif
C close
      ret=AMTK_closeH5(hnd)
      end
