/*  This sample program can be used for SST Ver.4 products but may have problem   *
 *  to read SST Ver.3 or older.                                                   *
 *  You can use the program to read Ver.4 or older products of other geophysical  *
 *  parameters (except SST).                                                      */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "hdf5.h"

// fixed value
#define MES "AMSR2 L3 GEOPHYSICAL DATA FILE (HDF5)"

int main(int argc, char *argv[]){
  // interface variable
  int i,j;          // loop variable
  herr_t ret;       // return status
  char *buf = NULL; // text buffer
  char *fn  = NULL; // filename
  hid_t fhnd;       // file handle
  hid_t ahnd;       // attribute handle
  hid_t atyp;       // attribute type
  hid_t dhnd;       // dataset handle
  hid_t shnd;       // dataspace handle
  hsize_t sz1[3];   // array size 1
  hsize_t sz2[3];   // array size 2
  int x;            // grid size x
  int y;            // grid size y

  // variable for sample display
  float r0,r1,r2,r3,r4,r5,r9; // range start & end value
  char *unit; // unit
  float v;    // grid value
  char c;     // grid character for display
  int ex;     // grid extract count for x
  int ey;     // grid extract count for y

  // meta data
  char *geo = NULL; // GeophysicalName
  char *gid = NULL; // GranuleID

  // array buffer
  float sca; // scale

  // array data
  float *geo1; // geophysical data layer 1
  float *geo2; // geophysical data layer 2
  float *geo3; // geophysical data layer 3
  float *geotmp; // geophysical data temporary

  // argument check
  if(argc < 2){
    printf("argument: %s\n", MES);
    exit(1);
  }

  // set filename
  fn = argv[1];
  printf("input file: %s\n", fn);

  // hdf5 initialize
  ret = H5open();
  if(ret < 0){
    printf("h5open error: %d\n" ,ret);
    exit(1);
  }

  // open
  fhnd = H5Fopen(fn, H5F_ACC_RDONLY, H5P_DEFAULT);
  if(fhnd < 0){
    printf("H5Fopen error: %s\n", fn);
    exit(1);
  }

  // read meta: GeophysicalName
  ahnd = H5Aopen(fhnd, "GeophysicalName", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &geo);
  if(ret < 0){
    printf("H5Aread error: GeophysicalName\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("GeophysicalName: %s\n", geo);

  // read meta: GranuleID
  ahnd = H5Aopen(fhnd, "GranuleID", H5P_DEFAULT);
  atyp = H5Aget_type(ahnd);
  ret = H5Aread(ahnd, atyp, &gid);
  if(ret < 0){
    printf("H5Aread error: GranuleID\n");
    exit(1);
  }
  ret = H5Aclose(ahnd);
  printf("GranuleID: %s\n", gid);

  // check geophysical type
  if(strncmp(gid+29,"CLW",3)!=0
  && strncmp(gid+29,"PRC",3)!=0
  && strncmp(gid+29,"SIC",3)!=0
  && strncmp(gid+29,"SMC",3)!=0
  && strncmp(gid+29,"SND",3)!=0
  && strncmp(gid+29,"SST",3)!=0
  && strncmp(gid+29,"SSW",3)!=0
  && strncmp(gid+29,"TPW",3)!=0
  && strncmp(gid+29,"ASW",3)!=0
  && strncmp(gid+29,"LST",3)!=0
  && strncmp(gid+29,"PWL",3)!=0
  && strncmp(gid+29,"TSI",3)!=0
  ){
    printf("wrong input file: this program is for %s\n", MES);
    exit(1);
  }

  // get grid size
  dhnd = H5Dopen(fhnd ,"Geophysical Data", H5P_DEFAULT);
  shnd = H5Dget_space(dhnd);
  ret = H5Sget_simple_extent_dims(shnd,sz1,sz2);
  if(ret < 0){
    printf("H5Sget_simple_extent_dims error: Geophysical Data\n");
    exit(1);
  }
  ret = H5Sclose(shnd);
  ret = H5Dclose(dhnd);
  x=sz1[1];
  y=sz1[0];
  printf("grid size x: %d\n", x);
  printf("grid size y: %d\n", y);

  // memory allocate layer 1
  geo1=malloc(sizeof(float)*x*y);
  if(geo1==NULL){
    printf("memory allocate error: geo1\n");
    exit(1);
  }

  // memory allocate layer 2 & tmp
  if(strncmp(gid+29,"SND",3)==0 || strncmp(gid+29,"TSI",3)==0 ){
    geo2=malloc(sizeof(float)*x*y);
    if(geo2==NULL){
      printf("memory allocate error: geo2\n");
      exit(1);
    }
    geotmp=malloc(sizeof(float)*x*y*2);
    if(geotmp==NULL){
      printf("memory allocate error: geotmp\n");
      exit(1);
    }
  }

  // memory allocate layer 3 & tmp
  if(strncmp(gid+29,"SST",3)==0 ){
    geo2=malloc(sizeof(float)*x*y);
    if(geo2==NULL){
      printf("memory allocate error: geo2\n");
      exit(1);
    }
    geo3=malloc(sizeof(float)*x*y);
    if(geo3==NULL){
      printf("memory allocate error: geo3\n");
      exit(1);
    }
    geotmp=malloc(sizeof(float)*x*y*3);
    if(geotmp==NULL){
      printf("memory allocate error: geotmp\n");
      exit(1);
    }
  }


  // read layer 1
  if(strncmp(gid+29,"SND",3)!=0 && strncmp(gid+29,"SST",3)!=0 && strncmp(gid+29,"TSI",3)!=0){
    dhnd = H5Dopen(fhnd, "Geophysical Data", H5P_DEFAULT);
    // get scale
    ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT);
    ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);
    ret = H5Aclose(ahnd);
    // read
    ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, geo1);
    if(ret < 0){
      printf("H5Dread error: Geophysical Data\n");
      exit(1);
    }
    ret = H5Dclose(dhnd);
    // change scale
    for(j = 0; j < y; ++j){
      for(i = 0; i < x; ++i){
        if(geo1[x*j+i] > -32767) geo1[x*j+i] = geo1[x*j+i] * sca;
      }
    }
  }

  // read layer 1 & 2
  if(strncmp(gid+29,"SND",3)==0 || strncmp(gid+29,"TSI",3)==0){
    dhnd = H5Dopen(fhnd, "Geophysical Data", H5P_DEFAULT);
    // get scale
    ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT);
    ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);
    ret = H5Aclose(ahnd);
    // read
    ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, geotmp);
    if(ret < 0){
      printf("H5Dread error: Geophysical Data\n");
      exit(1);
    }
    ret = H5Dclose(dhnd);
    // separate
    for(j = 0; j < y; ++j){
      for(i = 0; i < x; ++i){
        geo1[x*j+i] = geotmp[x*j*2+i*2+0];
        geo2[x*j+i] = geotmp[x*j*2+i*2+1];
      }
    }
    // change scale
    for(j = 0; j < y; ++j){
      for(i = 0; i < x; ++i){
        if(geo1[x*j+i] > -32767) geo1[x*j+i] = geo1[x*j+i] * sca;
        if(geo2[x*j+i] > -32767) geo2[x*j+i] = geo2[x*j+i] * sca;
      }
    }
  }

  // read layer 1 , 2 & 3
  if(strncmp(gid+29,"SST",3)==0 ){
    dhnd = H5Dopen(fhnd, "Geophysical Data", H5P_DEFAULT);
    // get scale
    ahnd = H5Aopen(dhnd, "SCALE FACTOR", H5P_DEFAULT);
    ret = H5Aread(ahnd, H5T_NATIVE_FLOAT, &sca);
    ret = H5Aclose(ahnd);
    // read
    ret = H5Dread(dhnd, H5T_NATIVE_FLOAT, H5S_ALL, H5S_ALL, H5P_DEFAULT, geotmp);
    if(ret < 0){
      printf("H5Dread error: Geophysical Data\n");
      exit(1);
    }
    ret = H5Dclose(dhnd);
    // separate
    for(j = 0; j < y; ++j){
      for(i = 0; i < x; ++i){
        geo1[x*j+i] = geotmp[x*j*3+i*3+0];
        geo2[x*j+i] = geotmp[x*j*3+i*3+1];
        geo3[x*j+i] = geotmp[x*j*3+i*3+2];
      }
    }
    // change scale
    for(j = 0; j < y; ++j){
      for(i = 0; i < x; ++i){
        if(geo1[x*j+i] > -32767) geo1[x*j+i] = geo1[x*j+i] * sca;
        if(geo2[x*j+i] > -32767) geo2[x*j+i] = geo2[x*j+i] * sca;
        if(geo3[x*j+i] > -32767) geo3[x*j+i] = geo3[x*j+i] * sca;
      }
    }
  }

  //// display sample

  // set extract count
  if(strncmp(gid+20,"EQ",2)==0){
    if(gid[32]=='L'){
      ex=20;
      ey=40;
    }
    else{
      ex=50;
      ey=100;
    }
  }
  else{
    if(gid[32]=='L'){
      if(strncmp(gid+29,"SIC",3)==0){
        ex=5;
        ey=10;
      }
      else{
        ex=7;
        ey=14;
      }
    }
    else{
      if(strncmp(gid+29,"SIC",3)==0){
        ex=10;
        ey=20;
      }
      else{
        ex=14;
        ey=28;
      }
    }
  }

  // set range value & unit
  if(strncmp(gid+29,"CLW",3)==0){
    r0=0.00F;
    r1=0.03F;
    r2=0.06F;
    r3=0.09F;
    r4=0.12F;
    r5=0.15F;
    r9=0.18F;
    unit="Kg/m2";
  }
  if(strncmp(gid+29,"PRC",3)==0){
    r0=0.0F;
    r1=0.5F;
    r2=1.0F;
    r3=1.5F;
    r4=2.0F;
    r5=2.5F;
    r9=3.0F;
    unit="mm/h";
  }
  if(strncmp(gid+29,"SIC",3)==0){
    r0= 0.0F;
    r1=15.0F;
    r2=30.0F;
    r3=45.0F;
    r4=60.0F;
    r5=75.0F;
    r9=90.0F;
    unit="%";
  }

  if(strncmp(gid+29,"TSI",3)==0){
    r0= 0.0F;
    r1=15.0F;
    r2=30.0F;
    r3=45.0F;
    r4=60.0F;
    r5=75.0F;
    r9=90.0F;
    unit="%";
  }

  if(strncmp(gid+29,"SMC",3)==0){
    r0= 0.0F;
    r1=10.0F;
    r2=20.0F;
    r3=30.0F;
    r4=40.0F;
    r5=50.0F;
    r9=60.0F;
    unit="%";
  }
  if(strncmp(gid+29,"SND",3)==0){
    r0= 0.0F;
    r1=10.0F;
    r2=20.0F;
    r3=30.0F;
    r4=40.0F;
    r5=50.0F;
    r9=60.0F;
    unit="cm";
  }
  if(strncmp(gid+29,"SST",3)==0){
    r0= 0.0F;
    r1= 5.0F;
    r2=10.0F;
    r3=15.0F;
    r4=20.0F;
    r5=25.0F;
    r9=30.0F;
    unit="C";
  }
  if(strncmp(gid+29,"SSW",3)==0){
    r0= 0.0F;
    r1= 3.0F;
    r2= 6.0F;
    r3= 9.0F;
    r4=12.0F;
    r5=15.0F;
    r9=18.0F;
    unit="m/s";
  }
  if(strncmp(gid+29,"TPW",3)==0 || strncmp(gid+29,"PWL",3)==0){
    r0= 0.0F;
    r1=10.0F;
    r2=20.0F;
    r3=30.0F;
    r4=40.0F;
    r5=50.0F;
    r9=60.0F;
    unit="Kg/m2";
  }
  if(strncmp(gid+29,"ASW",3)==0){
    r0= 0.0F;
    r1=10.0F;
    r2=20.0F;
    r3=30.0F;
    r4=40.0F;
    r5=50.0F;
    r9=60.0F;
    unit="m/s";
  }
  if(strncmp(gid+29,"LST",3)==0){
    r0= 0.0F;
    r1= 8.0F;
    r2=16.0F;
    r3=24.0F;
    r4=32.0F;
    r5=40.0F;
    r9=48.0F;
    unit="C";
  }

  // layer 1
  printf("\n");
  printf("ASCII ART OF GEOPHYSICAL DATA LAYER #1 (X/%dGRID Y/%dGRID)\n",ex,ey);
  putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
  for(j=0;j<y;j=j+ey){
    putchar('|');
    for(i=0;i<x;i=i+ex){
      v=geo1[x*j+i];
           if(v==-32768)   c='#';
      else if(v==-32767)   c=' ';
      else if(r0<=v&&v<r1) c='0';
      else if(r1<=v&&v<r2) c='1';
      else if(r2<=v&&v<r3) c='2';
      else if(r3<=v&&v<r4) c='3';
      else if(r4<=v&&v<r5) c='4';
      else if(r5<=v&&v<r9) c='5';
      else                 c='*';
      putchar(c);
    }
    putchar('|');
    putchar('\n');
  }
  putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');

  // layer 2
  if(strncmp(gid+29,"SND",3)==0 || strncmp(gid+29,"SST",3)==0 || strncmp(gid+29,"TSI",3)==0){
    printf("\n");
    printf("ASCII ART OF GEOPHYSICAL DATA LAYER #2 (X/%dGRID Y/%dGRID)\n",ex,ey);
    putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
    for(j=0;j<y;j=j+ey){
      putchar('|');
      for(i=0;i<x;i=i+ex){
        v=geo2[x*j+i];
             if(v==-32768)   c='#';
        else if(v==-32767)   c=' ';
        else if(r0<=v&&v<r1) c='0';
        else if(r1<=v&&v<r2) c='1';
        else if(r2<=v&&v<r3) c='2';
        else if(r3<=v&&v<r4) c='3';
        else if(r4<=v&&v<r5) c='4';
        else if(r5<=v&&v<r9) c='5';
        else                 c='*';
        putchar(c);
      }
      putchar('|');
      putchar('\n');
    }
    putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
  }

  // layer 3
  if(strncmp(gid+29,"SST",3)==0){
    printf("\n");
    printf("ASCII ART OF GEOPHYSICAL DATA LAYER #3 (X/%dGRID Y/%dGRID)\n",ex,ey);
    putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
    for(j=0;j<y;j=j+ey){
      putchar('|');
      for(i=0;i<x;i=i+ex){
        v=geo3[x*j+i];
             if(v==-32768)   c='#';
        else if(v==-32767)   c=' ';
        else if(r0<=v&&v<r1) c='0';
        else if(r1<=v&&v<r2) c='1';
        else if(r2<=v&&v<r3) c='2';
        else if(r3<=v&&v<r4) c='3';
        else if(r4<=v&&v<r5) c='4';
        else if(r5<=v&&v<r9) c='5';
        else                 c='*';
        putchar(c);
      }
      putchar('|');
      putchar('\n');
    }
    putchar('+');for(i=0;i<x;i=i+ex)putchar('-');putchar('+');putchar('\n');
  }


  // legend

  if(strncmp(gid+29,"TSI",3)==0){
    printf("\n");
    printf("LAYER #1(TSI)\n");
    printf("[#]:missing\n");
    printf("[ ]:out of observation\n");
    printf("[0]:Not thin ice or Sea water\n");
    printf("[1]:Thin ice\n");
    printf("[*]:other\n");
    printf("\n");
    printf("LAYER #2(SIC)\n");
    printf("[#]:missing\n");
    printf("[ ]:out of observation\n");
    printf("[0]: %6.3f - %6.3f %s\n",r0,r1,unit);
    printf("[1]: %6.3f - %6.3f %s\n",r1,r2,unit);
    printf("[2]: %6.3f - %6.3f %s\n",r2,r3,unit);
    printf("[3]: %6.3f - %6.3f %s\n",r3,r4,unit);
    printf("[4]: %6.3f - %6.3f %s\n",r4,r5,unit);
    printf("[5]: %6.3f - %6.3f %s\n",r5,r9,unit);
    printf("[*]:other\n");
  }
  else{
    printf("\n");
    printf("[#]:missing\n");
    printf("[ ]:out of observation\n");
    printf("[0]: %6.3f - %6.3f %s\n",r0,r1,unit);
    printf("[1]: %6.3f - %6.3f %s\n",r1,r2,unit);
    printf("[2]: %6.3f - %6.3f %s\n",r2,r3,unit);
    printf("[3]: %6.3f - %6.3f %s\n",r3,r4,unit);
    printf("[4]: %6.3f - %6.3f %s\n",r4,r5,unit);
    printf("[5]: %6.3f - %6.3f %s\n",r5,r9,unit);
    printf("[*]:other\n");
  }
  // memory free
  free(geo1);
  if(strncmp(gid+29,"SND",3)==0 || strncmp(gid+29,"SST",3)==0 || strncmp(gid+29,"TSI",3)==0){
    free(geo2);
  }
  if(strncmp(gid+29,"SST",3)==0){
    free(geo3);
  } // close
  ret = H5Fclose(fhnd);
  ret = H5close();
}
