C   This sample program can be used for SST Ver.4 products but may have
C   problem to read SST Ver.3 or older.
C   You can use the program to read Ver.4 or older products of other
C   geophysical parameters (except SST).

      program main
      use hdf5
      implicit none
C intrinsic function
      integer(4) iargc ! pgf need explicitly declaration
C fixed value
      character(len=*),parameter::MES=
     +'AMSR2 L3 GEOPHYSICAL DATA FILE (HDF5)'
C interface variable
      integer(4) i,j          ! loop variable
      integer(4) ret          ! return status
      character(len=512) buf  ! text buffer
      character(len=512) fn   ! filename
      integer(HID_T) fhnd     ! file handle
      integer(HID_T) ahnd     ! attribute handle
      integer(HID_T) atyp     ! attribute type
      integer(HID_T) dhnd     ! dataset handle
      integer(HID_T) shnd     ! dataspace handle
      integer(HSIZE_T) sz1(3) ! array size 1
      integer(HSIZE_T) sz2(3) ! array size 2
      integer(4) x            ! grid size x
      integer(4) y            ! grid size y
C variable for sample display
      real(4) r0,r1,r2,r3,r4,r5,r9 ! range start & end value
      character(len=10) unit ! unit
      real(4) v              ! grid value
      character c            ! grid character for display
      integer(4) ex          ! grid extract count for x
      integer(4) ey          ! grid extract count for y
C meta data
C HDF5 FORTRAN LIBRARY CAN'T RETRIEVE VARIABLE STRING !
C retrieve GranuleID from filename
C     character(len=512) geo  ! GeophysicalName
      character(len=512) gid  ! GranuleID
C array buffer
      real(4) sca ! scale
C array data
      real(4),allocatable::geo1(:,:) ! geophysical data layer 1
      real(4),allocatable::geo2(:,:) ! geophysical data layer 2
      real(4),allocatable::geo3(:,:) ! geophysical data layer 3
      real(4),allocatable::geotmp(:,:,:) ! geophysical data temporary
C argument check
      if(iargc().lt.1)then
        write(*,'(a)')'argument: '//MES
        call exit(1)
      endif
C set filename
      call getarg(1,fn)
      write(*,'(a,a)')'input file: ',fn(1:len_trim(fn))
C hdf5 initialize
      call H5open_f(ret)
      if(ret.lt.0)then
        write(*,'(a,i12)')'h5open_f error: ',ret
        call exit(1)
      endif
C open
      call H5Fopen_f(fn,H5F_ACC_RDONLY_F,fhnd,ret,H5P_DEFAULT_F)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Fopen error: ',fn(1:len_trim(fn))
        call exit(1)
      endif
C HDF5 FORTRAN LIBRARY CAN'T RETRIEVE VARIABLE STRING !
C retrieve GranuleID from filename
      i=index(fn,'/',.true.)
      gid=fn(i+1:i+41)
      write(*,'(a,a)')'GranuleID(RETRIEVE FROM FILENAME): '
     +,gid(1:len_trim(gid))
C check geophysical type
      if(  gid(30:32).ne.'CLW'
     +.and.gid(30:32).ne.'PRC'
     +.and.gid(30:32).ne.'SIC'
     +.and.gid(30:32).ne.'SMC'
     +.and.gid(30:32).ne.'SND'
     +.and.gid(30:32).ne.'SST'
     +.and.gid(30:32).ne.'SSW'
     +.and.gid(30:32).ne.'TPW'
     +.and.gid(30:32).ne.'ASW'
     +.and.gid(30:32).ne.'LST'
     +.and.gid(30:32).ne.'PWL'
     +.and.gid(30:32).ne.'TSI')then
        write(*,'(a)')'wrong input file: this program is for '//MES
        call exit(1)
      endif
C get grid size
      call H5Dopen_f(fhnd,'Geophysical Data',dhnd,ret)
      call H5Dget_space_f(dhnd,shnd,ret)
      call H5Sget_simple_extent_dims_f(shnd,sz1,sz2,ret)
      if(ret.lt.0)then
        write(*,'(a,a)')'H5Sget_simple_extent_dims error: '
     +  ,'Geophysical Data'
        call exit(1)
      endif
      call H5Sclose_f(shnd,ret)
      call H5Dclose_f(dhnd,ret)
      x=sz1(2);
      y=sz1(3);
      write(*,'(a,i12)')'grid size x: ',x
      write(*,'(a,i12)')'grid size y: ',y
C memory allocate layer 1
      allocate(geo1(x,y),stat=ret)
      if(ret.ne.0)then
        write(*,'(a)')'memory allocate error: geo1'
        call exit(1)
      endif
C memory allocate layer 2 & tmp
      if((gid(30:32).eq.'SND').or.(gid(30:32).eq.'TSI'))then
        allocate(geo2(x,y),stat=ret)
        if(ret.ne.0)then
          write(*,'(a)')'memory allocate error: geo2'
          call exit(1)
        endif
        allocate(geotmp(2,x,y),stat=ret)
        if(ret.ne.0)then
          write(*,'(a)')'memory allocate error: geotmp'
          call exit(1)
        endif
      endif
C memory allocate layer 3 & tmp
      if((gid(30:32).eq.'SST'))then
        allocate(geo2(x,y),stat=ret)
        if(ret.ne.0)then
          write(*,'(a)')'memory allocate error: geo2'
          call exit(1)
        endif
        allocate(geo3(x,y),stat=ret)
        if(ret.ne.0)then
          write(*,'(a)')'memory allocate error: geo3'
          call exit(1)
        endif
        allocate(geotmp(3,x,y),stat=ret)
        if(ret.ne.0)then
          write(*,'(a)')'memory allocate error: geotmp'
          call exit(1)
        endif
      endif
C read layer 1
      if((gid(30:32).ne.'SND').and.(gid(30:32).ne.'SST')
     +    .and.(gid(30:32).ne.'TSI'))then
        call H5Dopen_f(fhnd,'Geophysical Data',dhnd,ret)
        ! get scale
        call H5Aopen_f(dhnd,'SCALE FACTOR',ahnd,ret)
        call H5Aread_f(ahnd,H5T_NATIVE_REAL,sca,sz1,ret)
        call H5Aclose_f(ahnd,ret)
        ! read
        sz1(1)=y
        sz1(2)=x
        call H5Dread_f(dhnd
     +  ,H5T_NATIVE_REAL,geo1,sz1,ret,H5S_ALL_F,H5S_ALL_F)
        if(ret.lt.0)then
          write(*,'(a,a)')'H5Dread error: '
     +    ,'Geophysical Data'
          call exit(1)
        endif
        call H5Dclose_f(dhnd,ret)
        ! change scale
        do j=1,y
          do i=1,x
            if(geo1(i,j).gt.-32767)geo1(i,j)=geo1(i,j)*sca
          enddo
        enddo
      endif
C read layer 1 & 2
      if((gid(30:32).eq.'SND').or.(gid(30:32).eq.'TSI'))then
        call H5Dopen_f(fhnd,'Geophysical Data',dhnd,ret)
        ! get scale
        call H5Aopen_f(dhnd,'SCALE FACTOR',ahnd,ret)
        call H5Aread_f(ahnd,H5T_NATIVE_REAL,sca,sz1,ret)
        call H5Aclose_f(ahnd,ret)
        ! read
        sz1(1)=y
        sz1(2)=x
        sz1(3)=2
        call H5Dread_f(dhnd
     +  ,H5T_NATIVE_REAL,geotmp,sz1,ret,H5S_ALL_F,H5S_ALL_F)
        if(ret.lt.0)then
          write(*,'(a,a)')'H5Dread error: '
     +    ,'Geophysical Data'
          call exit(1)
        endif
        call H5Dclose_f(dhnd,ret)
        ! separate
        do j=1,y
          do i=1,x
            geo1(i,j)=geotmp(1,i,j)
            geo2(i,j)=geotmp(2,i,j)
          enddo
        enddo
        ! change scale
        do j=1,y
          do i=1,x
            if(geo1(i,j).gt.-32767)geo1(i,j)=geo1(i,j)*sca
            if(geo2(i,j).gt.-32767)geo2(i,j)=geo2(i,j)*sca
          enddo
        enddo
      endif
C read layer 1, 2 & 3
      if((gid(30:32).eq.'SST'))then
        call H5Dopen_f(fhnd,'Geophysical Data',dhnd,ret)
        ! get scale
        call H5Aopen_f(dhnd,'SCALE FACTOR',ahnd,ret)
        call H5Aread_f(ahnd,H5T_NATIVE_REAL,sca,sz1,ret)
        call H5Aclose_f(ahnd,ret)
        ! read
        sz1(1)=y
        sz1(2)=x
        sz1(3)=3
        call H5Dread_f(dhnd
     +  ,H5T_NATIVE_REAL,geotmp,sz1,ret,H5S_ALL_F,H5S_ALL_F)
        if(ret.lt.0)then
          write(*,'(a,a)')'H5Dread error: '
     +    ,'Geophysical Data'
          call exit(1)
        endif
        call H5Dclose_f(dhnd,ret)
        ! separate
        do j=1,y
          do i=1,x
            geo1(i,j)=geotmp(1,i,j)
            geo2(i,j)=geotmp(2,i,j)
            geo3(i,j)=geotmp(3,i,j)
          enddo
        enddo
        ! change scale
        do j=1,y
          do i=1,x
            if(geo1(i,j).gt.-32767)geo1(i,j)=geo1(i,j)*sca
            if(geo2(i,j).gt.-32767)geo2(i,j)=geo2(i,j)*sca
            if(geo3(i,j).gt.-32767)geo3(i,j)=geo3(i,j)*sca
          enddo
        enddo
      endif
CC display sample
C set extract count
      if(gid(21:22).eq.'EQ')then
        if(gid(33:33).eq.'L')then
          ex=20
          ey=40
        else
          ex=50
          ey=100
        endif
      else
        if(gid(33:33).eq.'L')then
          if(gid(30:32).eq.'SIC')then
            ex=5
            ey=10
          else
            ex=7
            ey=14
          endif
        else
          if(gid(30:32).eq.'SIC')then
            ex=10
            ey=20
          else
            ex=14
            ey=28
          endif
        endif
      endif
C set range value & unit
      if(gid(30:32).eq.'CLW')then
        r0=0.00
        r1=0.03
        r2=0.06
        r3=0.09
        r4=0.12
        r5=0.15
        r9=0.18
        unit='Kg/m2'
      endif
      if(gid(30:32).eq.'PRC')then
        r0=0.0
        r1=0.5
        r2=1.0
        r3=1.5
        r4=2.0
        r5=2.5
        r9=3.0
        unit='mm/h'
      endif
      if(gid(30:32).eq.'SIC'.or.gid(30:32).eq.'TSI')then
        r0= 0.0
        r1=15.0
        r2=30.0
        r3=45.0
        r4=60.0
        r5=75.0
        r9=90.0
        unit='%'
      endif
      if(gid(30:32).eq.'SMC')then
        r0= 0.0
        r1=10.0
        r2=20.0
        r3=30.0
        r4=40.0
        r5=50.0
        r9=60.0
        unit='%'
      endif
      if(gid(30:32).eq.'SND')then
        r0= 0.0
        r1=10.0
        r2=20.0
        r3=30.0
        r4=40.0
        r5=50.0
        r9=60.0
        unit='cm'
      endif
      if(gid(30:32).eq.'SST')then
        r0= 0.0
        r1= 5.0
        r2=10.0
        r3=15.0
        r4=20.0
        r5=25.0
        r9=30.0
        unit='C'
      endif
      if(gid(30:32).eq.'SSW')then
        r0= 0.0
        r1= 3.0
        r2= 6.0
        r3= 9.0
        r4=12.0
        r5=15.0
        r9=18.0
        unit='m/s'
      endif
      if(gid(30:32).eq.'TPW'.or.gid(30:32).eq.'PWL')then
        r0= 0.0
        r1=10.0
        r2=20.0
        r3=30.0
        r4=40.0
        r5=50.0
        r9=60.0
        unit='Kg/m2'
      endif
      if(gid(30:32).eq.'ASW')then
        r0= 0.0
        r1=10.0
        r2=20.0
        r3=30.0
        r4=40.0
        r5=50.0
        r9=60.0
        unit='m/s'
      endif
      if(gid(30:32).eq.'LST')then
        r0= 0.0
        r1= 8.0
        r2=16.0
        r3=24.0
        r4=32.0
        r5=40.0
        r9=48.0
        unit='C'
      endif
C layer 1
      write(*,'(a)')' '
      write(*,'(a$)')'ASCII ART OF GEOPHYSICAL DATA LAYER #1'
      write(*,'(a,i3,a,i3,a)')' (X/',ex,'GRID Y/',ey,'GRID)'
      write(*,'(a$)')'+'
      do i=1,x,ex;write(*,'(a$)')'-';enddo
      write(*,'(a$)')'+'
      write(*,'(a)')' '
      do j=1,y,ey
        write(*,'(a$)')'|'
        do i=1,x,ex
          v=geo1(i,j);
               if(v.eq.-32768)then        ;c='#'
          else if(v.eq.-32767)then        ;c=' '
          else if(r0.le.v.and.v.lt.r1)then;c='0'
          else if(r1.le.v.and.v.lt.r2)then;c='1'
          else if(r2.le.v.and.v.lt.r3)then;c='2'
          else if(r3.le.v.and.v.lt.r4)then;c='3'
          else if(r4.le.v.and.v.lt.r5)then;c='4'
          else if(r5.le.v.and.v.lt.r9)then;c='5'
          else                            ;c='*'
          endif
          write(*,'(a$)')c
        enddo
        write(*,'(a$)')'|'
        write(*,'(a)')' '
      enddo
      write(*,'(a$)')'+'
      do i=1,x,ex;write(*,'(a$)')'-';enddo
      write(*,'(a$)')'+'
      write(*,'(a)')' '
C layer 2
      if((gid(30:32).eq.'SND').or.(gid(30:32).eq.'SST')
     +    .or.(gid(30:32).eq.'TSI'))then
        write(*,'(a)')' '
        write(*,'(a$)')'ASCII ART OF GEOPHYSICAL DATA LAYER #2'
        write(*,'(a,i3,a,i3,a)')' (X/',ex,'GRID Y/',ey,'GRID)'
        write(*,'(a$)')'+'
        do i=1,x,ex;write(*,'(a$)')'-';enddo
        write(*,'(a$)')'+'
        write(*,'(a)')' '
        do j=1,y,ey
          write(*,'(a$)')'|'
          do i=1,x,ex
            v=geo2(i,j);
                 if(v.eq.-32768)then        ;c='#'
            else if(v.eq.-32767)then        ;c=' '
            else if(r0.le.v.and.v.lt.r1)then;c='0'
            else if(r1.le.v.and.v.lt.r2)then;c='1'
            else if(r2.le.v.and.v.lt.r3)then;c='2'
            else if(r3.le.v.and.v.lt.r4)then;c='3'
            else if(r4.le.v.and.v.lt.r5)then;c='4'
            else if(r5.le.v.and.v.lt.r9)then;c='5'
            else                            ;c='*'
            endif
            write(*,'(a$)')c
          enddo
          write(*,'(a$)')'|'
          write(*,'(a)')' '
        enddo
        write(*,'(a$)')'+'
        do i=1,x,ex;write(*,'(a$)')'-';enddo
        write(*,'(a$)')'+'
        write(*,'(a)')' '
      endif
C layer 3
      if((gid(30:32).eq.'SST'))then
        write(*,'(a)')' '
        write(*,'(a$)')'ASCII ART OF GEOPHYSICAL DATA LAYER #3'
        write(*,'(a,i3,a,i3,a)')' (X/',ex,'GRID Y/',ey,'GRID)'
        write(*,'(a$)')'+'
        do i=1,x,ex;write(*,'(a$)')'-';enddo
        write(*,'(a$)')'+'
        write(*,'(a)')' '
        do j=1,y,ey
          write(*,'(a$)')'|'
          do i=1,x,ex
            v=geo3(i,j);
                 if(v.eq.-32768)then        ;c='#'
            else if(v.eq.-32767)then        ;c=' '
            else if(r0.le.v.and.v.lt.r1)then;c='0'
            else if(r1.le.v.and.v.lt.r2)then;c='1'
            else if(r2.le.v.and.v.lt.r3)then;c='2'
            else if(r3.le.v.and.v.lt.r4)then;c='3'
            else if(r4.le.v.and.v.lt.r5)then;c='4'
            else if(r5.le.v.and.v.lt.r9)then;c='5'
            else                            ;c='*'
            endif
            write(*,'(a$)')c
          enddo
          write(*,'(a$)')'|'
          write(*,'(a)')' '
        enddo
        write(*,'(a$)')'+'
        do i=1,x,ex;write(*,'(a$)')'-';enddo
        write(*,'(a$)')'+'
        write(*,'(a)')' '
      endif
C legend
      if((gid(30:32).eq.'TSI'))then
        write(*,'(a)')' '
        write(*,'(a)')'LAYER #1(TSI)'
        write(*,'(a)')'[#]:missing'
        write(*,'(a)')'[ ]:out of observation'
        write(*,'(a,f6.3,a,f6.3,a,a)')'[0]:Not thin ice or Sea water'
        write(*,'(a,f6.3,a,f6.3,a,a)')'[1]:Thin ice'
        write(*,'(a)')'[*]:other'
        write(*,'(a)')' '
        write(*,'(a)')'LAYER #2(SIC)'
        write(*,'(a)')'[#]:missing'
        write(*,'(a)')'[ ]:out of observation'
        write(*,'(a,f6.3,a,f6.3,a,a)')'[0]: ',r0,' - ',r1,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[1]: ',r1,' - ',r2,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[2]: ',r2,' - ',r3,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[3]: ',r3,' - ',r4,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[4]: ',r4,' - ',r5,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[5]: ',r5,' - ',r9,' ',unit
        write(*,'(a)')'[*]:other'
      else
        write(*,'(a)')' '
        write(*,'(a)')'[#]:missing'
        write(*,'(a)')'[ ]:out of observation'
        write(*,'(a,f6.3,a,f6.3,a,a)')'[0]: ',r0,' - ',r1,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[1]: ',r1,' - ',r2,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[2]: ',r2,' - ',r3,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[3]: ',r3,' - ',r4,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[4]: ',r4,' - ',r5,' ',unit
        write(*,'(a,f6.3,a,f6.3,a,a)')'[5]: ',r5,' - ',r9,' ',unit
        write(*,'(a)')'[*]:other'
      endif
C memory free
      stop
      deallocate(geo1)
      if((gid(30:32).eq.'SND').or.(gid(30:32).eq.'SST')
     +    .or.(gid(30:32).eq.'TSI'))then
        deallocate(geo2)
      endif
      if((gid(30:32).eq.'SST'))then
        deallocate(geo3)
      endif
C close
      call H5Fclose_f(fhnd,ret)
      call H5close_f(ret)
      end
